@@ -18,23 +18,40 @@
 
 Bazaar can be used with Subversion branches through the bzr-svn plugin.
 
-Most Bazaar commands should work fine with Subversion branches. To 
-create new branches in Subversion using push, it is currently necessary
-to use the svn-push command rather than the standard push command.
+Most Bazaar commands should work fine with Subversion branches. 
 
 bzr-svn also adds four new commands to Bazaar:
 
- - bzr dpush
  - bzr svn-import
- - bzr svn-serve
  - bzr svn-layout
 
 For more information about bzr-svn, see the bzr-svn FAQ.
 
 """
 import bzrlib
-from bzrlib import config
 import bzrlib.api
+
+# versions ending in 'exp' mean experimental mappings
+# versions ending in 'dev' mean development version
+# versions ending in 'final' mean release (well tested, etc)
+version_info = (0, 6, 2, 'final', 0)
+
+if version_info[3] == 'final':
+    version_string = '%d.%d.%d' % version_info[:3]
+else:
+    version_string = '%d.%d.%d%s%d' % version_info
+__version__ = version_string
+
+COMPATIBLE_BZR_VERSIONS = [(1, 15, 0), (1, 16, 0), (1, 17, 0)]
+MINIMUM_SUBVERTPY_VERSION = (0, 6, 1)
+
+bzrlib.api.require_any_api(bzrlib, COMPATIBLE_BZR_VERSIONS)
+
+
+from bzrlib import config
+from bzrlib.branch import (
+    network_format_registry as branch_network_format_registry,
+    )
 from bzrlib.bzrdir import (
     BzrDirFormat,
     format_registry,
@@ -45,12 +62,21 @@ from bzrlib.commands import (
 from bzrlib.errors import (
     DependencyNotPresent,
     )
+from bzrlib.filters import (
+    lazy_register_filter_stack_map,
+    )
 from bzrlib.foreign import (
     foreign_vcs_registry,
     )
 from bzrlib.help_topics import (
     topic_registry,
     )
+from bzrlib.repository import (
+    network_format_registry as repository_network_format_registry,
+    )
+from bzrlib.revisionspec import (
+    revspec_registry,
+    )
 from bzrlib.trace import (
     mutter,
     )
@@ -58,20 +84,10 @@ from bzrlib.transport import (
     register_lazy_transport,
     register_transport_proto,
     )
+from bzrlib.version_info_formats.format_rio import (
+    RioVersionInfoBuilder,
+    )
 
-# versions ending in 'exp' mean experimental mappings
-# versions ending in 'dev' mean development version
-# versions ending in 'final' mean release (well tested, etc)
-version_info = (0, 5, 3, 'final', 0)
-
-if version_info[3] == 'final':
-    version_string = '%d.%d.%d' % version_info[:3]
-else:
-    version_string = '%d.%d.%d%s%d' % version_info
-__version__ = version_string
-
-COMPATIBLE_BZR_VERSIONS = [(1, 13, 0)]
-MINIMUM_SUBVERTPY_VERSION = (0, 6, 1)
 
 
 def check_subversion_version():
@@ -101,8 +117,8 @@ def get_client_string():
 def init_subvertpy():
     try:
         import subvertpy 
-    except ImportError:
-        raise DependencyNotPresent("subvertpy", "bzr-svn: unable to find subvertpy. Please install from http://launchpad.net/subvertpy.")
+    except ImportError, e:
+        raise DependencyNotPresent("subvertpy", "bzr-svn: %s" % e.message)
 
     check_subversion_version()
 
@@ -120,7 +136,6 @@ def lazy_check_versions():
         return
     _versions_checked = True
     init_subvertpy()
-    bzrlib.api.require_any_api(bzrlib, COMPATIBLE_BZR_VERSIONS)
 
 from bzrlib.plugins.svn import format
 
@@ -144,25 +159,51 @@ topic_registry.register_lazy('svn-branching-schemes',
 topic_registry.register_lazy('svn-layout', 
                              'bzrlib.plugins.svn.layout',
                              'help_layout', 'Subversion repository layouts')
-BzrDirFormat.register_control_format(format.SvnRemoteFormat)
+#BzrDirFormat.register_control_server_format(format.SvnRemoteFormat)
+# Register as the first control server format, since the default smart
+# server implementation tries to do a POST request against .bzr/smart and 
+# this causes some Subversion servers to reply with 401 Authentication required
+# even though they are accessible without authentication.
+BzrDirFormat._control_server_formats.insert(0, format.SvnRemoteFormat)
 BzrDirFormat.register_control_format(format.SvnWorkingTreeDirFormat)
-format_registry.register_lazy("subversion", "bzrlib.plugins.svn.format", "SvnRemoteFormat", 
+branch_network_format_registry.register_lazy("subversion",
+        'bzrlib.plugins.svn.branch', 'SvnBranchFormat')
+repository_network_format_registry.register_lazy("subversion",
+        'bzrlib.plugins.svn.repository', 'SvnRepositoryFormat')
+
+format_registry.register_lazy("subversion", "bzrlib.plugins.svn.format", 
+                         "SvnRemoteFormat", 
                          "Subversion repository. ", 
                          native=False)
-format_registry.register_lazy("subversion-wc", "bzrlib.plugins.svn.format", "SvnWorkingTreeDirFormat", 
+format_registry.register_lazy("subversion-wc", "bzrlib.plugins.svn.format", 
+                         "SvnWorkingTreeDirFormat", 
                          "Subversion working copy. ", 
                          native=False, hidden=True)
-from bzrlib.revisionspec import revspec_registry
 revspec_registry.register_lazy("svn:", "bzrlib.plugins.svn.revspec", 
     "RevisionSpec_svn")
 
-config.credential_store_registry.register_lazy(
-    "subversion", "bzrlib.plugins.svn.auth", "SubversionCredentialStore", 
-    help=__doc__)
+try:
+    config.credential_store_registry.register_lazy(
+        "subversion", "bzrlib.plugins.svn.auth", "SubversionCredentialStore", 
+        help=__doc__, fallback=True)
+except TypeError:
+    # Fallback credentials stores were introduced in Bazaar 1.15
+    config.credential_store_registry.register_lazy(
+        "subversion", "bzrlib.plugins.svn.auth", "SubversionCredentialStore", 
+        help=__doc__)
 
 foreign_vcs_registry.register_lazy("svn", "bzrlib.plugins.svn.mapping",
                                    "foreign_vcs_svn")
 
+try:
+    from bzrlib.transport import transport_server_registry
+except ImportError:
+    pass
+else:
+    transport_server_registry.register_lazy('svn', 
+        'bzrlib.plugins.svn.server', 'serve_svn', 
+        "Subversion svn_ra protocol. (default port: 3690)")
+
 _optimizers_registered = False
 def lazy_register_optimizers():
     """Register optimizers for fetching between Subversion and Bazaar 
@@ -180,19 +221,44 @@ def lazy_register_optimizers():
 
 
 plugin_cmds.register_lazy('cmd_svn_import', [], 'bzrlib.plugins.svn.commands')
-plugin_cmds.register_lazy('cmd_foreign_mapping_upgrade', ['svn-upgrade'], 
-                          'bzrlib.plugins.svn.foreign')
-plugin_cmds.register_lazy('cmd_svn_push', [], 'bzrlib.plugins.svn.commands')
 plugin_cmds.register_lazy('cmd_svn_branching_scheme', [], 
                           'bzrlib.plugins.svn.mapping3.commands')
-plugin_cmds.register_lazy('cmd_svn_set_revprops', [], 
-                          'bzrlib.plugins.svn.commands')
 plugin_cmds.register_lazy('cmd_svn_layout', [], 
                           'bzrlib.plugins.svn.commands')
-plugin_cmds.register_lazy('cmd_svn_serve', [], 
-                          'bzrlib.plugins.svn.commands')
 
-plugin_cmds.register_lazy('cmd_dpush', [], 'bzrlib.plugins.svn.foreign')
+lazy_register_filter_stack_map("svn-keywords", 
+        "bzrlib.plugins.svn.keywords", "create_svn_keywords_filter")
+
+
+def info_svn_repository(repository, stats, outf):
+    if "svn-uuid" in stats:
+        outf.write("Subversion UUID: %s\n" % stats["svn-uuid"])
+    if "svn-last-revnum" in stats:
+        outf.write("Subversion Last Revision: %d\n" % stats["svn-last-revnum"])
+
+
+try:
+    from bzrlib.info import (
+        hooks as info_hooks,
+        )
+except ImportError:
+    # Info hooks were introduced in Bazaar 1.15
+    pass
+else:
+    info_hooks.install_named_hook('repository', info_svn_repository, None)
+
+
+def update_stanza(rev, stanza):
+    revmeta = getattr(rev, "svn_meta", None)
+    if revmeta is None:
+        return
+    stanza.add("svn-revno", str(revmeta.revnum))
+    stanza.add("svn-uuid", revmeta.uuid)
+
+
+RioVersionInfoBuilder.hooks.install_named_hook('revision',
+    update_stanza, None)
+
 
 def test_suite():
     """Returns the testsuite for bzr-svn."""
