@@ -13,15 +13,24 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
 """Annotate support."""
 
-from cStringIO import StringIO
-from subvertpy.delta import apply_txdelta_window
 
-from bzrlib.annotate import reannotate
+from cStringIO import (
+    StringIO,
+    )
+from subvertpy.delta import (
+    apply_txdelta_window,
+    )
 
-from bzrlib.plugins.svn import changes
-from bzrlib.plugins.svn.fileids import idmap_lookup
+from bzrlib.annotate import (
+    reannotate,
+    )
+
+from bzrlib.plugins.svn import (
+    changes,
+    )
 
 
 class Annotater(object):
@@ -33,7 +42,8 @@ class Annotater(object):
         self._repository = repository
         self.fileid = fileid
         self._related_revs = {}
-        (_, branch_path, revnum), mapping = self._repository.lookup_revision_id(revid)
+        (_, branch_path, revnum), mapping = self._repository.lookup_revision_id(
+            revid)
         # FIXME: Cope with restarts
         for (revmeta, mapping) in self._repository._iter_reverse_revmeta_mapping_history(branch_path, revnum, 
                 to_revnum=0, mapping=mapping):
@@ -55,15 +65,16 @@ class Annotater(object):
             raise KeyError(path)
         ip = path[len(revmeta.branch_path):].strip("/")
         idmap = self._repository.get_fileid_map(revmeta, mapping)
-        return idmap_lookup(idmap, mapping, ip)
+        return idmap.lookup(mapping, ip)
 
     def _handler(self, path, rev, revprops):
         try:
             fileid, revid, _ = self._get_ids(path, rev, revprops)
+            assert revid is not None
         except KeyError:
-            # Not interested, doesn't appear to be related
-            return None
-        assert revid is not None
+            # Related file in Subversion but not in Bazaar
+            # We still apply the delta since we'll need the fulltext later
+            fileid = None
         stream = StringIO()
         def apply_window(window):
             if window is None:
