@@ -13,8 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
 """Authentication token retrieval."""
 
+
+import subvertpy
 from subvertpy import (
     ra,
     )
@@ -25,6 +28,9 @@ from bzrlib.config import (
     AuthenticationConfig,
     CredentialStore,
     )
+from bzrlib.errors import (
+    BzrError,
+    )
 from bzrlib.trace import (
     mutter,
     )
@@ -32,14 +38,17 @@ from bzrlib.ui import (
     ui_factory,
     )
 
-AUTH_PARAM_DEFAULT_USERNAME = 'svn:auth:username'
-AUTH_PARAM_DEFAULT_PASSWORD = 'svn:auth:password'
+AUTH_PARAM_DEFAULT_USERNAME = getattr(subvertpy, "AUTH_PARAM_DEFAULT_USERNAME",
+                                      'svn:auth:username')
+AUTH_PARAM_DEFAULT_PASSWORD = getattr(subvertpy, "AUTH_PARAM_DEFAULT_PASSWORD", 
+                                      'svn:auth:password')
+
+SSL_NOTYETVALID = getattr(subvertpy, "SSL_NOTYETVALID", 0x00000001)
+SSL_EXPIRED     = getattr(subvertpy, "SSL_EXPIRED", 0x00000002)
+SSL_CNMISMATCH  = getattr(subvertpy, "SSL_CNMISMATCH", 0x00000004)
+SSL_UNKNOWNCA   = getattr(subvertpy, "SSL_UNKNOWNCA", 0x00000008)
+SSL_OTHER       = getattr(subvertpy, "SSL_OTHER", 0x40000000)
 
-SSL_NOTYETVALID = 0x00000001
-SSL_EXPIRED     = 0x00000002
-SSL_CNMISMATCH  = 0x00000004
-SSL_UNKNOWNCA   = 0x00000008
-SSL_OTHER       = 0x40000000
 
 class SubversionAuthenticationConfig(AuthenticationConfig):
     """Simple extended version of AuthenticationConfig that can provide 
@@ -47,7 +56,10 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
     """
     def __init__(self, scheme, host, port, path, file=None):
         super(SubversionAuthenticationConfig, self).__init__(file)
-        self.scheme = scheme
+        if scheme.startswith('svn+http'):
+            self.scheme = scheme[len("svn+"):]
+        else:
+            self.scheme = scheme
         self.host = host
         self.port = port
         self.path = path
@@ -60,10 +72,8 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         """
         mutter("Obtaining username for SVN connection")
         username = self.get_user(self.scheme, host=self.host, path=self.path, 
-                                 realm=realm)
-        if username is None:
-            return None
-        return (username, False)
+                                 realm=realm, ask=True)
+        return (username.encode('utf-8'), False)
 
     def get_svn_simple(self, realm, username, may_save):
         """Look up a Subversion user name+password combination in the Bazaar 
@@ -76,7 +86,7 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         mutter("Obtaining username and password for SVN connection %r (username: %r)", 
                realm, username)
         username = self.get_user(self.scheme, 
-                host=self.host, path=self.path, realm=realm) or username
+                host=self.host, path=self.path, realm=realm, ask=True)
         password = self.get_password(self.scheme, host=self.host, 
             path=self.path, user=username, 
             realm=realm, prompt="%s %s password" % (realm, username))
@@ -208,12 +218,45 @@ class SubversionCredentialStore(CredentialStore):
         super(SubversionCredentialStore, self).__init__()
         self.auth = ra.Auth([ra.get_simple_provider()])
 
+    def _get_realm(self, credentials):
+        if credentials.get('port') is None:
+            import socket
+            try:
+                credentials['port'] = socket.getservbyname(credentials['scheme'])
+            except socket.error:
+                mutter("Unable to look up default port for %(scheme)s", 
+                       credentials)
+                return None
+        return "<%(scheme)s://%(host)s:%(port)s> %(realm)s" % credentials
+
     def decode_password(self, credentials):
-        svn_realm = "<%(scheme)s://%(host)s:%(port)s> %(realm)s" % credentials
+        svn_realm = self._get_realm(credentials)
+        if svn_realm is None:
+            return None
         creds = self.auth.credentials("svn.simple", svn_realm)
-        (username, password, may_save) = creds.next()
+        try:
+            (username, password, may_save) = creds.next()
+        except StopIteration:
+            return None
         if credentials.get("user") not in (username, None):
             # Subversion changed the username
             return None
         return password
 
+    def get_credentials(self, scheme, host, port=None, user=None, path=None, 
+                        realm=None):
+        assert isinstance(realm, str) or realm is None
+        credentials = { "scheme": scheme, "host": host, "port": port, 
+            "realm": realm, "user": user}
+        svn_realm = self._get_realm(credentials)
+        if svn_realm is None:
+            return None
+        creds = self.auth.credentials("svn.simple", svn_realm)
+        try:
+            (username, password, may_save) = creds.next()
+        except StopIteration:
+            return None
+        credentials['user'] = username
+        credentials['password'] = password
+        return credentials
+
