@@ -13,8 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Handles branch-specific operations."""
 
+
 import os
 from subvertpy import (
     ERR_FS_NO_SUCH_REVISION,
@@ -24,7 +27,9 @@ from subvertpy import (
     )
 
 from bzrlib import (
+    osutils,
     tag,
+    trace,
     ui,
     urlutils,
     )
@@ -32,6 +37,8 @@ from bzrlib.branch import (
     Branch,
     BranchFormat,
     BranchCheckResult,
+    BranchPushResult,
+    GenericInterBranch,
     InterBranch,
     PullResult,
     )
@@ -43,19 +50,19 @@ from bzrlib.decorators import (
     )
 from bzrlib.errors import (
     DivergedBranches,
+    LocalRequiresBoundBranch,
     NoSuchRevision,
     NotBranchError,
     UnstackableBranchFormat,
     )
+from bzrlib.foreign import (
+    ForeignBranch,
+    )
 from bzrlib.revision import (
     NULL_REVISION,
     is_null,
     ensure_null,
     )
-from bzrlib.trace import (
-    is_quiet,
-    mutter,
-    )
 from bzrlib.workingtree import (
     WorkingTree,
     )
@@ -63,21 +70,19 @@ from bzrlib.workingtree import (
 from bzrlib.plugins.svn import (
     util,
     )
+from bzrlib.plugins.svn.fetch import (
+    InterFromSvnRepository,
+    )
 from bzrlib.plugins.svn.push import (
     InterToSvnRepository,
     create_branch_with_hidden_commit,
-    dpush,
-    push,
     )
 from bzrlib.plugins.svn.config import (
     BranchConfig,
     )
 from bzrlib.plugins.svn.errors import (
     NotSvnBranchPath,
-    )
-from bzrlib.plugins.svn.foreign import (
-    ForeignBranch,
-    FakeControlFiles,
+    PushToEmptyBranch,
     )
 from bzrlib.plugins.svn.format import (
     get_rich_root_format,
@@ -96,19 +101,60 @@ class SubversionBranchCheckResult(BranchCheckResult):
     """Result of checking a Subversion branch."""
 
 
-class SubversionPullResult(PullResult):
-    """Subversion pull result. """
+class SubversionSourcePullResult(PullResult):
+    """Subversion source pull result. """
 
     def report(self, to_file):
-        if not is_quiet():
-            if self.old_revid == self.new_revid:
+        if not trace.is_quiet():
+            if self.old_revid in (self.new_revid, NULL_REVISION):
                 to_file.write('No revisions to pull.\n')
             else:
+                if self.new_revmeta is None:
+                    self.new_revmeta, _ = self.source_branch.repository._get_revmeta(self.new_revid)
                 to_file.write('Now on revision %d (svn revno: %d).\n' % 
                         (self.new_revno, self.new_revmeta.revnum))
         self._show_tag_conficts(to_file)
 
 
+class SubversionTargetBranchPushResult(BranchPushResult):
+    """Subversion target branch push result."""
+
+    def _lookup_revno(self, revid):
+        assert isinstance(revid, str), "was %r" % revid
+        # Try in source branch first, it'll be faster
+        try:
+            return self.source_branch.revision_id_to_revno(revid)
+        except NoSuchRevision:
+            return self.target_branch.revision_id_to_revno(revid)
+
+    @property
+    def old_revno(self):
+        return self._lookup_revno(self.old_revid)
+
+    @property
+    def new_revno(self):
+        return self._lookup_revno(self.new_revid)
+
+
+class SubversionTargetPullResult(PullResult):
+    """Subversion target branch pull result."""
+
+    def _lookup_revno(self, revid):
+        # Try in source branch first, it'll be faster
+        try:
+            return self.source_branch.revision_id_to_revno(revid)
+        except NoSuchRevision:
+            return self.target_branch.revision_id_to_revno(revid)
+
+    @property
+    def old_revno(self):
+        return self._lookup_revno(self.old_revid)
+
+    @property
+    def new_revno(self):
+        return self._lookup_revno(self.new_revid)
+
+
 class SvnBranch(ForeignBranch):
     """Maps to a Branch in a Subversion repository """
 
@@ -127,7 +173,6 @@ class SvnBranch(ForeignBranch):
         self.layout = self.repository.get_layout()
         assert isinstance(self.repository, SvnRepository)
         super(SvnBranch, self).__init__(mapping or self.repository.get_mapping())
-        self.control_files = FakeControlFiles()
         self._lock_mode = None
         self._lock_count = 0
         self._branch_path = branch_path.strip("/")
@@ -151,6 +196,9 @@ class SvnBranch(ForeignBranch):
         if type not in ('branch', 'tag') or ip != '':
             raise NotBranchError(branch_path)
 
+    def _push_should_merge_tags(self):
+        return self.supports_tags()
+
     def check_path(self):
         return self.repository.transport.check_path(self._branch_path, self._revnum or self.repository.get_latest_revnum())
 
@@ -167,18 +215,6 @@ class SvnBranch(ForeignBranch):
         """
         self._branch_path = branch_path.strip("/")
 
-    def _get_append_revisions_only(self):
-        return self.get_config().get_append_revisions_only()
-
-    def unprefix(self, relpath):
-        """Remove the branch path from a relpath.
-
-        :param relpath: path from the repository root.
-        """
-        assert relpath.startswith(self.get_branch_path()), \
-                "expected %s prefix, got %s" % (self.get_branch_path(), relpath)
-        return relpath[len(self.get_branch_path()):].strip("/")
-
     def get_branch_path(self, revnum=None):
         """Find the branch path of this branch in the specified revnum.
 
@@ -208,8 +244,7 @@ class SvnBranch(ForeignBranch):
         """Return the revmeta element for the last revision in this branch.
         """
         for revmeta, mapping in self._revision_meta_history():
-            if not revmeta.is_hidden(mapping):
-                return revmeta, mapping
+            return revmeta, mapping
         return None, None
 
     def check(self):
@@ -267,14 +302,14 @@ class SvnBranch(ForeignBranch):
             revnum = self.get_revnum()
 
         svn_url = bzr_to_svn_url(self.base)
-        os.mkdir(to_location)
-        wc.ensure_adm(to_location, self.repository.uuid, svn_url,
+        os.mkdir(to_location.encode(osutils._fs_enc))
+        wc.ensure_adm(to_location.encode("utf-8"), self.repository.uuid, svn_url,
                       bzr_to_svn_url(self.repository.base), revnum)
-        adm = wc.WorkingCopy(None, to_location, write_lock=True)
+        adm = wc.WorkingCopy(None, to_location.encode("utf-8"), write_lock=True)
         try:
             conn = self.repository.transport.connections.get(svn_url)
             try:
-                update_wc(adm, to_location, conn, revnum)
+                update_wc(adm, to_location.encode("utf-8"), conn, revnum)
             finally:
                 if not conn.busy:
                     self.repository.transport.add_connection(conn)
@@ -328,9 +363,8 @@ class SvnBranch(ForeignBranch):
             base_revid = rev.parent_ids[0]
         else:
             base_revid = NULL_REVISION
-        push(self.repository.get_graph(), self.repository, 
-             self.get_branch_path(), self.get_config(), base_revid, 
-             self.repository, rev)
+        interrepo = InterToSvnRepository(self.repository, self.repository)
+        interrepo.push(self.get_branch_path(), self.get_config(), rev)
         self._clear_cached_state()
 
     def set_last_revision_info(self, revno, revid):
@@ -340,8 +374,10 @@ class SvnBranch(ForeignBranch):
         todo = self._mainline_missing_revisions(other_repo, stop_revision)
         if todo is not None:
             return todo
-        # Not possible to add cleanly onto mainline, perhaps need a replace operation
-        return self._otherline_missing_revisions(other_repo, stop_revision, overwrite)
+        # Not possible to add cleanly onto mainline, perhaps need a replace 
+        # operation
+        return self._otherline_missing_revisions(other_repo, stop_revision,
+                                                 overwrite)
 
     def _mainline_missing_revisions(self, other_repo, stop_revision):
         """Find the revisions missing on the mainline.
@@ -358,12 +394,14 @@ class SvnBranch(ForeignBranch):
             missing.append(revid)
         return None
 
-    def _otherline_missing_revisions(self, other_repo, stop_revision, overwrite=False):
+    def _otherline_missing_revisions(self, other_repo, stop_revision, 
+                                     overwrite=False):
         """Find the revisions missing on the mainline.
         
         :param other: Other branch to retrieve revisions from.
         :param stop_revision: Revision to stop fetching at.
-        :param overwrite: Whether or not the existing data should be overwritten
+        :param overwrite: Whether or not the existing data should be 
+            overwritten
         """
         missing = []
         for revid in other_repo.iter_reverse_revision_history(stop_revision):
@@ -387,7 +425,8 @@ class SvnBranch(ForeignBranch):
         if is_null(revision_id):
             return 0
         revmeta_history = self._revision_meta_history()
-        # FIXME: Maybe we can parse revision_id as a bzr-svn roundtripped revision?
+        # FIXME: Maybe we can parse revision_id as a bzr-svn roundtripped 
+        # revision?
         for revmeta, mapping in revmeta_history:
             if revmeta.get_revision_id(mapping) == revision_id:
                 return revmeta.get_revno(mapping)
@@ -402,26 +441,31 @@ class SvnBranch(ForeignBranch):
 
     def set_push_location(self, location):
         """See Branch.set_push_location()."""
-        mutter("setting push location for %s to %s", self.base, location)
+        trace.mutter("setting push location for %s to %s", self.base, location)
 
     def get_push_location(self):
         """See Branch.get_push_location()."""
         # get_push_location not supported on Subversion
         return None
 
-    def _iter_revision_meta_ancestry(self):
+    def _iter_revision_meta_ancestry(self, pb=None):
         return self.repository._iter_reverse_revmeta_mapping_ancestry(
-            self.get_branch_path(), self._revnum or self.repository.get_latest_revnum(), self.mapping, lhs_history=self._revision_meta_history())
+            self.get_branch_path(), 
+            self._revnum or self.repository.get_latest_revnum(), self.mapping,
+            lhs_history=self._revision_meta_history(), pb=pb)
 
     def _revision_meta_history(self):
         if self._revmeta_cache is None:
-            self._revmeta_cache = util.lazy_readonly_list(self.repository._iter_reverse_revmeta_mapping_history(self.get_branch_path(), self._revnum or self.repository.get_latest_revnum(), to_revnum=0, mapping=self.mapping))
+            self._revmeta_cache = util.lazy_readonly_list(
+                self.repository._iter_reverse_revmeta_mapping_history(
+                    self.get_branch_path(), 
+                    self._revnum or self.repository.get_latest_revnum(), 
+                    to_revnum=0, mapping=self.mapping))
         return self._revmeta_cache
 
     def _gen_revision_history(self):
-        """Generate the revision history from last revision
-        """
-        history = [revmeta.get_revision_id(mapping) for revmeta, mapping in self._revision_meta_history() if not revmeta.is_hidden(mapping)]
+        """Generate the revision history from last revision."""
+        history = [revmeta.get_revision_id(mapping) for revmeta, mapping in self._revision_meta_history()]
         history.reverse()
         return history
 
@@ -432,54 +476,19 @@ class SvnBranch(ForeignBranch):
         last_revmeta, mapping = self.last_revmeta()
         return last_revmeta.get_revision_id(mapping)
 
-    @needs_write_lock
-    def dpull(self, source, stop_revision=None):
-        return dpush(self, source, stop_revision)
+    def get_push_merged_revisions(self):
+        return (self.layout.push_merged_revisions(self.project) and 
+                self.get_config().get_push_merged_revisions())
 
     def import_last_revision_info(self, source_repo, revno, revid):
         interrepo = InterToSvnRepository(source_repo, self.repository)
         todo = self._missing_revisions(source_repo, revid, overwrite=False)
         if todo is None:
             raise DivergedBranches(self, None)
-        interrepo.push_branch(todo, self.layout, self.project, self.get_branch_path(), self.get_config())
-
-    def pull(self, source, overwrite=False, stop_revision=None, 
-             _hook_master=None, run_hooks=True, _push_merged=None,
-             _override_svn_revprops=None):
-        """See Branch.pull()."""
-        result = PullResult()
-        result.source_branch = source
-        result.master_branch = None
-        result.target_branch = self
-        source.lock_read()
-        try:
-            (result.old_revno, result.old_revid) = self.last_revision_info()
-            self.update_revisions(source, stop_revision, overwrite, 
-                                  _push_merged=_push_merged,
-                                  _override_svn_revprops=_override_svn_revprops)
-            result.tag_conflicts = source.tags.merge_to(self.tags, overwrite)
-            (result.new_revno, result.new_revid) = self.last_revision_info()
-            return result
-        finally:
-            source.unlock()
-
-    @needs_write_lock
-    def update_revisions(self, other, stop_revision=None, overwrite=False,
-                         graph=None, _override_svn_revprops=None, 
-                         _push_merged=None):
-        """Pull in new perfect-fit revisions.
-
-        :param other: Another Branch to pull from
-        :param stop_revision: Updated until the given revision
-        :param overwrite: Always set the branch pointer, rather than checking
-            to see if it is a proper descendant.
-        :param graph: A Graph object that can be used to query history
-            information. This can be None.
-        :return: None
-        """
-        return InterBranch.get(other, self).update_revisions(stop_revision,
-            overwrite, graph, _override_svn_revprops=_override_svn_revprops, 
-            _push_merged=_push_merged)
+        assert todo != []
+        interrepo.push_branch(todo, self.layout, self.project, 
+            self.get_branch_path(), self.get_config(),
+            self.get_push_merged_revisions(), overwrite=False)
 
     def generate_revision_history(self, revision_id, last_rev=None, 
         other_branch=None):
@@ -507,14 +516,15 @@ class SvnBranch(ForeignBranch):
           be None to copy complete history.
         """
         if revision_id is None:
-            revno, revision_id = self.last_revision_info()
-        else:
-            revno = self.revision_id_to_revno(revision_id)
-        destination.set_last_revision_info(revno, revision_id)
+            revision_id = self.last_revision()
+        destination.generate_revision_history(revision_id)
 
     def is_locked(self):
         return self._lock_count != 0
 
+    def break_lock(self):
+        pass
+
     def lock_write(self):
         """See Branch.lock_write()."""
         # TODO: Obtain lock on the remote server?
@@ -568,12 +578,20 @@ class SvnBranch(ForeignBranch):
 
     __repr__ = __str__
 
+    def _basic_push(self, target, overwrite=False, stop_revision=None):
+        return InterBranch.get(self, target)._basic_push(
+            overwrite, stop_revision)
+
 
 class SvnBranchFormat(BranchFormat):
     """Branch format for Subversion Branches."""
+
     def __init__(self):
         BranchFormat.__init__(self)
 
+    def network_name(self):
+        return "subversion"
+
     def __get_matchingbzrdir(self):
         """See BranchFormat.__get_matchingbzrdir()."""
         from bzrlib.plugins.svn.remote import SvnRemoteFormat
@@ -603,7 +621,7 @@ class SvnBranchFormat(BranchFormat):
             return tag.DisabledTags(branch)
 
 
-class InterSvnOtherBranch(InterBranch):
+class InterSvnOtherBranch(GenericInterBranch):
     """InterBranch implementation that is optimized for copying from 
     Subversion.
 
@@ -632,7 +650,10 @@ class InterSvnOtherBranch(InterBranch):
             # common case of having something to pull, and so that the 
             # check for already merged can operate on the just fetched 
             # graph, which will be cached in memory.
-            self.target.fetch(self.source, stop_revision)
+            interrepo = InterFromSvnRepository(self.source.repository,
+                                               self.target.repository)
+            interrepo.fetch(stop_revision, project=self.source.project,
+                            mapping=self.source.mapping)
             # Check to see if one is an ancestor of the other
             if not overwrite:
                 if graph is None:
@@ -646,11 +667,24 @@ class InterSvnOtherBranch(InterBranch):
         finally:
             self.source.unlock()
 
+    def _basic_push(self, overwrite=False, stop_revision=None):
+        result = BranchPushResult()
+        result.source_branch = self.source
+        result.target_branch = self.target
+        graph = self.target.repository.get_graph(self.source.repository)
+        result.old_revno, result.old_revid = self.target.last_revision_info()
+        self.update_revisions(stop_revision, overwrite=overwrite, graph=graph)
+        # FIXME: Tags
+        result.new_revno, result.new_revid = self.target.last_revision_info()
+        return result
+
     def pull(self, overwrite=False, stop_revision=None, 
              _hook_master=None, run_hooks=True, possible_transports=None,
-             _override_hook_target=None):
+             _override_hook_target=None, local=False):
         """See InterBranch.pull()."""
-        result = SubversionPullResult()
+        if local:
+            raise LocalRequiresBoundBranch()
+        result = SubversionSourcePullResult()
         if _override_hook_target is None:
             result.target_branch = self
         else:
@@ -660,23 +694,41 @@ class InterSvnOtherBranch(InterBranch):
         result.target_branch = self.target
         self.source.lock_read()
         try:
-            (result.old_revno, result.old_revid) = self.target.last_revision_info()
-            try:
-                result.old_revmeta, _ = self.source.repository._get_revmeta(result.old_revid)
-                tags_since_revnum = result.old_revmeta.revnum
-            except NoSuchRevision:
+            (result.old_revno, result.old_revid) = \
+                self.target.last_revision_info()
+            if result.old_revid == NULL_REVISION:
                 result.old_revmeta = None
                 tags_since_revnum = None
-            if stop_revision is not None:
-                result.new_revmeta, _ = self.source.repository._get_revmeta(stop_revnum)
+            else:
+                try:
+                    result.old_revmeta, _ = \
+                        self.source.repository._get_revmeta(result.old_revid)
+                    tags_since_revnum = result.old_revmeta.revnum
+                except NoSuchRevision:
+                    result.old_revmeta = None
+                    tags_since_revnum = None
+            if stop_revision == NULL_REVISION:
+                result.new_revid = NULL_REVISION
+                result.new_revmeta = None
+                tags_until_revnum = 0
+            elif stop_revision is not None:
+                result.new_revmeta, _ = \
+                    self.source.repository._get_revmeta(stop_revision)
                 tags_until_revnum = result.new_revmeta.revnum
             else:
+                result.new_revmeta = None
                 tags_until_revnum = self.source.repository.get_latest_revnum()
             self.update_revisions(stop_revision, overwrite)
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags, overwrite, _from_revnum=tags_since_revnum, _to_revnum=tags_until_revnum)
-            (result.new_revno, result.new_revid) = self.target.last_revision_info()
-            if stop_revision is None:
-                result.new_revmeta, _ = self.source.repository._get_revmeta(result.new_revid)
+            (result.new_revno, result.new_revid) = \
+                self.target.last_revision_info()
+            if self.source.supports_tags():
+                if result.old_revid == result.new_revid:
+                    # Upstream branch wasn't added but perhaps new tags were 
+                    # added since.
+                    tags_since_revnum = self.source.get_revnum()
+                result.tag_conflicts = self.source.tags.merge_to(
+                    self.target.tags, overwrite, _from_revnum=tags_since_revnum,
+                    _to_revnum=tags_until_revnum)
             if _hook_master:
                 result.master_branch = _hook_master
                 result.local_branch = result.target_branch
@@ -692,7 +744,8 @@ class InterSvnOtherBranch(InterBranch):
 
     @classmethod
     def is_compatible(self, source, target):
-        return isinstance(source, SvnBranch)
+        return (isinstance(source, SvnBranch) and 
+                not isinstance(target, SvnBranch))
 
 InterBranch.register_optimiser(InterSvnOtherBranch)
 
@@ -702,41 +755,193 @@ class InterOtherSvnBranch(InterBranch):
     Subversion.
 
     """
-    def update_revisions(self, stop_revision=None, overwrite=False, graph=None,
-        _push_merged=False, _override_svn_revprops=None):
+    def update_revisions(self, stop_revision=None, overwrite=False, graph=None):
         """See Branch.update_revisions()."""
-        if stop_revision is None:
-            stop_revision = ensure_null(self.source.last_revision())
-        if (self.target.last_revision() == stop_revision or
-            self.target.last_revision() == self.source.last_revision()):
-            return
-        # Request graph from other repository, since it's most likely 
-        # than Subversion
-        graph = self.source.repository.get_graph(self.target.repository)
-        if not graph.is_ancestor(self.target.last_revision(), stop_revision):
-            if graph.is_ancestor(stop_revision, self.target.last_revision()):
-                return
+        self._update_revisions(stop_revision=stop_revision, overwrite=overwrite,
+            graph=graph)
+
+    def _target_is_empty(self, graph, revid):
+        parent_revids = tuple(graph.get_parent_map([revid])[revid])
+        if parent_revids != (NULL_REVISION,):
+            return False
+        tree_contents = self.target.repository.transport.get_dir(
+            self.target.get_branch_path(), self.target.get_revnum())[0]
+        return tree_contents == {}
+
+    def _todo(self, graph, stop_revision, overwrite=False):
+        old_last_revid = self.target.last_revision()
+        if not graph.is_ancestor(old_last_revid, stop_revision):
+            if graph.is_ancestor(stop_revision, old_last_revid):
+                return []
             if not overwrite:
+                if self._target_is_empty(graph, old_last_revid):
+                    raise PushToEmptyBranch(self.target, self.source)
                 raise DivergedBranches(self.target, self.source)
-        todo = self.target._missing_revisions(self.source.repository, stop_revision, overwrite)
+        todo = self.target._missing_revisions(self.source.repository,
+            stop_revision, overwrite)
         if todo is None:
             raise DivergedBranches(self.target, self.source)
-        if _push_merged is None:
-            _push_merged = (self.target.layout.push_merged_revisions(self.target.project) and 
-                            self.target.get_config().get_push_merged_revisions())
-        if self.target.mapping.supports_hidden and self.target.repository.has_revision(stop_revision):
+        return todo
+
+    def _get_interrepo(self, graph):
+        interrepo = InterToSvnRepository(self.source.repository, 
+                self.target.repository, graph)
+        # Add the current branch path as a hint, since it's very likely we'll 
+        # need it as base for new revisions that will be pushed, and 
+        # push should preferably use it over other paths in the 
+        # repository that contain the same data.
+        last_branch_path = self.target.get_branch_path()
+        last_revnum = self.target.repository._log.find_latest_change(
+            last_branch_path, self.target.repository.get_latest_revnum())
+        interrepo.add_path_info(self.target.last_revision(),
+            last_branch_path,
+            ((self.target.repository.uuid, last_branch_path, last_revnum), self.target.mapping))
+        return interrepo
+
+    def _update_revisions(self, stop_revision=None, overwrite=False, graph=None,
+        push_merged=False, override_svn_revprops=None):
+        old_last_revid = self.target.last_revision()
+        if stop_revision is None:
+            stop_revision = ensure_null(self.source.last_revision())
+        if old_last_revid == stop_revision:
+            return old_last_revid, old_last_revid, None
+        # Request graph from other repository, since it's most likely faster
+        # than Subversion
+        graph = self.source.repository.get_graph(self.target.repository)
+        todo = self._todo(graph, stop_revision, overwrite=overwrite)
+        if todo == []:
+            return old_last_revid, old_last_revid, None
+        if push_merged is None:
+            push_merged = self.target.get_push_merged_revisions() 
+        if (self.target.mapping.supports_hidden and 
+            self.target.repository.has_revision(stop_revision)):
             # Revision is already present in the repository, so just 
             # copy from there.
-            create_branch_with_hidden_commit(self.target.repository, 
-                self.target.get_branch_path(), stop_revision, set_metadata=True,
-                deletefirst=True)
+            new_last_revid, new_foreign_info = create_branch_with_hidden_commit(
+                self.target.repository, self.target.get_branch_path(),
+                stop_revision, set_metadata=True, deletefirst=True)
         else:
-            interrepo = InterToSvnRepository(self.source.repository, 
-                self.target.repository, graph)
-            interrepo.push_branch(todo, self.target.layout, self.target.project, 
-                self.target.get_branch_path(), self.target.get_config(),
-                push_merged=_push_merged)
+            interrepo = self._get_interrepo(graph)
+            assert todo != []
+            (count, (new_last_revid, new_foreign_info)) = interrepo.push_branch(
+                todo, self.target.layout, self.target.project, 
+                self.target.get_branch_path(), self.target.get_config(), 
+                push_merged, overwrite=overwrite)
         self.target._clear_cached_state()
+        assert isinstance(new_last_revid, str)
+        assert isinstance(old_last_revid, str)
+        return (old_last_revid, new_last_revid, new_foreign_info)
+
+    def _update_revisions_lossy(self, stop_revision=None):
+        """Push derivatives of the revisions missing from target from source 
+        into target.
+
+        :param target: Branch to push into
+        :param source: Branch to retrieve revisions from
+        :param stop_revision: If not None, stop at this revision.
+        :return: Map of old revids to new revids.
+        """
+        self.source.lock_write()
+        try:
+            if stop_revision is None:
+                stop_revision = ensure_null(self.source.last_revision())
+            graph = self.source.repository.get_graph(self.target.repository)
+            todo = self._todo(graph, stop_revision)
+            if todo == []:
+                return { self.source.last_revision(): self.source.last_revision() }
+            revid_map = {}
+            target_branch_path = self.target.get_branch_path()
+            target_config = self.target.get_config()
+            # Request graph from other repository, since it's most likely faster
+            # than Subversion
+            interrepo = self._get_interrepo(graph)
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                # FIXME: Call create_branch_with_hidden_commit if the revision 
+                # is already present in the target repository ?
+                for rev in self.source.repository.get_revisions(todo):
+                    pb.update("pushing revisions", todo.index(rev.revision_id), 
+                              len(todo))
+                    if len(rev.parent_ids) == 0:
+                        base_revid = NULL_REVISION
+                    elif rev.parent_ids[0] in revid_map:
+                        base_revid = revid_map[rev.parent_ids[0]]
+                    else:
+                        base_revid = rev.parent_ids[0]
+                    revid_map[rev.revision_id], _ = interrepo.push(
+                        target_branch_path, target_config, rev, 
+                        push_metadata=False, base_revid=base_revid)
+            finally:
+                pb.finished()
+            interrepo = InterFromSvnRepository(self.target.repository, 
+                                               self.source.repository)
+            interrepo.fetch(revision_id=revid_map[rev.revision_id],
+                            mapping=self.target.mapping,
+                            project=self.target.project)
+            self.target._clear_cached_state()
+            assert stop_revision in revid_map
+            assert len(revid_map.keys()) > 0
+            return revid_map
+        finally:
+            self.source.unlock()
+
+    def lossy_push(self, stop_revision=None):
+        """See InterBranch.lossy_push()."""
+        result = SubversionTargetBranchPushResult()
+        result.target_branch = self.target
+        result.master_branch = None
+        result.source_branch = self.source
+        self.source.lock_write()
+        try:
+            result.old_revid = self.target.last_revision()
+            result.revidmap = self._update_revisions_lossy(stop_revision)
+            result.new_revid = self.target.last_revision()
+            # FIXME: Tags ?
+            return result
+        finally:
+            self.source.unlock()
+
+    def update_tags(self, overwrite=False):
+        return self.source.tags.merge_to(self.target.tags, overwrite)
+
+    def push(self, overwrite=False, stop_revision=None, 
+            _push_merged=None, _override_svn_revprops=None):
+        """See InterBranch.push()."""
+        result = SubversionTargetBranchPushResult()
+        result.target_branch = self.target
+        result.master_branch = None
+        result.source_branch = self.source
+        self.source.lock_read()
+        try:
+            (result.old_revid, result.new_revid, result.new_foreign_info) = \
+                self._update_revisions(stop_revision, overwrite, 
+                    push_merged=_push_merged, 
+                    override_svn_revprops=_override_svn_revprops)
+            result.tag_conflicts = self.update_tags(overwrite)
+            return result
+        finally:
+            self.source.unlock()
+
+    def pull(self, overwrite=False, stop_revision=None, 
+             _hook_master=None, run_hooks=True, possible_transports=None,
+             _push_merged=None, _override_svn_revprops=None, local=False):
+        """See InterBranch.pull()."""
+        if local:
+            raise LocalRequiresBoundBranch()
+        result = SubversionTargetPullResult()
+        result.source_branch = self.source
+        result.master_branch = None
+        result.target_branch = self.target
+        self.source.lock_read()
+        try:
+            (result.old_revid, result.new_revid, result.new_foreign_info) = \
+                self._update_revisions(stop_revision, overwrite, 
+                    push_merged=_push_merged, 
+                    override_svn_revprops=_override_svn_revprops)
+            result.tag_conflicts = self.update_tags(overwrite)
+            return result
+        finally:
+            self.source.unlock()
 
     @classmethod
     def is_compatible(self, source, target):
