@@ -13,8 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Stores per-repository settings."""
 
+
 import os
 from subvertpy import (
     SubversionException,
@@ -22,25 +25,25 @@ from subvertpy import (
     )
 
 from bzrlib import (
-        osutils,
-        urlutils,
-        trace,
-        )
+    osutils,
+    urlutils,
+    trace,
+    )
 from bzrlib.config import (
-        ConfigObj,
-        IniBasedConfig,
-        config_dir,
-        ensure_config_dir_exists,
-        GlobalConfig,
-        LocationConfig,
-        Config,
-        STORE_BRANCH,
-        STORE_GLOBAL,
-        STORE_LOCATION,
-        )
+    ConfigObj,
+    IniBasedConfig,
+    config_dir,
+    ensure_config_dir_exists,
+    GlobalConfig,
+    LocationConfig,
+    Config,
+    STORE_BRANCH,
+    STORE_GLOBAL,
+    STORE_LOCATION,
+    )
 from bzrlib.errors import (
-        BzrError,
-        )
+    BzrError,
+    )
 
 
 
@@ -366,13 +369,13 @@ class BranchConfig(Config):
                 return value
         return None
 
-    def get_append_revisions_only(self):
+    def get_append_revisions_only(self, default=None):
         """Check whether it is possible to remove revisions from the mainline.
         """
         try:
             return self.get_bool("append_revisions_only")
         except KeyError:
-            return None
+            return default
 
     def _get_user_id(self):
         """Get the user id from the 'email' key in the current section."""
