@@ -13,12 +13,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Fetching revisions from Subversion repositories in batches."""
 
-from collections import (
-    deque,
-    defaultdict,
-    )
+
+from collections import deque
+try:
+    from collections import defaultdict
+except ImportError:
+    from bzrlib.plugins.svn.pycompat import defaultdict
+
 from cStringIO import (
     StringIO,
     )
@@ -38,6 +43,7 @@ from bzrlib import (
     delta,
     lru_cache,
     osutils,
+    trace,
     ui,
     urlutils,
     )
@@ -57,9 +63,6 @@ from bzrlib.revision import (
 from bzrlib.repository import (
     InterRepository,
     )
-from bzrlib.trace import (
-    mutter,
-    )
 from bzrlib.versionedfile import (
     FulltextContentFactory,
     )
@@ -71,15 +74,10 @@ from bzrlib.plugins.svn.errors import (
     AbsentPath,
     FileIdMapIncomplete,
     InvalidFileName,
-    InconsistentLHSParent,
     convert_svn_error,
     )
 from bzrlib.plugins.svn.fileids import (
     get_local_changes,
-    idmap_lookup,
-    )
-from bzrlib.plugins.svn.foreign import (
-    escape_commit_message,
     )
 from bzrlib.plugins.svn.mapping import (
     SVN_PROP_BZR_PREFIX,
@@ -92,10 +90,9 @@ from bzrlib.plugins.svn.transport import (
     url_join_unescaped_path,
     )
 
-# Number of revisions to fetch before writing pack to disk
-FETCH_COMMIT_WRITE_SIZE = 999
-# Size of group in which revids are checked when looking for missing revisions
-CHECK_PRESENT_INTERVAL = 1000
+# Max size of group in which revids are checked when looking for missing 
+# revisions
+MAX_CHECK_PRESENT_INTERVAL = 1000
 # Size of the text cache to keep
 TEXT_CACHE_SIZE = 1024 * 1024 * 50
 
@@ -118,26 +115,6 @@ def inventory_ancestors(inv, fileid, exceptions):
     return ret
 
 
-def check_inventory_delta(delta):
-    """Check the internal consistency of an inventory delta.
-
-    :param delta: An inventory delta
-    """
-    old_paths = set()
-    new_paths = set()
-    file_ids = set()
-    for (old_path, new_path, file_id, ie) in delta:
-        if old_path is not None and old_path in old_paths:
-            raise AssertionError("old path %s exists more than once" % old_path)
-        if new_path is not None and new_path in new_paths:
-            raise AssertionError("new path %s exists more than once" % new_path)
-        if file_id in file_ids:
-            raise AssertionError("file id %s exists more than once" % file_id)
-        old_paths.add(old_path)
-        new_paths.add(new_path)
-        file_ids.add(file_id)
-
-
 def md5_strings(lines):
     """Return the MD5sum of a list of lines."""
     s = osutils.md5()
@@ -190,7 +167,8 @@ class PathStrippingDirectoryEditor(object):
         if path.strip("/") == self.editor.prefix:
             t = self.editor.actual.open_root(base_revnum)
         elif self.actual is not None:
-            t = self.actual.open_directory(self.editor.strip_prefix(path), base_revnum)
+            t = self.actual.open_directory(self.editor.strip_prefix(path),
+                base_revnum)
         else:
             t = None
         return PathStrippingDirectoryEditor(self.editor, path, t)
@@ -240,7 +218,10 @@ class PathStrippingEditor(object):
         self.prefix = path.strip("/")
 
     def __getattr__(self, name):
-        return getattr(super(PathStrippingEditor, self), name, getattr(self.actual, name))
+        try:
+            return getattr(super(PathStrippingEditor, self), name)
+        except AttributeError:
+            return getattr(self.actual, name)
 
     def strip_prefix(self, path):
         path = path.strip("/")
@@ -276,7 +257,8 @@ class DeltaBuildEditor(object):
         self.mapping = mapping
 
     def set_target_revision(self, revnum):
-        assert self.revmeta.revnum == revnum, "Expected %d, got %d" % (self.revmeta.revnum, revnum)
+        if self.revmeta.revnum != revnum:
+            raise AssertionError("Expected %d, got %d" % (self.revmeta.revnum, revnum))
 
     def open_root(self, base_revnum=None):
         return self._open_root(base_revnum)
@@ -298,7 +280,8 @@ class DirectoryBuildEditor(object):
         if self.path == "":
             # If it has changed, it has definitely been reported by now
             if not self.editor.revmeta.knows_fileprops():
-                self.editor.revmeta._fileprops = self.editor.revmeta.get_previous_fileprops()
+                self.editor.revmeta._fileprops = \
+                    self.editor.revmeta.get_previous_fileprops()
         self._close()
 
     def add_directory(self, path, copyfrom_path=None, copyfrom_revnum=-1):
@@ -322,7 +305,8 @@ class DirectoryBuildEditor(object):
         if self.path == "":
             # Replay lazy_dict, since it may be more expensive
             if not self.editor.revmeta.knows_fileprops():
-                self.editor.revmeta._fileprops = dict(self.editor.revmeta.get_previous_fileprops())
+                self.editor.revmeta._fileprops = dict(
+                    self.editor.revmeta.get_previous_fileprops())
             if value is None:
                 if name in self.editor.revmeta._fileprops:
                     del self.editor.revmeta._fileprops[name]
@@ -339,7 +323,7 @@ class DirectoryBuildEditor(object):
         elif (name.startswith(properties.PROP_WC_PREFIX)):
             pass
         elif name.startswith(properties.PROP_PREFIX):
-            mutter('unsupported dir property %r', name)
+            trace.mutter('unsupported dir property %r', name)
 
         if (not name.startswith(properties.PROP_ENTRY_PREFIX) and
             not name.startswith(properties.PROP_WC_PREFIX)):
@@ -385,7 +369,7 @@ class FileBuildEditor(object):
         elif name == properties.PROP_ENTRY_COMMITTED_REV:
             self.last_file_rev = int(value)
         elif name == properties.PROP_EXTERNALS:
-            mutter('svn:externals property on file!')
+            trace.mutter('svn:externals property on file!')
         elif name in (properties.PROP_ENTRY_COMMITTED_DATE,
                       properties.PROP_ENTRY_LAST_AUTHOR,
                       properties.PROP_ENTRY_LOCK_TOKEN,
@@ -396,7 +380,7 @@ class FileBuildEditor(object):
             pass
         elif (name.startswith(properties.PROP_PREFIX) or
               name.startswith(SVN_PROP_BZR_PREFIX)):
-            mutter('unsupported file property %r', name)
+            trace.mutter('unsupported file property %r', name)
 
     def close(self, checksum=None):
         assert isinstance(self.path, unicode)
@@ -405,8 +389,8 @@ class FileBuildEditor(object):
 
 class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
 
-    def __init__(self, editor, old_path, path, old_id, new_id, parent_file_id, 
-        parent_revids=[], renew_fileids=None):
+    def __init__(self, editor, old_path, path, old_id, new_id, 
+            parent_file_id, parent_revids=[], renew_fileids=None):
         super(DirectoryRevisionBuildEditor, self).__init__(editor, path)
         assert isinstance(new_id, str)
         self.old_id = old_id
@@ -426,7 +410,8 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
             if p in self.editor._deleted:
                 return
             self.editor._deleted.add(p)
-            self.editor._inv_delta.append((p, None, ie.file_id, None))
+            if ie.file_id not in self.editor._get_id_map().values():
+                self.editor._inv_delta.append((p, None, ie.file_id, None))
             if ie.kind != 'directory':
                 return
             for c in ie.children.values():
@@ -445,7 +430,7 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
             text_parents = self.editor._get_text_parents(self.path) or self.parent_revids
             self.editor.texts.add_lines(
                 (self.new_id, self.new_ie.revision),
-                [(self.new_id, revid) for revid in text_parents], [])
+                tuple([(self.new_id, revid) for revid in text_parents]), [])
             self.editor._inv_delta.append((self.old_path, self.path, self.new_id, self.new_ie))
         if self._renew_fileids:
             # Make sure files get re-added that weren't mentioned explicitly
@@ -467,18 +452,17 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
             # This directory was moved here from somewhere else, but the 
             # other location hasn't been removed yet. 
             if copyfrom_path is None:
-                # This should ideally never happen!
-                copyfrom_path = self.editor.old_inventory.id2path(file_id)
-                mutter('no copyfrom path set, assuming %r', copyfrom_path)
-            assert copyfrom_path == self.editor.old_inventory.id2path(file_id)
-            old_path = copyfrom_path
+                old_path = self.editor.old_inventory.id2path(file_id)
+            else:
+                assert copyfrom_path == self.editor.old_inventory.id2path(file_id)
+                old_path = copyfrom_path
             old_file_id = file_id
         else:
             old_path = None
             old_file_id = None
 
-        return DirectoryRevisionBuildEditor(self.editor, old_path, path, 
-            old_file_id, file_id, self.new_id, [])
+        return DirectoryRevisionBuildEditor(self.editor, old_path, path, old_file_id, file_id, 
+            self.new_id, [])
 
     def _open_directory(self, path, base_revnum):
         base_file_id = self.editor._get_old_id(self.old_id, path)
@@ -504,21 +488,21 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
             # This file was moved here from somewhere else, but the 
             # other location hasn't been removed yet. 
             if copyfrom_path is None:
-                # This should ideally never happen
-                copyfrom_path = self.editor.old_inventory.id2path(file_id)
-                mutter('no copyfrom path set, assuming %r', copyfrom_path)
-            assert copyfrom_path == self.editor.old_inventory.id2path(file_id)
-            # No need to rename if it's already in the right spot
-            old_path = copyfrom_path
+                old_path = self.editor.old_inventory.id2path(file_id)
+            else:
+                assert copyfrom_path == self.editor.old_inventory.id2path(file_id)
+                # No need to rename if it's already in the right spot
+                old_path = copyfrom_path
         else:
             old_path = None
-        # TODO: Retrieve base text here when used by replay/replay_range()
-        return FileRevisionBuildEditor(self.editor, old_path, path, 
-                                       file_id, self.new_id)
-
-    def _get_record_stream(self, file_id, revision_id):
-        return self.editor.texts.get_record_stream([(file_id, revision_id)], 
-                                                   'unordered', True).next()
+        if copyfrom_path is not None:
+            # Delta's will be against this text
+            copyfrom_ie = self.editor.get_old_ie(copyfrom_path, copyfrom_revnum)
+            data = self.editor._get_chunked(ie.file_id, ie.revision)
+        else:
+            data = []
+        return FileRevisionBuildEditor(self.editor, old_path, path, file_id, self.new_id,
+            data=data)
 
     def _open_file(self, path, base_revnum):
         base_file_id = self.editor._get_old_id(self.old_id, path)
@@ -526,10 +510,7 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
         file_id = self.editor._get_existing_id(self.old_id, self.new_id, path)
         base_ie = self.editor.old_inventory[base_file_id]
         is_symlink = (base_ie.kind == 'symlink')
-        file_data = self.editor._text_cache.get((base_file_id, base_revid))
-        if file_data is None: # Not present in cache
-            record = self._get_record_stream(base_file_id, base_revid)
-            file_data = record.get_bytes_as('chunked')
+        file_data = self.editor._get_chunked(base_file_id, base_revid)
         if file_id == base_file_id:
             file_parents = [base_revid]
             old_path = path
@@ -538,14 +519,14 @@ class DirectoryRevisionBuildEditor(DirectoryBuildEditor):
             old_path = None
             file_parents = []
             self._delete_entry(path, base_revnum)
-        return FileRevisionBuildEditor(self.editor, old_path, path, file_id, 
-            self.new_id, file_parents, file_data, is_symlink=is_symlink)
+        return FileRevisionBuildEditor(self.editor, old_path, path, file_id, self.new_id, 
+            file_parents, file_data, is_symlink=is_symlink)
 
 
 class FileRevisionBuildEditor(FileBuildEditor):
 
-    def __init__(self, editor, old_path, path, file_id, parent_file_id, 
-        file_parents=[], data="", is_symlink=False):
+    def __init__(self, editor, old_path, path, file_id, parent_file_id, file_parents=[], 
+        data=[], is_symlink=False):
         super(FileRevisionBuildEditor, self).__init__(editor, path)
         self.old_path = old_path
         self.file_id = file_id
@@ -559,7 +540,7 @@ class FileRevisionBuildEditor(FileBuildEditor):
     def _apply_textdelta(self, base_checksum=None):
         actual_checksum = md5_strings(self.file_data)
         if base_checksum is not None and base_checksum != actual_checksum:
-            raise VersionedFileInvalidChecksum("base checksum mismatch: %r != %r" % (base_checksum, actual_checksum))
+            raise VersionedFileInvalidChecksum("base checksum mismatch: %r != %r in %s (%s:%d)" % (base_checksum, actual_checksum, self.editor.revid, self.editor.branch_path, self.editor.revnum))
         self.file_stream = StringIO()
         return apply_txdelta_handler_chunks(self.file_data, self.file_stream)
 
@@ -577,7 +558,7 @@ class FileRevisionBuildEditor(FileBuildEditor):
         text_parents = self.editor._get_text_parents(self.path)
         if text_parents is None:
             text_parents = self.file_parents
-        parent_keys = [(self.file_id, revid) for revid in text_parents]
+        parent_keys = tuple([(self.file_id, revid) for revid in text_parents])
         parent_texts = {}
         if parent_keys:
             parent_text = self.editor._content_cache.get(parent_keys[0], None)
@@ -656,28 +637,49 @@ class RevisionBuildEditor(DeltaBuildEditor):
         self._inv_delta = []
         self._deleted = set()
         self._explicitly_deleted = set()
+        self.inventory = None
         super(RevisionBuildEditor, self).__init__(revmeta, mapping)
 
+    def get_old_ie(self, path, revnum):
+        # Find the ancestor of self.revmeta with revnum revnum
+        last_revmeta = None
+        for revmeta, mapping in self.source._iter_reverse_revmeta_mapping_history(
+            self.revmeta.branch_path, self.revmeta.revnum, revnum, self.mapping):
+            last_revmeta = revmeta
+        assert last_revmeta is not None and last_revmeta.revnum == revnum
+        revid = last_revmeta.get_revision_id(mapping)
+        # TODO: Use InterRepository._get_inventory() for better performance, 
+        # as it does (some) caching ?
+        inv = self.target.get_inventory(revid)
+        file_id = inv.path2id(path)
+        return inv[file_id]
+
+    def _get_record(self, file_id, revision_id):
+        return self.texts.get_record_stream([(file_id, revision_id)], 'unordered', True).next()
+
+    def _get_chunked(self, file_id, revision_id):
+        file_data = self._text_cache.get((file_id, revision_id))
+        if file_data is not None: 
+            return file_data
+        record = self._get_record(file_id, revision_id)
+        if record.storage_kind == 'absent':
+            raise RevisionNotPresent(revision_id, file_id)
+        return record.get_bytes_as('chunked')
+
     def _finish_commit(self):
         rev = self.revmeta.get_revision(self.mapping)
         # Escaping the commit message is really the task of the serialiser
-        rev.message = escape_commit_message(rev.message)
-        if getattr(self.target, "add_inventory_by_delta", None) is not None:
-            try:
-                basis_id = rev.parent_ids[0]
-            except IndexError:
-                basis_id = NULL_REVISION
-            rev.inventory_sha1, self.inventory = self.target.add_inventory_by_delta(basis_id,
-                      self._inv_delta, rev.revision_id,
-                      [r for r in rev.parent_ids if r in self.target.has_revisions(rev.parent_ids)])
-        else:
-            self.inventory = self.old_inventory
-            if "check" in debug.debug_flags:
-                check_inventory_delta(self._inv_delta)
-            self.inventory.apply_delta(self._inv_delta)
-            self.inventory.revision_id = rev.revision_id
-            rev.inventory_sha1 = self.target.add_inventory(rev.revision_id, 
-                    self.inventory, rev.parent_ids)
+        if rev.message is not None:
+            if getattr(self.target._serializer, "squashes_xml_invalid_characters", True):
+                from bzrlib.xml_serializer import escape_invalid_chars
+                rev.message, num_replaced = escape_invalid_chars(rev.message)
+        try:
+            basis_id = rev.parent_ids[0]
+        except IndexError:
+            basis_id = NULL_REVISION
+        rev.inventory_sha1, self.inventory = self.target.add_inventory_by_delta(basis_id,
+                  self._inv_delta, rev.revision_id,
+                  tuple([r for r in rev.parent_ids if r in self.target.has_revisions(rev.parent_ids)]))
         self.target.add_revision(self.revid, rev)
 
         # Only fetch signature if it's cheap
@@ -736,9 +738,7 @@ class RevisionBuildEditor(DeltaBuildEditor):
             return self._id_map
 
         local_changes = get_local_changes(self.revmeta.get_paths(), 
-                    self.revmeta.branch_path, self.mapping,
-                    self.source.get_layout(),
-                    self.source.generate_revision_id)
+                    self.revmeta.branch_path)
         self._id_map = self.source.fileid_map.get_idmap_delta(local_changes, self.revmeta, 
             self.mapping)
 
@@ -761,7 +761,7 @@ class RevisionBuildEditor(DeltaBuildEditor):
             except KeyError:
                 raise FileIdMapIncomplete(basename, self.old_inventory.id2path(parent_id), self.revmeta) 
         else:
-            ret = parent_id_basename_index.iteritems([(parent_id, basename)])
+            ret = parent_id_basename_index.iteritems([(parent_id or '', basename.encode("utf-8"))])
             try:
                 return ret.next()[1]
             except IndexError:
@@ -878,16 +878,13 @@ class TreeDeltaBuildEditor(DeltaBuildEditor):
 
     def _was_renamed(self, path):
         fileid = self._get_new_id(path)
-        for fid, _ in self._parent_idmap.values():
-            if fileid == fid:
-                return True
-        return False
+        return self._parent_idmap.has_fileid(fid)
 
     def _get_old_id(self, path):
-        return idmap_lookup(self._parent_idmap, self.mapping, path)[0]
+        return self._parent_idmap.lookup(self.mapping, path)[0]
 
     def _get_new_id(self, path):
-        return idmap_lookup(self._idmap, self.mapping, path)[0]
+        return self._idmap.lookup(self.mapping, path)[0]
 
 
 @convert_svn_error
@@ -917,6 +914,7 @@ class FetchRevisionFinder(object):
         self.needed = deque()
         self.checked = set()
         self.extra = list()
+        self._check_present_interval = 1
 
     def get_missing(self):
         """Return the revisions that should be fetched, children before parents.
@@ -930,15 +928,14 @@ class FetchRevisionFinder(object):
         map = {}
         for revmeta, mapping in revmetas:
             try:
-                if revmeta.is_hidden(mapping):
-                    continue
                 map[revmeta, mapping] = revmeta.get_revision_id(mapping)
             except SubversionException, (_, ERR_FS_NOT_DIRECTORY):
                 continue
         present_revids = self.target.has_revisions(map.values())
         return [k for k in revmetas if k in map and map[k] not in present_revids]
 
-    def find_iter_revisions(self, iter, master_mapping, prefix=None, heads=None, pb=None):
+    def find_iter_revisions(self, iter, master_mapping, needs_manual_check,
+                            heads=None, pb=None):
         """Find revisions to fetch based on an iterator over available revmetas.
         
         :param iter: Iterator over RevisionMetadata objects
@@ -962,15 +959,15 @@ class FetchRevisionFinder(object):
                 if (m != master_mapping and 
                     not m.is_branch_or_tag(revmeta.branch_path)):
                     continue
-                if prefix is not None:
-                    lhs_parent_revmeta = revmeta.get_direct_lhs_parent_revmeta()
-                    if lhs_parent_revmeta is not None and not changes.path_is_child(prefix, lhs_parent_revmeta.branch_path):
-                        # Parent branch path is outside of prefix; we need to 
-                        # check manually
-                        self.needed.extend(self.find_mainline(lhs_parent_revmeta.get_foreign_revid(), lhsm))
+                lhs_parent_revmeta = revmeta.get_lhs_parent_revmeta(m)
+                if (lhs_parent_revmeta is not None and 
+                    needs_manual_check(lhs_parent_revmeta)):
+                    self.needed.extend(self.find_mainline(
+                        lhs_parent_revmeta.get_foreign_revid(), lhsm))
                 if lhsm != master_mapping or heads is not None:
-                    needed_mappings[revmeta.get_direct_lhs_parent_revmeta()].add(lhsm)
-                needs_checking.append((revmeta, m))
+                    needed_mappings[lhs_parent_revmeta].add(lhsm)
+                if not revmeta.is_hidden(m):
+                    needs_checking.append((revmeta, m))
         self.needed.extend(reversed(self.check_revmetas(needs_checking)))
 
     def find_all(self, mapping, pb=None):
@@ -980,9 +977,15 @@ class FetchRevisionFinder(object):
         :return: List with revmeta, mapping tuples to fetch
         """
         from_revnum = self.source.get_latest_revnum()
-        return self.find_iter_revisions(self.source._revmeta_provider.iter_all_revisions(self.source.get_layout(), check_unusual_path=mapping.is_branch_or_tag, from_revnum=from_revnum, pb=pb), mapping)
+        return self.find_iter_revisions(
+            self.source._revmeta_provider.iter_all_revisions(
+                self.source.get_layout(), 
+                check_unusual_path=mapping.is_branch_or_tag,
+                from_revnum=from_revnum, pb=pb), mapping,
+            lambda x: False)
 
-    def find_mainline(self, foreign_revid, mapping, project=None, find_ghosts=False, pb=None):
+    def find_mainline(self, foreign_revid, mapping, find_ghosts=False,
+                      pb=None):
         if (foreign_revid, mapping) in self.checked:
             return []
         revmetas = deque()
@@ -998,7 +1001,8 @@ class FetchRevisionFinder(object):
                 # This revision (and its ancestry) has already been checked
                 break
             needs_checking.append((revmeta, mapping))
-            if not find_ghosts and not self.target_is_empty and len(needs_checking) == CHECK_PRESENT_INTERVAL:
+            if (not find_ghosts and not self.target_is_empty and 
+                len(needs_checking) == self._check_present_interval):
                 missing_revmetas = self.check_revmetas(needs_checking)
                 for r in missing_revmetas:
                     revmetas.appendleft(r)
@@ -1006,6 +1010,7 @@ class FetchRevisionFinder(object):
                 needs_checking = []
                 if done:
                     break
+                self._check_present_interval = min(self._check_present_interval*2, MAX_CHECK_PRESENT_INTERVAL)
             self.checked.add((revmeta.get_foreign_revid(), mapping))
         for r in self.check_revmetas(needs_checking):
             revmetas.appendleft(r)
@@ -1013,26 +1018,26 @@ class FetchRevisionFinder(object):
         for revmeta, mapping in revmetas:
             for p in revmeta.get_rhs_parents(mapping):
                 try:
-                    foreign_revid, rhs_mapping = self.source.lookup_revision_id(p, project=project)
+                    foreign_revid, rhs_mapping = self.source.lookup_revision_id(p, foreign_sibling=revmeta.get_foreign_revid())
                 except NoSuchRevision:
                     pass # Ghost
                 else:
-                    self.extra.append((foreign_revid, project, rhs_mapping))
+                    self.extra.append((foreign_revid, rhs_mapping))
         return revmetas
 
-    def find_until(self, foreign_revid, mapping, find_ghosts=False, pb=None,
-                    project=None):
+    def find_until(self, foreign_revid, mapping, find_ghosts=False, 
+                   pb=None):
         """Find all missing revisions until revision_id
 
         :param revision_id: Stop revision
         :param find_ghosts: Find ghosts
         :return: List with revmeta, mapping tuples to fetch
         """
-        self.extra.append((foreign_revid, project, mapping))
+        self.extra.append((foreign_revid, mapping))
         while len(self.extra) > 0:
-            foreign_revid, project, mapping = self.extra.pop()
+            foreign_revid, mapping = self.extra.pop()
             self.needed.extend(self.find_mainline(foreign_revid, mapping, 
-                project, find_ghosts=find_ghosts, pb=pb))
+                find_ghosts=find_ghosts, pb=pb))
 
 
 class InterFromSvnRepository(InterRepository):
@@ -1040,8 +1045,6 @@ class InterFromSvnRepository(InterRepository):
 
     _matching_repo_format = SvnRepositoryFormat()
 
-    _supports_revmetas = True
-
     @staticmethod
     def _get_repo_format_to_test():
         return None
@@ -1052,11 +1055,14 @@ class InterFromSvnRepository(InterRepository):
             return sum(map(len, lines))
         self._text_cache = lru_cache.LRUSizeCache(TEXT_CACHE_SIZE,
                                                   compute_size=lines_to_size)
-        # TODO: it would be nice to get rid of this extra cache, we don't (yet)
-        #       because the cached content objects are supposed to be opaque
+        # TODO: it would be nice to get rid of this extra cache, we don't
+        # (yet) because the cached content objects are supposed to be opaque
         self._content_cache = lru_cache.LRUCache(TEXT_CACHE_SIZE /
                                                  (1*1024*1024))
 
+        self._use_replay_range = self.source.transport.has_capability("partial-replay") and False
+        self._use_replay = self.source.transport.has_capability("partial-replay") and False
+
     def copy_content(self, revision_id=None, pb=None):
         """See InterRepository.copy_content."""
         self.fetch(revision_id, pb, find_ghosts=False)
@@ -1075,6 +1081,14 @@ class InterFromSvnRepository(InterRepository):
             assert self.target.has_revision(revid)
         return self.target.get_inventory(revid)
 
+    def _inconsistent_lhs_parent(self, revid, stored_lhs_parent_revid, 
+            found_lhs_parent_revid):
+        trace.warning(
+            "Recorded left hand side parent %s for revision "
+            "%s does not match actual left hand side parent "
+            "%s.", revid, stored_lhs_parent_revid, found_lhs_parent_revid)
+        self.fetch(found_lhs_parent_revid)
+
     def _get_editor(self, revmeta, mapping):
         revid = revmeta.get_revision_id(mapping)
         assert revid is not None
@@ -1086,7 +1100,9 @@ class InterFromSvnRepository(InterRepository):
             lhs_parent_revmeta = revmeta.get_lhs_parent_revmeta(mapping)
             expected_lhs_parent_revid = revmeta.get_implicit_lhs_parent_revid(mapping)
             if revmeta.get_stored_lhs_parent_revid(mapping) not in (None, expected_lhs_parent_revid):
-                raise InconsistentLHSParent(revmeta, revmeta.get_stored_lhs_parent_revid(mapping), expected_lhs_parent_revid)
+                self._inconsistent_lhs_parent(revmeta.get_revision_id(mapping),
+                    revmeta.get_stored_lhs_parent_revid(mapping), 
+                    expected_lhs_parent_revid)
             raise
 
     def _fetch_revision_switch(self, editor, revmeta, parent_revmeta):
@@ -1132,7 +1148,8 @@ class InterFromSvnRepository(InterRepository):
         assert revmeta.revnum >= 0
         conn = self.source.transport.get_connection(revmeta.branch_path)
         try:
-            conn.replay(revmeta.revnum, 0, editor_strip_prefix(editor, revmeta.branch_path), True)
+            conn.replay(revmeta.revnum, 0, editor_strip_prefix(editor, revmeta.branch_path), 
+                True)
         finally:
             if not conn.busy:
                 self.source.transport.add_connection(conn)
@@ -1147,23 +1164,22 @@ class InterFromSvnRepository(InterRepository):
         accidental_file_revs = set()
         self._prev_inv = None
 
-        for num, (revmeta, mapping) in enumerate(revs):
-            try:
-                revid = revmeta.get_revision_id(mapping)
-            except SubversionException, (_, ERR_FS_NOT_DIRECTORY):
-                continue
-            assert revid != NULL_REVISION
-            if pb is not None:
-                pb.update('copying revision', num, len(revs))
+        self.target.start_write_group()
+        try:
+            for num, (revmeta, mapping) in enumerate(revs):
+                try:
+                    revid = revmeta.get_revision_id(mapping)
+                except SubversionException, (_, ERR_FS_NOT_DIRECTORY):
+                    continue
+                assert revid != NULL_REVISION
+                if pb is not None:
+                    pb.update('copying revision', num, len(revs))
 
-            parent_revmeta = revmeta.get_lhs_parent_revmeta(mapping)
-            if parent_revmeta in accidental_file_revs:
-                accidental_file_revs.add(revmeta)
-                continue
+                parent_revmeta = revmeta.get_lhs_parent_revmeta(mapping)
+                if parent_revmeta in accidental_file_revs:
+                    accidental_file_revs.add(revmeta)
+                    continue
 
-            if not self.target.is_in_write_group():
-                self.target.start_write_group()
-            try:
                 editor = self._get_editor(revmeta, mapping)
                 try:
                     if use_replay:
@@ -1180,20 +1196,13 @@ class InterFromSvnRepository(InterRepository):
                 except:
                     editor.abort()
                     raise
-            except:
-                if self.target.is_in_write_group():
-                    self.target.abort_write_group()
-                raise
-            if num % FETCH_COMMIT_WRITE_SIZE == 0:
-                self.target.commit_write_group()
-
-            self._prev_inv = editor.inventory
-            assert self._prev_inv.revision_id == revid
-        if self.target.is_in_write_group():
+                self._prev_inv = editor.inventory
+                assert self._prev_inv.revision_id == revid
+        finally:
             self.target.commit_write_group()
 
     def fetch(self, revision_id=None, pb=None, find_ghosts=False, 
-              needed=None, mapping=None, fetch_spec=None):
+              needed=None, mapping=None, project=None, fetch_spec=None):
         """Fetch revisions. """
         if revision_id == NULL_REVISION:
             return
@@ -1202,9 +1211,6 @@ class InterFromSvnRepository(InterRepository):
         if pb:
             pb.update("fetch phase", 0, 2)
 
-        use_replay_range = self.source.transport.has_capability("partial-replay") and False
-        use_replay = self.source.transport.has_capability("partial-replay") and False
-
         # Loop over all the revnums until revision_id
         # (or youngest_revnum) and call self.target.add_revision() 
         # or self.target.add_inventory() each time
@@ -1218,14 +1224,16 @@ class InterFromSvnRepository(InterRepository):
                                                      target_is_empty)
                 if needed is None:
                     if revision_id is not None:
-                        foreign_revid, mapping = self.source.lookup_revision_id(revision_id)
+                        foreign_revid, mapping = self.source.lookup_revision_id(revision_id, project=project)
                         revisionfinder.find_until(foreign_revid,
-                            mapping, find_ghosts, pb=nested_pb)
+                            mapping, find_ghosts=find_ghosts,
+                            pb=nested_pb)
                     elif fetch_spec is not None:
                         for head in fetch_spec.heads:
-                            foreign_revid, mapping = self.source.lookup_revision_id(head)
+                            foreign_revid, mapping = self.source.lookup_revision_id(head, project=project)
                             revisionfinder.find_until(foreign_revid,
-                                mapping, find_ghosts, pb=nested_pb)
+                                mapping, find_ghosts=find_ghosts,
+                                pb=nested_pb)
                     else:
                         revisionfinder.find_all(self.source.get_mapping(), pb=nested_pb)
                     needed = revisionfinder.get_missing()
@@ -1245,10 +1253,11 @@ class InterFromSvnRepository(InterRepository):
             else:
                 nested_pb = None
             try:
-                if use_replay_range:
+                if self._use_replay_range:
                     self._fetch_revision_chunks(needed, pb)
                 else:
-                    self._fetch_revisions(needed, pb, use_replay=use_replay)
+                    self._fetch_revisions(needed, pb,
+                        use_replay=self._use_replay)
             finally:
                 if nested_pb is not None:
                     nested_pb.finished()
@@ -1260,8 +1269,6 @@ class InterFromSvnRepository(InterRepository):
                 # itself was roundtripped.
                 raise NoSuchRevision(self.source, revision_id)
         finally:
-            if self.target.is_in_write_group():
-                self.target.abort_write_group()
             self.target.unlock()
 
     def _fetch_revision_chunks(self, revs, pb=None):
@@ -1285,9 +1292,7 @@ class InterFromSvnRepository(InterRepository):
         finally:
             pb.finished()
 
-        if not self.target.is_in_write_group():
-            self.target.start_write_group()
-
+        self.target.start_write_group()
         try:
             def cmprange((ak, av),(bk, bv)):
                 return cmp(av[0], bv[0])
@@ -1302,9 +1307,11 @@ class InterFromSvnRepository(InterRepository):
                             revmeta = r
                             break
                     revmeta._revprops = revprops
-                    return editor_strip_prefix(self._get_editor(revmeta, mapping), revmeta.branch_path, self._text_cache, self._content_cache)
+                    return editor_strip_prefix(self._get_editor(revmeta, mapping), 
+                        revmeta.branch_path)
 
                 def revfinish(revision, revprops, editor):
+                    assert editor.inventory is not None
                     self._prev_inv = editor.inventory
 
                 conn = self.source.transport.get_connection(revmetas[-1].branch_path)
@@ -1313,12 +1320,8 @@ class InterFromSvnRepository(InterRepository):
                 finally:
                     if not conn.busy:
                         self.source.transport.add_connection(conn)
-
-                if i % FETCH_COMMIT_WRITE_SIZE == 0:
-                    self.target.commit_write_group()
         finally:
-            if self.target.is_in_write_group():
-                self.target.commit_write_group()
+            self.target.commit_write_group()
 
     @staticmethod
     def is_compatible(source, target):
