@@ -13,20 +13,37 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Generation of file-ids."""
 
-from collections import defaultdict
+
+try:
+    from collections import defaultdict
+except ImportError:
+    from bzrlib.plugins.svn.pycompat import defaultdict
+
 import urllib
 
 from bzrlib import (
     ui,
     urlutils,
     )
-from bzrlib.errors import RevisionNotPresent
-from bzrlib.knit import make_file_factory
-from bzrlib.revision import NULL_REVISION
-from bzrlib.trace import mutter
-from bzrlib.versionedfile import ConstantMapper
+from bzrlib.errors import (
+    RevisionNotPresent,
+    )
+from bzrlib.knit import (
+    make_file_factory,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
+from bzrlib.trace import (
+    mutter,
+    )
+from bzrlib.versionedfile import (
+    ConstantMapper,
+    )
 
 from bzrlib.plugins.svn import (
     changes, 
@@ -46,16 +63,16 @@ from bzrlib.plugins.svn import (
 # = rev 2
 #    A /bla (from /foo:1)
 
-def idmap_lookup(idmap, mapping, path):
+def idmap_lookup(get, mapping, path):
     """Lookup a path in an idmap.
 
-    :param idmap: The idmap to look up in.
+    :param get: Function to access idmap
     :param mapping: Mapping
     :param path: Path to look up
     :return: Tuple with file id and text revision
     """
     try:
-        return idmap[path]
+        return get(path)
     except KeyError:
         base = path
         while base != "":
@@ -63,13 +80,15 @@ def idmap_lookup(idmap, mapping, path):
                 base = base.rsplit("/", 1)[0]
             else:
                 base = u""
-            if base in idmap:
-                create_revid = idmap[base][2]
+            try:
+                create_revid = get(base)[2]
                 if create_revid is None:
                     raise AssertionError("Inconsistency; child %s appeared while parent was never copied" % path)
                 return (mapping.generate_file_id(create_revid, path),
                         mapping.revision_id_foreign_to_bzr(create_revid),
                         create_revid)
+            except:
+                pass
         raise KeyError("Unable to determine file id for %r" % path)
 
 
@@ -89,7 +108,7 @@ def idmap_reverse_lookup(idmap, mapping, fileid):
     else:
         uuid = None
     # Unfortunately, the map is the other way around
-    for k in sorted(idmap.keys()):
+    for k in sorted(idmap.iterkeys()):
         (v, ck, child_create_foreign_revid) = idmap[k]
         if v == fileid:
             return k
@@ -114,54 +133,21 @@ def determine_text_revisions(changes, default_revid, specific_revids):
     """
     ret = {}
     ret.update(specific_revids)
-    for p, data in changes.iteritems():
+    for p, (action, copy_from) in changes.iteritems():
         assert isinstance(p, unicode)
         # The root changes often because of file properties, so we don't 
         # consider it really changing.
-        if data[0] in ('A', 'R', 'M') and p not in ret and p != u"":
+        if action in ('A', 'R', 'M') and p not in ret and p != u"":
             ret[p] = default_revid
     return ret
 
 
-def apply_idmap_delta(map, text_revisions, delta, changes, default_revid, 
-                      mapping, foreign_revid):
-    """Update a file id map.
-
-    :param map: Existing file id map that needs to be updated
-    :param text_revisions: Text revisions for the map
-    :param delta: Id map delta.
-    :param changes: Changes for the revision in question.
-    """
-    for p, data in changes.iteritems():
-        if data[0] in ('D', 'R'):
-            for xp in map.keys():
-                if (p == xp or xp.startswith(u"%s/" % p)) and not xp in delta:
-                    del map[xp]
-
-    for x in sorted(text_revisions.keys() + delta.keys()):
-        assert isinstance(x, unicode)
-        if x in delta and (not x in map or map[x][0] != delta[x]):
-            if x in changes and changes[x][1] is not None:
-                # if this was a copy from somewhere else there can be 
-                # implicit children
-                child_create_revid = foreign_revid
-            else:
-                child_create_revid = None
-            map[x] = (delta[x], text_revisions.get(x) or default_revid, child_create_revid)
-        else:
-            prev_entry = idmap_lookup(map, mapping, x)
-            map[x] = (prev_entry[0], text_revisions.get(x) or default_revid, prev_entry[2])
-
-
-def get_local_changes(paths, branch, mapping, layout, generate_revid):
+def get_local_changes(paths, branch):
     """Obtain all of the changes relative to a particular path
     (usually a branch path).
 
     :param paths: Changes
     :param branch: Path under which to select changes
-    :param mapping: Mapping to use to determine what are valid branch paths
-    :param layout: Layout to use 
-    :param generate_revid: Function for generating revision id from svn revnum
     """
     new_paths = {}
     for p in sorted(paths.keys(), reverse=False):
@@ -170,19 +156,13 @@ def get_local_changes(paths, branch, mapping, layout, generate_revid):
         data = paths[p]
         new_p = p[len(branch):].strip("/")
         if data[1] is not None:
-            try:
-                (pt, proj, cbp, crp) = layout.parse(data[1])
-
-                # Branch copy
-                if (crp == "" and new_p == ""):
-                    data = ('M', None, None)
-                else:
-                    data = (data[0], crp, generate_revid(
-                                  data[2], cbp, mapping))
-            except errors.NotSvnBranchPath:
-                # Copied from outside of a known branch
-                data = (data[0], data[1], None)
-
+            # Branch copy
+            if new_p == "":
+                data = ('M', None)
+            else:
+                data = (data[0], (data[1], data[2]))
+        else:
+            data = (data[0], None)
         new_paths[new_p.decode("utf-8")] = data
     return new_paths
 
@@ -196,15 +176,90 @@ def simple_apply_changes(new_file_id, changes):
     Does not track renames. """
     delta = {}
     for p in sorted(changes.keys(), reverse=False):
-        data = changes[p]
+        (action, copy_from) = changes[p]
         assert isinstance(p, unicode)
         # Only generate new file ids for root if it's new
-        if data[0] in ('A', 'R') and (p != u"" or data[1] is None):
+        if action in ('A', 'R') and (p != u"" or copy_from is None):
             delta[p] = new_file_id(p)
     return delta 
 
 
 class FileIdMap(object):
+
+    def has_fileid(self, fileid):
+        raise NotImplementedError(self.has_fileid)
+
+    def as_dict(self):
+        raise NotImplementedError(self.as_dict)
+
+    def lookup(self, mapping, path):
+        raise NotImplementedError(self.lookup)
+
+    def reverse_lookup(self, mapping, fileid):
+        raise NotImplementedError(self.reverse_lookup)
+
+    def apply_delta(self, text_revisions, delta, changes, default_revid, 
+                      mapping, foreign_revid):
+        raise NotImplementedError(self.apply_delta)
+
+
+class DictFileIdMap(FileIdMap):
+
+    def __init__(self, data):
+        self.data = data
+
+    def apply_delta(self, text_revisions, delta, changes, default_revid, 
+                      mapping, foreign_revid):
+        """Update a file id map.
+
+        :param text_revisions: Text revisions for the map
+        :param delta: Id map delta.
+        :param changes: Changes for the revision in question.
+        """
+        assert "" in self.data or self.data == {}, "'' missing in %r" % self.data
+        for p, (action, copy_from) in changes.iteritems():
+            if action in ('D', 'R'):
+                for xp in self.data.keys():
+                    if (p == xp or xp.startswith(u"%s/" % p)) and not xp in delta:
+                        del self.data[xp]
+
+        for x in sorted(text_revisions.keys() + delta.keys()):
+            assert isinstance(x, unicode)
+            if x in delta and (not x in self.data or self.data[x][0] != delta[x]):
+                if x in changes and changes[x][1] is not None:
+                    # if this was a copy from somewhere else there can be 
+                    # implicit children
+                    child_create_revid = foreign_revid
+                else:
+                    child_create_revid = None
+                self.data[x] = (delta[x], text_revisions.get(x) or default_revid, child_create_revid)
+            else:
+                try:
+                    prev_entry = self.lookup(mapping, x)
+                except KeyError:
+                    raise AssertionError("Unable to find old fileid for %s in %r" % (x, foreign_revid))
+                self.data[x] = (prev_entry[0], text_revisions.get(x) or default_revid, prev_entry[2])
+
+        if not "" in self.data:
+            raise AssertionError("root no longer exists after %r in %r" % (foreign_revid, self.data))
+
+    def has_fileid(self, fileid):
+        for fid, _ in self.data.itervalues():
+            if fid == fileid:
+                return True
+        return False
+
+    def as_dict(self):
+        return self.data
+
+    def lookup(self, mapping, path):
+        return idmap_lookup(self.data.__getitem__, mapping, path)
+
+    def reverse_lookup(self, mapping, fileid):
+        return idmap_reverse_lookup(self.data, mapping, fileid)
+
+
+class FileIdMapStore(object):
     """File id store. 
 
     Keeps a map
@@ -218,8 +273,8 @@ class FileIdMap(object):
     def get_idmap_delta(self, changes, revmeta, mapping):
         """Change file id map to incorporate specified changes.
 
+        :param changes: Dictionary with (local) changes
         :param revmeta: RevisionMetadata object for revision with changes
-        :param renames: List of renames (known file ids for particular paths)
         :param mapping: Mapping
         """
         foreign_revid = revmeta.get_foreign_revid()
@@ -231,33 +286,27 @@ class FileIdMap(object):
         return idmap
 
     def update_idmap(self, map, revmeta, mapping):
-        local_changes = get_local_changes(revmeta.get_paths(), 
-                    revmeta.branch_path, mapping,
-                    self.repos.get_layout(),
-                    self.repos.generate_revision_id)
+        local_changes = get_local_changes(revmeta.get_paths(), revmeta.branch_path)
         idmap = self.get_idmap_delta(local_changes, revmeta, mapping)
         revid = revmeta.get_revision_id(mapping)
         text_revisions = determine_text_revisions(local_changes, revid, 
                 revmeta.get_text_revisions(mapping))
-        apply_idmap_delta(map, text_revisions, idmap, local_changes, revid,
+        map.apply_delta(text_revisions, idmap, local_changes, revid,
                           mapping, revmeta.get_foreign_revid())
 
     def get_map(self, foreign_revid, mapping):
         """Make sure the map is up to date until revnum."""
         (uuid, branch, revnum) = foreign_revid
-        # First, find the last cached map
-        if revnum == 0:
-            assert branch == ""
-            return {"": (mapping.generate_file_id(foreign_revid, u""), 
-                self.repos.lookup_foreign_revision_id((uuid, "", 0), mapping), 
-                None)}
-
         todo = []
         next_parent_revs = []
         if mapping.is_branch(""):
-            map = {u"": (mapping.generate_file_id((uuid, "", 0), u""), NULL_REVISION, None)}
+            map = DictFileIdMap({u"": (mapping.generate_file_id((uuid, "", 0), u""), mapping.revision_id_foreign_to_bzr((uuid, "", 0)), None)})
         else:
-            map = {}
+            map = DictFileIdMap({})
+
+        if revnum == 0:
+            assert branch == ""
+            return map
 
         # No history -> empty map
         todo = list(self.repos._iter_reverse_revmeta_mapping_history(branch, revnum, to_revnum=0, mapping=mapping))
@@ -273,6 +322,7 @@ class FileIdMap(object):
         return map
 
 
+
 class FileIdMapCache(object):
 
     def __init__(self, cache_transport):
@@ -319,21 +369,19 @@ class FileIdMapCache(object):
         return map
 
 
-class CachingFileIdMap(object):
+class CachingFileIdMapStore(object):
     """A file id map that uses a cache."""
 
-    def __init__(self, cache_transport, actual):
-        self.cache = FileIdMapCache(cache_transport)
+    def __init__(self, cache, actual):
+        self.cache = cache
         self.actual = actual
         self.repos = actual.repos
         self.get_idmap_delta = actual.get_idmap_delta
 
     def get_map(self, (uuid, branch, revnum), mapping):
         """Make sure the map is up to date until revnum."""
-        # First, find the last cached map
         if revnum == 0:
             return self.actual.get_map((uuid, branch, revnum), mapping)
-
         todo = []
         next_parent_revs = []
 
@@ -346,7 +394,7 @@ class CachingFileIdMap(object):
                     continue
                 revid = revmeta.get_revision_id(mapping)
                 try:
-                    map = self.cache.load(revid)
+                    map = DictFileIdMap(self.cache.load(revid))
                     # found the nearest cached map
                     next_parent_revs = [revid]
                     break
@@ -361,9 +409,9 @@ class CachingFileIdMap(object):
 
         if len(next_parent_revs) == 0:
             if mapping.is_branch(""):
-                map = {u"": (mapping.generate_file_id((uuid, "", 0), u""), NULL_REVISION, None)}
+                map = DictFileIdMap({u"": (mapping.generate_file_id((uuid, "", 0), u""), NULL_REVISION, None)})
             else:
-                map = {}
+                map = DictFileIdMap({})
 
         pb = ui.ui_factory.nested_progress_bar()
 
@@ -374,7 +422,7 @@ class CachingFileIdMap(object):
                 self.actual.update_idmap(map, revmeta, mapping)
                 parent_revs = next_parent_revs
                 if revnum % FILEID_MAP_SAVE_INTERVAL == 0 or revnum == revmeta.revnum:
-                    self.cache.save(revid, parent_revs, map)
+                    self.cache.save(revid, parent_revs, map.as_dict())
                 next_parent_revs = [revid]
         finally:
             pb.finished()
