@@ -13,7 +13,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from functools import partial
+try:
+    from functools import partial
+except ImportError:
+    from bzrlib.plugins.svn.pycompat import partial
+
+try:
+    all
+except NameError:
+    from bzrlib.plugins.svn.pycompat import all
+
 from subvertpy import NODE_DIR
 
 from bzrlib import urlutils
@@ -68,7 +77,7 @@ class TrunkLayout(RepositoryLayout):
         
         :param project: Name of the project.
         """
-        return False
+        return True
 
     def get_branch_path(self, name, project=""):
         """Return the path at which the branch with specified name should be found.
@@ -114,7 +123,7 @@ class TrunkLayout(RepositoryLayout):
     def get_branches(self, repository, revnum, project=None, pb=None):
         """Retrieve a list of paths that refer to branches in a specific revision.
 
-        :result: Iterator over tuples with (project, branch path)
+        :result: Iterator over tuples with (project, branch path, has_props)
         """
         return get_root_paths(repository, 
              [self._add_project(x, project) for x in "branches/*", "trunk"], 
@@ -143,6 +152,8 @@ class TrunkLayout(RepositoryLayout):
 TrunkLayout0 = partial(TrunkLayout, 0)
 TrunkLayout1 = partial(TrunkLayout, 1)
 TrunkLayout2 = partial(TrunkLayout, 2)
+TrunkLayout3 = partial(TrunkLayout, 3)
+TrunkLayout4 = partial(TrunkLayout, 4)
 
 
 class RootLayout(RepositoryLayout):
@@ -207,7 +218,7 @@ class RootLayout(RepositoryLayout):
 
         :result: Iterator over tuples with (project, branch path)
         """
-        return [("", "", "trunk")]
+        return [("", "", "trunk", None)]
 
     def get_tags(self, repository, revnum, project=None, pb=None):
         """Retrieve a list of paths that refer to tags in a specific revision.
@@ -300,6 +311,18 @@ class CustomLayout(RepositoryLayout):
     def __repr__(self):
         return "%s(%r,%r)" % (self.__class__.__name__, self.branches, self.tags)
 
+    def _is_prefix(self, prefixes, path, project=None):
+        for branch in prefixes:
+            if branch.startswith("%s/" % path):
+                return True
+        return False
+
+    def is_branch_parent(self, path, project=None):
+        return self._is_prefix(self.branches, path, project)
+
+    def is_tag_parent(self, path, project=None):
+        return self._is_prefix(self.tags, path, project)
+
 
 class WildcardLayout(RepositoryLayout):
 
