@@ -13,47 +13,56 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Cache of the Subversion history log."""
 
+
 import subvertpy
 
 from bzrlib import (
+    debug,
     ui,
+    trace,
     urlutils,
     )
-from bzrlib.errors import NoSuchRevision
+from bzrlib.errors import (
+    NoSuchRevision,
+    )
+
+from bzrlib.plugins.svn import (
+    changes,
+    )
+from bzrlib.plugins.svn.transport import (
+    SvnRaTransport,
+    )
+from bzrlib.plugins.svn.util import (
+    lazy_dict,
+    )
 
-from bzrlib.plugins.svn import changes
-from bzrlib.plugins.svn.cache import CacheTable
-from bzrlib.plugins.svn.transport import SvnRaTransport
-from bzrlib.plugins.svn.util import lazy_dict
 
 # Maximum number of extra revisions to fetch in caching logwalker
 MAX_OVERHEAD_FETCH = 1000
 
-def iter_changes(paths, from_revnum, to_revnum, get_revision_paths, 
-    revprop_list, limit=0, pb=None):
+def iter_changes(prefixes, from_revnum, to_revnum, get_revision_paths, 
+    revprop_list, limit=0):
     ascending = (to_revnum > from_revnum)
 
     revnum = from_revnum
 
-    if paths is None:
+    if prefixes is None:
         path = ""
     else:
-        assert len(paths) == 1
-        path = paths[0].strip("/")
+        assert len(prefixes) == 1
+        path = prefixes[0].strip("/")
 
-    assert from_revnum >= to_revnum or path == ""
+    assert from_revnum >= to_revnum or path == "", \
+            "path: %s, %d >= %d" % (path, from_revnum, to_revnum)
 
     i = 0
 
     while ((not ascending and revnum >= to_revnum) or
            (ascending and revnum <= to_revnum)):
-        if pb is not None:
-            if ascending:
-                pb.update("determining changes", revnum-from_revnum, to_revnum-from_revnum)
-            else:
-                pb.update("determining changes", from_revnum-revnum, from_revnum-to_revnum)
         if not (revnum > 0 or path == ""):
             raise AssertionError("Inconsistent path,revnum: %r,%r" % (revnum, path))
         revpaths = get_revision_paths(revnum)
@@ -63,11 +72,9 @@ def iter_changes(paths, from_revnum, to_revnum, get_revision_paths,
         else:
             next = changes.find_prev_location(revpaths, path, revnum)
 
-        revprops = revprop_list(revnum)
-
         if path == "" or changes.changes_path(revpaths, path, True):
             assert isinstance(revnum, int)
-            yield (revpaths, revnum, revprops)
+            yield (revpaths, revnum, revprop_list(revnum))
             i += 1
             if limit != 0 and i == limit:
                 break
@@ -82,138 +89,59 @@ def iter_changes(paths, from_revnum, to_revnum, get_revision_paths,
         assert isinstance(revnum, int)
 
 
-
-class LogCache(CacheTable):
-    """Log browser cache table manager. The methods of this class
-    encapsulate the SQL commands used by CachingLogWalker to access
-    the log cache tables."""
-    
-    def __init__(self, cache_db=None):
-        CacheTable.__init__(self, cache_db)
-
-    def _create_table(self):
-        self.cachedb.executescript("""
-            create table if not exists changed_path(rev integer, action text, path text, copyfrom_path text, copyfrom_rev integer);
-            create index if not exists path_rev on changed_path(rev);
-            create unique index if not exists path_rev_path on changed_path(rev, path);
-            create unique index if not exists path_rev_path_action on changed_path(rev, path, action);
-
-            create table if not exists revprop(rev integer, name text, value text);
-            create table if not exists revinfo(rev integer, all_revprops int);
-            create index if not exists revprop_rev on revprop(rev);
-            create unique index if not exists revprop_rev_name on revprop(rev, name);
-            create unique index if not exists revinfo_rev on revinfo(rev);
-        """)
-        self._commit_interval = 5000
+class LogCache(object):
+    """Log browser cache. """
     
     def find_latest_change(self, path, revnum):
-        if path == "":
-            return self.cachedb.execute("select max(rev) from changed_path where rev <= ?", (revnum,)).fetchone()[0]
-        return self.cachedb.execute("""
-            select max(rev) from changed_path
-            where rev <= ?
-            and (path=?
-                 or path glob ?
-                 or (? glob (path || '/*')
-                     and action in ('A', 'R')))
-        """, (revnum, path, path + "/*", path)).fetchone()[0]
+        raise NotImplementedError(self.find_latest_change)
 
     def get_revision_paths(self, revnum):
         """Return all history information for a given revision number.
         
         :param revnum: Revision number of revision.
         """
-        result = self.cachedb.execute("select path, action, copyfrom_path, copyfrom_rev from changed_path where rev=?", (revnum,))
-        paths = {}
-        for p, act, cf, cr in result:
-            if cf is not None:
-                cf = cf.encode("utf-8")
-            paths[p.encode("utf-8")] = (act, cf, cr)
-        return paths
-    
-    def insert_path(self, rev, path, action, copyfrom_path=None, copyfrom_rev=-1):
+        raise NotImplementedError(self.get_revision_paths)
+
+    def insert_paths(self, rev, orig_paths):
         """Insert new history information into the cache.
         
         :param rev: Revision number of the revision
-        :param path: Path that was changed
-        :param action: Action on path (single-character)
-        :param copyfrom_path: Optional original path this path was copied from.
-        :param copyfrom_rev: Optional original rev this path was copied from.
+        :param orig_paths: SVN-style changes dictionary
         """
-        assert action in ("A", "R", "D", "M")
-        assert not path.startswith("/")
-        assert isinstance(path, unicode)
-        assert copyfrom_path is None or isinstance(copyfrom_path, unicode)
-        self.cachedb.execute("replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev) values (?, ?, ?, ?, ?)", (rev, path, action, copyfrom_path, copyfrom_rev))
-
+        raise NotImplementedError(self.insert_paths)
+    
     def drop_revprops(self, revnum):
-        self.cachedb.execute("update revinfo set all_revprops = 0 where rev = ?", (revnum,))
+        raise NotImplementedError(self.drop_revprops)
 
     def get_revprops(self, revnum):
         """Retrieve all the cached revision properties.
 
         :param revnum: Revision number of revision to retrieve revprops for.
         """
-        result = self.cachedb.execute("select name, value from revprop where rev = ?", (revnum,))
-        revprops = {}
-        for k, v in result:
-            revprops[k.encode("utf-8")] = v.encode("utf-8")
-        return revprops
+        raise NotImplementedError(self.get_revprops)
 
-    def insert_revprop(self, rev, name, value):
-        """Add a revision property.
-
-        :param rev: Revision number of the revision.
-        :param name: Name of the revision property.
-        :param value: Contents of the revision property.
-        """
-        assert isinstance(name, str) and isinstance(value, str)
-        self.cachedb.execute("replace into revprop (rev, name, value) values (?, ?, ?)", (rev, name.decode("utf-8", "replace"), value.decode("utf-8", "replace")))
-    
-    def insert_revprops(self, revision, revprops):
-        if revprops is None:
-            return
-        for k, v in revprops.iteritems():
-            self.insert_revprop(revision, k, v)
-
-    def has_all_revprops(self, revnum):
-        """Check whether all revprops for a revision have been cached.
-
-        :param revnum: Revision number of the revision.
-        """
-        row = self.cachedb.execute("select all_revprops from revinfo where rev = ?", (revnum,)).fetchone()
-        if row is None:
-            return False
-        return row[0]
-
-    def insert_revinfo(self, rev, all_revprops):
-        """Insert metadata for a revision.
-
-        :param rev: Revision number of the revision.
-        :param all_revprops: Whether or not the full revprops have been stored.
-        """
-        self.cachedb.execute("""
-            replace into revinfo (rev, all_revprops) values (?, ?)
-        """, (rev, all_revprops))
+    def insert_revprops(self, revision, revprops, all):
+        raise NotImplementedError(self.insert_revprops)
 
     def last_revnum(self):
-        saved_revnum = self.cachedb.execute("SELECT MAX(rev) FROM revinfo").fetchone()[0]
-        if saved_revnum is None:
-            return 0
-        return saved_revnum
+        raise NotImplementedError(self.last_revnum)
 
 
-class CachingLogWalker(CacheTable):
+class CachingLogWalker(object):
     """Subversion log browser."""
 
-    def __init__(self, actual, cache_db=None):
-        self.cache = LogCache(cache_db)
+    def __init__(self, actual, cache):
+        self.cache = cache
         self.actual = actual
         self.quick_revprops = actual.quick_revprops
         self._transport = actual._transport
         self.saved_revnum = self.cache.last_revnum()
         self._latest_revnum = None
 
+    def mutter(self, text, *args, **kwargs):
+        if "logwalker" in debug.debug_flags:
+            trace.mutter(text, *args, **kwargs)
+
     def find_latest_change(self, path, revnum):
         """Find latest revision that touched path.
 
@@ -225,16 +153,19 @@ class CachingLogWalker(CacheTable):
         self._fetch_revisions(revnum)
 
         self.mutter("latest change: %r:%r", path, revnum)
-        revnum = self.cache.find_latest_change(path.strip("/"), revnum)
+        try:
+            revnum = self.cache.find_latest_change(path.strip("/"), revnum)
+        except NotImplementedError:
+            return self.actual.find_latest_change(path, revnum)
         if revnum is None and path == "":
             return 0
         
         return revnum
 
-    def iter_changes(self, paths, from_revnum, to_revnum=0, limit=0, pb=None):
+    def iter_changes(self, prefixes, from_revnum, to_revnum=0, limit=0, pb=None):
         """Return iterator over all the revisions between from_revnum and to_revnum named path or inside path.
 
-        :param paths:    Paths to report about.
+        :param prefixes:    Prefixes of paths to report about
         :param from_revnum:  Start revision.
         :param to_revnum: End revision.
         :return: An iterator that yields tuples with (paths, revnum, revprops)
@@ -242,11 +173,11 @@ class CachingLogWalker(CacheTable):
             in revnum.
         """
         assert from_revnum >= 0 and to_revnum >= 0, "%r -> %r" % (from_revnum, to_revnum)
-        self.mutter("iter changes %r->%r (%r)", from_revnum, to_revnum, paths)
+        self.mutter("iter changes %r->%r (%r)", from_revnum, to_revnum, prefixes)
         self._fetch_revisions(max(from_revnum, to_revnum), pb=pb)
 
-        return iter(iter_changes(paths, from_revnum, to_revnum, 
-            self.get_revision_paths, self.revprop_list, limit, pb))
+        return iter(iter_changes(prefixes, from_revnum, to_revnum, 
+            self.get_revision_paths, self.revprop_list, limit))
 
     def get_revision_paths(self, revnum):
         if revnum == 0:
@@ -260,8 +191,7 @@ class CachingLogWalker(CacheTable):
         self._fetch_revisions(revnum)
 
         if revnum > 0:
-            has_all_revprops = self.cache.has_all_revprops(revnum)
-            known_revprops = self.cache.get_revprops(revnum)
+            (known_revprops, has_all_revprops) = self.cache.get_revprops(revnum)
         else:
             has_all_revprops = False
             known_revprops = {}
@@ -273,8 +203,7 @@ class CachingLogWalker(CacheTable):
 
     def _caching_revprop_list(self, revnum):
         revprops = self._transport.revprop_list(revnum)
-        self.cache.insert_revprops(revnum, revprops)
-        self.cache.insert_revinfo(revnum, True)
+        self.cache.insert_revprops(revnum, revprops, True)
         self.cache.commit()
         return revprops
 
@@ -302,18 +231,10 @@ class CachingLogWalker(CacheTable):
 
         def rcvr(orig_paths, revision, revprops, has_children=None):
             nested_pb.update('fetching svn revision info', revision, to_revnum)
-            if orig_paths is None:
-                orig_paths = {}
-            for p in orig_paths:
-                copyfrom_path = orig_paths[p][1]
-                if copyfrom_path is not None:
-                    copyfrom_path = copyfrom_path.strip("/").decode("utf-8")
-
-                self.cache.insert_path(revision, p.strip("/").decode("utf-8"), orig_paths[p][0], copyfrom_path, orig_paths[p][2])
-            self.cache.insert_revprops(revision, revprops)
-            self.cache.insert_revinfo(revision, todo_revprops is None)
+            self.cache.insert_paths(revision, orig_paths)
+            self.cache.insert_revprops(revision, revprops, 
+                                       todo_revprops is None)
             self.saved_revnum = revision
-            self.cache.commit_conditionally()
 
         self.mutter("get_log %d->%d", self.saved_revnum, to_revnum)
 
@@ -332,9 +253,9 @@ class CachingLogWalker(CacheTable):
                         revision="Revision number %d" % to_revnum)
                 raise
         finally:
+            self.cache.commit()
             if pb is None:
                 nested_pb.finished()
-        self.cache.commit()
 
 
 def strip_slashes(changed_paths):
@@ -386,10 +307,10 @@ class LogWalker(object):
     def revprop_list(self, revnum):
         return lazy_dict({}, self._transport.revprop_list, revnum)
 
-    def iter_changes(self, paths, from_revnum, to_revnum=0, limit=0, pb=None):
+    def iter_changes(self, prefixes, from_revnum, to_revnum=0, limit=0, pb=None):
         """Return iterator over all the revisions between revnum and 0 named path or inside path.
 
-        :param paths:    Paths report about (in revnum)
+        :param prefixes:    Prefixes to report about (in from_revnum)
         :param from_revnum:  Start revision.
         :param to_revnum: End revision.
         :return: An iterator that yields tuples with (paths, revnum, revprops)
@@ -404,12 +325,10 @@ class LogWalker(object):
             else:
                 todo_revprops = ["svn:author", "svn:log", "svn:date"]
 
-            iterator = self._transport.iter_log(paths, from_revnum, to_revnum, limit, 
+            iterator = self._transport.iter_log(prefixes, from_revnum, to_revnum, limit, 
                                                     True, False, False, revprops=todo_revprops)
 
             for (changed_paths, revnum, known_revprops, has_children) in iterator:
-                if pb is not None:
-                    pb.update("determining changes", from_revnum-revnum, from_revnum-to_revnum)
                 if revnum == 0 and changed_paths is None:
                     revpaths = changes.REV0_CHANGES
                 elif isinstance(changed_paths, dict):
@@ -454,9 +373,9 @@ class DictBasedLogWalker(object):
         self.paths = paths
         self.revprops = revprops
 
-    def iter_changes(self, paths, from_revnum, to_revnum=0, limit=0, pb=None):
-        return iter(iter_changes(paths, from_revnum, to_revnum, 
-            self.get_revision_paths, self.revprop_list, limit, pb))
+    def iter_changes(self, prefixes, from_revnum, to_revnum=0, limit=0, pb=None):
+        return iter(iter_changes(prefixes, from_revnum, to_revnum, 
+            self.get_revision_paths, self.revprop_list, limit))
 
     def get_revision_paths(self, revnum):
         if revnum == 0:
