@@ -13,8 +13,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
 """Maps between Subversion and Bazaar semantics."""
 
+
 import calendar
 from subvertpy import properties
 import time
@@ -24,8 +26,12 @@ from bzrlib import (
     foreign,
     osutils,
     )
-from bzrlib.lazy_import import lazy_import
-from bzrlib.errors import InvalidRevisionId
+from bzrlib.lazy_import import (
+    lazy_import,
+    )
+from bzrlib.errors import (
+    InvalidRevisionId,
+    )
 from bzrlib.revision import (
     NULL_REVISION,
     Revision,
@@ -33,11 +39,13 @@ from bzrlib.revision import (
 from bzrlib.trace import mutter
 
 from bzrlib.plugins.svn import (
+    changes,
     errors,
     get_client_string,
     version_info,
     )
 
+
 SVN_PROP_BZR_PREFIX = 'bzr:'
 SVN_PROP_BZR_ANCESTRY = 'bzr:ancestry:'
 SVN_PROP_BZR_FILEIDS = 'bzr:file-ids'
@@ -198,6 +206,8 @@ def parse_svn_log(log):
 
 def parse_svn_revprops(svn_revprops, rev):
     rev.committer = svn_revprops.get(properties.PROP_REVISION_AUTHOR, "")
+    if rev.committer is None:
+        rev.committer = ""
     rev.message = parse_svn_log(svn_revprops.get(properties.PROP_REVISION_LOG))
 
     assert svn_revprops.has_key(properties.PROP_REVISION_DATE)
@@ -224,7 +234,11 @@ def parse_revision_metadata(text, rev):
         elif key == "properties":
             in_properties = True
         elif key[0] == "\t" and in_properties:
-            rev.properties[str(key[1:])] = value.decode("utf-8")
+            name = str(key[1:])
+            if not name in rev.properties:
+                rev.properties[name] = value.decode("utf-8")
+            else:
+                rev.properties[name] += "\n" + value.decode("utf-8")
         else:
             raise errors.InvalidPropertyValue(SVN_PROP_BZR_REVISION_INFO, 
                     "Invalid key %r" % key)
@@ -272,9 +286,8 @@ def generate_revision_metadata(timestamp, timezone, committer, revprops):
         for k, v in sorted(revprops.iteritems()):
             if "\n" in k:
                 raise AssertionError("Invalid property name: %s" % k)
-            if "\n" in v:
-                raise AssertionError("Invalid property %s: %s" % (k, v))
-            text += "\t%s: %s\n" % (k.encode("utf-8"), v.encode("utf-8"))
+            for vline in v.encode("utf-8").split("\n"):
+                text += "\t%s: %s\n" % (k.encode("utf-8"), vline)
     assert isinstance(text, str)
     return text
 
@@ -763,23 +776,26 @@ class BzrSvnMappingFileProps(object):
         # Lookup the revision from the bzr:revision-id-vX property
         text = fileprops.get(SVN_PROP_BZR_REVISION_ID+self.name, None)
         if text is None:
-            return (None, None)
+            return (None, None, 
+                    self.is_bzr_revision_hidden_fileprops(fileprops))
 
         try:
             new_lines = find_new_lines(text)
         except ValueError, e:
             mutter(str(e))
-            return (None, None)
+            return (None, None, 
+                    self.is_bzr_revision_hidden_fileprops(fileprops))
 
         if len(new_lines) != 1:
             mutter("unexpected number of lines: %r" % new_lines)
-            return (None, None)
+            return (None, None, self.is_bzr_revision_hidden_fileprops(fileprops))
 
         try:
-            return parse_revid_property(new_lines[0])
+            (revno, revid) = parse_revid_property(new_lines[0])
+            return (revno, revid, self.is_bzr_revision_hidden_fileprops(fileprops))
         except errors.InvalidPropertyValue, e:
             mutter(str(e))
-            return (None, None)
+            return (None, None, self.is_bzr_revision_hidden_fileprops(fileprops))
 
     def export_fileid_map_fileprops(self, fileids, fileprops):
         if fileids != {}:
@@ -858,10 +874,10 @@ class BzrSvnMappingRevProps(object):
     def get_revision_id_revprops(self, revprops):
         if (not is_bzr_revision_revprops(revprops) or 
             not SVN_REVPROP_BZR_REVISION_ID in revprops):
-            return (None, None)
+            return (None, None, self.is_bzr_revision_hidden_revprops(revprops))
         revid = revprops[SVN_REVPROP_BZR_REVISION_ID]
         revno = int(revprops[SVN_REVPROP_BZR_REVNO])
-        return (revno, revid)
+        return (revno, revid, self.is_bzr_revision_hidden_revprops(revprops))
 
     def export_message_revprops(self, message, revprops):
         revprops[SVN_REVPROP_BZR_LOG] = message.encode("utf-8")
