@@ -15,7 +15,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Pushing to Subversion repositories."""
 
-from collections import defaultdict
+try:
+    from collections import defaultdict
+except ImportError:
+    from bzrlib.plugins.svn.pycompat import defaultdict
 
 import subvertpy
 from subvertpy import (
@@ -47,15 +50,18 @@ from bzrlib.trace import (
     mutter,
     )
 
+from bzrlib.plugins.svn import (
+    mapping,
+    )
 from bzrlib.plugins.svn.commit import (
     SvnCommitBuilder,
-    _check_dirs_exist,
     )
 from bzrlib.plugins.svn.config import (
     BranchConfig,
     )
 from bzrlib.plugins.svn.errors import (
     ChangesRootLHSHistory, 
+    MissingPrefix,
     convert_svn_error,
     )
 from bzrlib.plugins.svn.repository import (
@@ -63,6 +69,8 @@ from bzrlib.plugins.svn.repository import (
     SvnRepository,
     )
 from bzrlib.plugins.svn.transport import (
+    check_dirs_exist,
+    create_branch_prefix,
     url_join_unescaped_path,
     )
 
@@ -75,62 +83,6 @@ def find_push_base_revision(source, target, stop_revision):
     return start_revid
 
 
-def dpush(target, source, stop_revision=None):
-    """Push derivatives of the revisions missing from target from source into 
-    target.
-
-    :param target: Branch to push into
-    :param source: Branch to retrieve revisions from
-    :param stop_revision: If not None, stop at this revision.
-    :return: Map of old revids to new revids.
-    """
-    source.lock_write()
-    try:
-        if stop_revision is None:
-            stop_revision = ensure_null(source.last_revision())
-        if target.last_revision() in (stop_revision, source.last_revision()):
-            return { source.last_revision(): source.last_revision() }
-        # Request graph from source repository, since it is most likely
-        # faster than the target (Subversion) repository
-        graph = source.repository.get_graph(target.repository)
-        if not graph.is_ancestor(target.last_revision(), stop_revision):
-            if graph.is_ancestor(stop_revision, target.last_revision()):
-                return { source.last_revision(): source.last_revision() }
-            raise DivergedBranches(source, target)
-        todo = target._missing_revisions(source.repository, stop_revision)
-        if todo is None:
-            raise DivergedBranches(self, other)
-        revid_map = {}
-        target_branch_path = target.get_branch_path()
-        target_config = target.get_config()
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            # FIXME: Call create_branch_with_hidden_commit if the revision is 
-            # already present in the target repository ?
-            for rev in source.repository.get_revisions(todo):
-                pb.update("pushing revisions", todo.index(rev.revision_id), 
-                          len(todo))
-                if len(rev.parent_ids) == 0:
-                    base_revid = NULL_REVISION
-                elif rev.parent_ids[0] in revid_map:
-                    base_revid = revid_map[rev.parent_ids[0]]
-                else:
-                    base_revid = rev.parent_ids[0]
-                revid_map[rev.revision_id], _ = push(graph, target.repository,
-                        target_branch_path, target_config, base_revid, 
-                        source.repository, rev, push_metadata=False)
-        finally:
-            pb.finished()
-        source.repository.fetch(target.repository, 
-                                revision_id=revid_map[rev.revision_id])
-        target._clear_cached_state()
-        assert stop_revision in revid_map
-        assert len(revid_map.keys()) > 0
-        return revid_map
-    finally:
-        source.unlock()
-
-
 def replay_delta(builder, old_trees, new_tree):
     """Replays a delta to a commit builder.
 
@@ -146,11 +98,11 @@ def replay_delta(builder, old_trees, new_tree):
 
 
 def push_revision_tree(graph, target_repo, branch_path, config, source_repo, 
-                       base_revid, revision_id, rev, push_metadata=True,
+                       base_revid, revision_id, rev, 
+                       base_foreign_revid, base_mapping,
+                       push_metadata=True,
                        append_revisions_only=True,
-                       override_svn_revprops=None,
-                       base_foreign_revid=None,
-                       base_mapping=None):
+                       override_svn_revprops=None):
     """Push a revision tree into a target repository.
 
     :param graph: Repository graph.
@@ -185,21 +137,20 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
     builder = SvnCommitBuilder(target_repo, branch_path, base_revids,
                                config, rev.timestamp,
                                rev.timezone, rev.committer, rev.properties, 
-                               revision_id, 
+                               revision_id, base_foreign_revid, base_mapping,
                                base_tree.inventory,
                                push_metadata=push_metadata,
                                graph=graph, opt_signature=opt_signature,
                                texts=source_repo.texts,
                                append_revisions_only=append_revisions_only,
-                               override_svn_revprops=override_svn_revprops,
-                               base_foreign_revid=base_foreign_revid,
-                               base_mapping=base_mapping)
+                               override_svn_revprops=override_svn_revprops)
     parent_trees = [base_tree]
     for p in rev.parent_ids[1:]:
         try:
             parent_trees.append(source_repo.revision_tree(p))
         except NoSuchRevision:
             pass # Ghost, ignore
+    # TODO: Use iter_changes() ?
     replay_delta(builder, parent_trees, old_tree)
     try:
         revid = builder.commit(rev.message)
@@ -213,33 +164,6 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
     return revid, (builder.result_foreign_revid, builder.mapping)
 
 
-def push(graph, target_repo, target_path, target_config, base_revid, source_repo, rev, push_metadata=True, base_foreign_revid=None, base_mapping=None):
-    """Push a revision into Subversion.
-
-    This will do a new commit in the target branch.
-
-    :param target: Branch to push to
-    :param source_repo: Branch to pull the revision from
-    :param rev: Revision id for the revision to push
-    :return: revision id of revision that was pushed
-    """
-    assert isinstance(source_repo, Repository)
-    mutter('pushing %r (%r)', rev.revision_id, rev.parent_ids)
-
-    source_repo.lock_read()
-    try:
-        revid, foreign_revid = push_revision_tree(graph, target_repo, target_path, 
-            target_config, source_repo, base_revid, rev.revision_id, rev, 
-            push_metadata=push_metadata, 
-            append_revisions_only=target_config.get_append_revisions_only(), 
-            override_svn_revprops=target_config.get_override_svn_revprops(),
-            base_foreign_revid=base_foreign_revid, base_mapping=base_mapping)
-    finally:
-        source_repo.unlock()
-    assert revid == rev.revision_id or not push_metadata
-    return revid, foreign_revid
-
-
 class InterToSvnRepository(InterRepository):
     """Any to Subversion repository actions."""
 
@@ -251,45 +175,74 @@ class InterToSvnRepository(InterRepository):
         # Dictionary: revid -> branch_path -> (foreign_revid, mapping)
         self._foreign_info = defaultdict(dict)
 
+    def _target_has_revision(self, revid):
+        """Slightly optimized version of self.target.has_revision()."""
+        if revid in self._foreign_info:
+            return True
+        return self.target.has_revision(revid)
+
     def _get_base_revision(self, revid, path):
-        if not revid in self._foreign_info:
+        """Find the revision info for a revision id."""
+        if revid == NULL_REVISION:
             return None, None
+        if not revid in self._foreign_info:
+            # FIXME: Prefer revisions in path
+            return self.target.lookup_revision_id(revid)
         if path in self._foreign_info[revid]:
             return self._foreign_info[revid][path]
         else:
             return self._foreign_info[revid].values()[0]
 
+    def add_path_info(self, revid, path, foreign_info):
+        self._foreign_info[revid][path] = foreign_info
+
     @staticmethod
     def _get_repo_format_to_test():
         """See InterRepository._get_repo_format_to_test()."""
         return None
 
     def push_branch(self, todo, layout, project, target_branch, target_config,
-        push_merged=False):
+        push_merged, overwrite):
         """Push a series of revisions into a Subversion repository.
 
         """
+        assert todo != []
+        count = 0
         pb = ui.ui_factory.nested_progress_bar()
         try:
             for rev in self.source.get_revisions(todo):
                 pb.update("pushing revisions", todo.index(rev.revision_id), 
                           len(todo))
-                if push_merged:
+                if push_merged and len(rev.parent_ids) > 1:
                     self.push_ancestors(layout, project, 
                         rev.parent_ids, create_prefix=True)
-                self.push(target_branch, target_config, rev)
+                last = self.push(target_branch, target_config, rev, 
+                    overwrite=overwrite)
+                count += 1
+            return (count, last)
         finally:
             pb.finished()
 
-    def push(self, target_path, target_config, rev, push_metadata=True):
-        if rev.parent_ids:
-            base_revid = rev.parent_ids[0]
-        else:
-            base_revid = NULL_REVISION
-        base_foreign_revid, base_mapping = self._get_base_revision(base_revid, target_path)
-        revid, foreign_info = push(self.get_graph(), self.target, target_path, target_config, base_revid, self.source, rev, push_metadata=push_metadata, base_foreign_revid=base_foreign_revid, base_mapping=base_mapping)
-        self._foreign_info[revid][target_path] = foreign_info
-        return revid, foreign_info
+    def push(self, target_path, target_config, rev, push_metadata=True, 
+             base_revid=None, overwrite=False):
+        if base_revid is None:
+            if rev.parent_ids:
+                base_revid = rev.parent_ids[0]
+            else:
+                base_revid = NULL_REVISION
+        base_foreign_revid, base_mapping = self._get_base_revision(base_revid,
+            target_path)
+        mutter('pushing %r (%r)', rev.revision_id, rev.parent_ids)
+        revid, foreign_info = push_revision_tree(self.get_graph(), self.target,
+            target_path, target_config, self.source, base_revid,
+            rev.revision_id, rev, 
+            base_foreign_revid, base_mapping,
+            push_metadata=push_metadata, 
+            append_revisions_only=self.get_append_revisions_only(target_config, overwrite), 
+            override_svn_revprops=target_config.get_override_svn_revprops())
+        assert revid == rev.revision_id or not push_metadata
+        self.add_path_info(target_path, revid, foreign_info)
+        return (revid, foreign_info)
 
     def _get_branch_config(self, branch_path):
         return BranchConfig(urlutils.join(self.target.base, branch_path), 
@@ -320,8 +273,9 @@ class InterToSvnRepository(InterRepository):
         mapping = self.target.get_mapping()
         if (start_revid != NULL_REVISION and 
             start_revid_parent != NULL_REVISION and 
-            stop_revision == start_revid and mapping.supports_hidden):
-            if (self.target.has_revision(start_revid) or 
+            stop_revision == start_revid and mapping.supports_hidden and
+            not append_revisions_only):
+            if (self._target_has_revision(start_revid) or 
                 start_revid == NULL_REVISION):
                 revid = start_revid
             else:
@@ -333,11 +287,11 @@ class InterToSvnRepository(InterRepository):
             revid, foreign_info = push_revision_tree(self.get_graph(), self.target, target_branch_path, 
                 self._get_branch_config(target_branch_path), 
                 self.source, start_revid_parent, start_revid, 
-                rev, push_metadata=push_metadata,
+                rev, base_foreign_revid, base_mapping,
+                push_metadata=push_metadata,
                 append_revisions_only=append_revisions_only,
-                override_svn_revprops=override_svn_revprops, 
-                base_foreign_revid=base_foreign_revid, base_mapping=base_mapping)
-        self._foreign_info[revid][target_branch_path] = foreign_info
+                override_svn_revprops=override_svn_revprops) 
+        self.add_path_info(target_branch_path, revid, foreign_info)
         return revid, foreign_info
 
     def push_ancestors(self, layout, project, parent_revids, 
@@ -349,31 +303,39 @@ class InterToSvnRepository(InterRepository):
         :param parent_revids: The revision ids of the basic ancestors to push
         :param create_prefix: Whether to optionally create the prefix of the branches.
         """
-        for parent_revid in self.target.has_revisions(parent_revids[1:]):
+        present_rhs_parents = self.target.has_revisions(parent_revids[1:])
+        unique_ancestors = set()
+        for parent_revid in parent_revids[1:]:
+            if parent_revid in present_rhs_parents:
+                continue
             # Push merged revisions
-            unique_ancestors = self.get_graph().find_unique_ancestors(parent_revid, [parent_revids[0]])
-            for x in self.get_graph().iter_topo_order(unique_ancestors):
-                if self.target.has_revision(x):
-                    continue
-                rev = self.source.get_revision(x)
-                rhs_branch_path = determine_branch_path(rev, layout, project)
-                try:
-                    self.push_new(rhs_branch_path, 
-                             x, append_revisions_only=False)
-                except MissingPrefix, e:
-                    if not create_prefix:
-                        raise
-                    revprops = {properties.PROP_REVISION_LOG: "Add branches directory."}
-                    if self.target.transport.has_capability("commit-revprops"):
-                        revprops[mapping.SVN_REVPROP_BZR_SKIP] = ""
-                    create_branch_prefix(self.target, revprops, e.path.split("/")[:-1], filter(lambda x: x != "", e.existing_path.split("/")))
-                    self.push_new(rhs_branch_path, x, append_revisions_only=False)
+            unique_ancestors.update(self.get_graph().find_unique_ancestors(parent_revid, [parent_revids[0]]))
+        for x in self.get_graph().iter_topo_order(unique_ancestors):
+            if self._target_has_revision(x):
+                continue
+            rev = self.source.get_revision(x)
+            rhs_branch_path = determine_branch_path(rev, layout, project)
+            try:
+                self.push_new(rhs_branch_path, x, append_revisions_only=False)
+            except MissingPrefix, e:
+                if not create_prefix:
+                    raise
+                revprops = {properties.PROP_REVISION_LOG: "Add branches directory."}
+                if self.target.transport.has_capability("commit-revprops"):
+                    revprops[mapping.SVN_REVPROP_BZR_SKIP] = ""
+                create_branch_prefix(self.target.transport, revprops, e.path.split("/")[:-1], filter(lambda x: x != "", e.existing_path.split("/")))
+                self.push_new(rhs_branch_path, x, append_revisions_only=False)
 
     def push_new_branch(self, layout, project, target_branch_path, 
-        stop_revision, override_svn_revprops=None, push_merged=False):
+        stop_revision, push_merged=None, override_svn_revprops=None,
+        overwrite=False):
         if self.target.transport.check_path(target_branch_path,
             self.target.get_latest_revnum()) != subvertpy.NODE_NONE:
             raise AlreadyBranchError(target_branch_path)
+        target_config = self._get_branch_config(target_branch_path)
+        if push_merged is None:
+            push_merged = (layout.push_merged_revisions(project) and
+                           target_config.get_push_merged_revisions())
         begin_revid, _ = self.push_new(target_branch_path, stop_revision, 
                  append_revisions_only=True, 
                  override_svn_revprops=override_svn_revprops)
@@ -383,9 +345,9 @@ class InterToSvnRepository(InterRepository):
                 break
             todo.append(revid)
         todo.reverse()
-        self.push_branch(todo, layout, project, target_branch_path, 
-            self._get_branch_config(target_branch_path), 
-            push_merged=push_merged)
+        if todo != []:
+            self.push_branch(todo, layout, project, target_branch_path, 
+                target_config, push_merged, overwrite)
 
     def push_nonmainline_revision(self, rev, layout):
         mutter('pushing %r', rev.revision_id)
@@ -396,19 +358,27 @@ class InterToSvnRepository(InterRepository):
 
         if parent_revid == NULL_REVISION:
             target_project = None
+            base_foreign_revid = None
+            base_mapping = None
         else:
-            (_, bp, _), _ = self.target.lookup_revision_id(parent_revid)
-            (_, target_project, _, _) = layout.parse(bp)
+            base_foreign_revid, base_mapping = self.target.lookup_revision_id(parent_revid,
+                foreign_sibling=rev.foreign_revid)
+            (_, target_project, _, _) = layout.parse(base_foreign_revid[1])
         bp = determine_branch_path(rev, layout, target_project)
-        target_config = BranchConfig(urlutils.join(self.target.base, bp), self.target.uuid)
+        target_config = self._get_branch_config(bp)
         if (layout.push_merged_revisions(target_project) and 
-            target_config.get_push_merged_revisions()):
+            target_config.get_push_merged_revisions() and
+            len(rev.parent_ids) > 1):
             self.push_ancestors(layout, target_project, 
                 rev.parent_ids, create_prefix=True)
         return push_revision_tree(self.get_graph(), self.target, 
             bp, target_config, 
             self.source, parent_revid, rev.revision_id, rev, 
-            append_revisions_only=target_config.get_append_revisions_only())
+            base_foreign_revid, base_mapping,
+            append_revisions_only=self.get_append_revisions_only(target_config))
+
+    def get_append_revisions_only(self, target_config, overwrite=False):
+        return target_config.get_append_revisions_only(not overwrite)
 
     def get_graph(self):
         if self._graph is None:
@@ -423,7 +393,7 @@ class InterToSvnRepository(InterRepository):
             # Go back over the LHS parent until we reach a revid we know
             todo = []
             for revision_id in self.source.iter_reverse_revision_history(revision_id):
-                if self.target.has_revision(revision_id):
+                if self._target_has_revision(revision_id):
                     break
                 todo.append(revision_id)
             if todo == []:
@@ -440,9 +410,14 @@ class InterToSvnRepository(InterRepository):
         finally:
             self.source.unlock()
 
-    def fetch(self, revision_id=None, pb=None, find_ghosts=False):
+    def fetch(self, revision_id=None, pb=None, find_ghosts=False, 
+        fetch_spec=None):
         """Fetch revisions. """
-        self.copy_content(revision_id=revision_id, pb=pb)
+        if fetch_spec is not None:
+            for revid in fetch_spec.heads:
+                self.copy_content(revision_id=revid, pb=pb)
+        else:
+            self.copy_content(revision_id=revision_id, pb=pb)
 
     @staticmethod
     def is_compatible(source, target):
@@ -465,44 +440,6 @@ def determine_branch_path(rev, layout, project=None):
         return layout.get_branch_path(nick, project)
 
 
-def create_branch_prefix(repository, revprops, bp_parts, existing_bp_parts):
-    """Create a branch prefixes (e.g. "branches")
-
-    :param repository: Subversion repository
-    :param revprops: Revision properties to set
-    :param bp_parts: Branch path elements that should be created (list of names, 
-        ["branches", "foo"] for "branches/foo")
-    :param existing_bp_parts: Branch path elements that already exist.
-    """
-    conn = repository.transport.get_connection()
-    try:
-        ci = convert_svn_error(conn.get_commit_editor)(revprops)
-        try:
-            root = ci.open_root()
-            name = None
-            batons = [root]
-            for p in existing_bp_parts:
-                if name is None:
-                    name = p
-                else:
-                    name += "/" + p
-                batons.append(batons[-1].open_directory(name))
-            for p in bp_parts[len(existing_bp_parts):]:
-                if name is None:
-                    name = p
-                else:
-                    name += "/" + p
-                batons.append(batons[-1].add_directory(name))
-            for baton in reversed(batons):
-                baton.close()
-        except:
-            ci.abort()
-            raise
-        ci.close()
-    finally:
-        repository.transport.add_connection(conn)
-
-
 def create_branch_with_hidden_commit(repository, branch_path, revid,
                                      set_metadata=True,
                                      deletefirst=False):
@@ -533,7 +470,7 @@ def create_branch_with_hidden_commit(repository, branch_path, revid,
     parent = urlutils.dirname(branch_path)
 
     bp_parts = branch_path.split("/")
-    existing_bp_parts =_check_dirs_exist(repository.transport, bp_parts, -1)
+    existing_bp_parts = check_dirs_exist(repository.transport, bp_parts, -1)
     if (len(bp_parts) not in (len(existing_bp_parts), len(existing_bp_parts)+1)):
         raise MissingPrefix("/".join(bp_parts), "/".join(existing_bp_parts))
 
