@@ -13,11 +13,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
 """Subversion BzrDir formats."""
 
+
 import subvertpy
 
 import bzrlib
+from bzrlib import (
+    trace,
+    )
 from bzrlib.bzrdir import (
     BzrDirFormat,
     BzrDir,
@@ -28,19 +34,52 @@ from bzrlib.errors import (
     NotLocalUrl,
     NoWorkingTree,
     )
-from bzrlib.trace import warning
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
+from bzrlib.push import (
+    PushResult,
+    )
 
 from bzrlib.plugins.svn.errors import (
     NoSvnRepositoryPresent,
     NotSvnBranchPath,
     )
-from bzrlib.plugins.svn.format import SvnRemoteFormat
-from bzrlib.plugins.svn.repository import SvnRepository
+from bzrlib.plugins.svn.format import (
+    SvnRemoteFormat,
+    get_rich_root_format,
+    )
+from bzrlib.plugins.svn.repository import (
+    SvnRepository,
+    )
 from bzrlib.plugins.svn.transport import (
     bzr_to_svn_url,
     get_svn_ra_transport,
     )
 
+
+class SubversionPushResult(PushResult):
+
+    def report(self, to_file):
+        """Write a human-readable description of the result."""
+        if self.branch_push_result is None:
+            trace.note('Created new branch at %s.', self.target_branch_path)
+        else:
+            self.branch_push_result.report(to_file)
+
+    @property
+    def old_revno(self):
+        if self.branch_push_result is None:
+            return 0
+        return self.branch_push_result.old_revno
+
+    @property
+    def old_revid(self):
+        if self.branch_push_result is None:
+            return NULL_REVISION
+        return self.branch_push_result.old_revid
+
+
 class SvnRemoteAccess(BzrDir):
     """BzrDir implementation for Subversion connections.
     
@@ -75,8 +114,9 @@ class SvnRemoteAccess(BzrDir):
         if self.branch_path == "":
             guessed_layout = self.find_repository().get_guessed_layout()
             if guessed_layout is not None and not guessed_layout.is_branch(""):
-                warning('Cloning Subversion repository as branch. '
-                        'To import the individual branches in the repository, use "bzr svn-import".')
+                trace.warning('Cloning Subversion repository as branch. '
+                        'To import the individual branches in the repository, '
+                        'use "bzr svn-import".')
         return super(SvnRemoteAccess, self).sprout(*args, **kwargs)
 
     def open_repository(self, _unsupported=False):
@@ -104,9 +144,9 @@ class SvnRemoteAccess(BzrDir):
         else:
             return SvnRepository(self, transport, self.branch_path)
 
-    def cloning_metadir(self, stacked=False):
+    def cloning_metadir(self, require_stacking=False):
         """Produce a metadir suitable for cloning with."""
-        return bzrlib.bzrdir.format_registry.make_bzrdir("1.9-rich-root")
+        return get_rich_root_format(require_stacking)
 
     def open_workingtree(self, _unsupported=False,
             recommend_upgrade=True):
@@ -134,8 +174,8 @@ class SvnRemoteAccess(BzrDir):
             format = BzrDirFormat.get_default_format()
         return not isinstance(self._format, format.__class__)
 
-    def import_branch(self, source, stop_revision=None, _push_merged=None,
-                      _override_svn_revprops=None):
+    def import_branch(self, source, stop_revision=None, overwrite=False, 
+                      _push_merged=None, _override_svn_revprops=None):
         """Create a new branch in this repository, possibly 
         with the specified history, optionally importing revisions.
         
@@ -159,8 +199,9 @@ class SvnRemoteAccess(BzrDir):
             if type not in ('branch', 'tag') or ip != '':
                 raise NotBranchError(branch_path)
             inter.push_new_branch(layout, project, target_branch_path, 
-                    stop_revision, override_svn_revprops=_override_svn_revprops,
-                    push_merged=_push_merged)
+                    stop_revision, 
+                    override_svn_revprops=_override_svn_revprops,
+                    push_merged=_push_merged, overwrite=overwrite)
             return self.open_branch()
         finally:
             source.unlock()
@@ -180,7 +221,7 @@ class SvnRemoteAccess(BzrDir):
         branch.bzrdir = self
         return branch
 
-    def open_branch(self, unsupported=True):
+    def open_branch(self, unsupported=True, ignore_fallbacks=False):
         """See BzrDir.open_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         repos = self.find_repository()
@@ -192,28 +233,31 @@ class SvnRemoteAccess(BzrDir):
         """See BzrDir.create_repository."""
         return self.open_repository()
 
-try:
-    from bzrlib.branch import InterBranchBzrDir
-except ImportError:
-    pass
-else:
-    class InterBranchSvnDir(InterBranchBzrDir):
-
-        @classmethod
-        def is_compatible(cls, source, target):
-            return isinstance(target, SvnRemoteAccess)
-
-        def push(self, revision_id=None, overwrite=False, remember=False):
+    def push_branch(self, source, revision_id=None, overwrite=False,
+        remember=False, create_prefix=False):
+        ret = SubversionPushResult()
+        ret.source_branch = source
+        ret.workingtree_updated = None
+        ret.stacked_on = None
+        ret.master_branch = None
+        try:
+            target_branch = self.open_branch()
+            if source.get_push_location() is None or remember:
+                source.set_push_location(target_branch.base)
+            ret.target_branch = target_branch
+            target_branch.lock_write()
             try:
-                target_branch = self.target.open_branch()
-                target_branch.lock_write()
-                try:
-                    return target_branch.pull(self.source, stop_revision=revision_id, overwrite=overwrite) 
-                finally:
-                    target_branch.unlock()
-            except NotBranchError:
-                # FIXME: Return a PullResult
-                target_branch = self.target.import_branch(self.source, revision_id)
-
-
-    InterBranchBzrDir.register_optimiser(InterBranchSvnDir)
+                ret.branch_push_result = source.push(
+                    target_branch, stop_revision=revision_id, 
+                    overwrite=overwrite) 
+            finally:
+                target_branch.unlock()
+        except NotBranchError:
+            ret.target_branch_path = "/%s" % self.branch_path.lstrip("/")
+            if create_prefix:
+                self.root_transport.create_prefix()
+            ret.target_branch = self.import_branch(source, revision_id, 
+                overwrite=overwrite)
+            if source.get_push_location() is None or remember:
+                source.set_push_location(ret.target_branch.base)
+        return ret
