@@ -15,11 +15,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Subversion repository access."""
 
-from collections import defaultdict
+try:
+    from collections import defaultdict
+except ImportError:
+    from bzrlib.plugins.svn.pycompat import defaultdict
+
 from itertools import chain
 import os
 import subvertpy
-import threading
+
+ERR_RA_DAV_FORBIDDEN = getattr(subvertpy, "ERR_RA_DAV_FORBIDDEN", 175013)
 
 from bzrlib import (
     branch,
@@ -28,7 +33,6 @@ from bzrlib import (
     osutils,
     ui,
     urlutils,
-    xml6,
     )
 from bzrlib.foreign import ForeignRepository
 from bzrlib.inventory import Inventory
@@ -60,7 +64,6 @@ from bzrlib.plugins.svn import (
     cache,
     changes,
     errors,
-    foreign,
     layout,
     logwalker,
     revmeta,
@@ -68,10 +71,8 @@ from bzrlib.plugins.svn import (
 from bzrlib.plugins.svn.branchprops import PathPropertyProvider
 from bzrlib.plugins.svn.config import SvnRepositoryConfig
 from bzrlib.plugins.svn.fileids import (
-    CachingFileIdMap,
-    FileIdMap,
-    idmap_lookup,
-    idmap_reverse_lookup,
+    CachingFileIdMapStore,
+    FileIdMapStore,
     simple_apply_changes,
     )
 from bzrlib.plugins.svn.layout.standard import WildcardLayout
@@ -95,11 +96,6 @@ from bzrlib.plugins.svn.revids import (
     )
 from bzrlib.plugins.svn.tree import SvnRevisionTree
 from bzrlib.plugins.svn.versionedfiles import SvnTexts
-from bzrlib.plugins.svn.foreign.versionedfiles import (
-    VirtualRevisionTexts, 
-    VirtualInventoryTexts,
-    VirtualSignatureTexts,
-    )
 
 LAYOUT_SOURCE_GUESSED = 'guess'
 LAYOUT_SOURCE_CONFIG = 'config'
@@ -112,6 +108,7 @@ class SvnRepositoryFormat(RepositoryFormat):
     """
     rich_root_data = True
     supports_tree_reference = False
+    _serializer = None
 
     def __get_matchingbzrdir(self):
         from remote import SvnRemoteFormat
@@ -125,6 +122,9 @@ class SvnRepositoryFormat(RepositoryFormat):
     def get_format_description(self):
         return "Subversion Repository"
 
+    def network_name(self):
+        return "subversion"
+
     def initialize(self, url, shared=False, _internal=False):
         raise bzr_errors.UninitializableFormat(self)
 
@@ -132,17 +132,6 @@ class SvnRepositoryFormat(RepositoryFormat):
         return target_repo_format.rich_root_data
 
 
-CACHE_DB_VERSION = 5
-
-_cachedbs = threading.local()
-def cachedbs():
-    """Get a cache for this thread's db connections."""
-    try:
-        return _cachedbs.cache
-    except AttributeError:
-        _cachedbs.cache = {}
-        return _cachedbs.cache
-
 
 class SubversionRepositoryCheckResult(branch.BranchCheckResult):
     """Results of checking a Subversion repository."""
@@ -303,10 +292,10 @@ class SubversionRepositoryCheckResult(branch.BranchCheckResult):
             # Every text revision either has to match the actual revision's 
             # revision id (if it was last changed there) or the text revisions 
             # in one of the parents.
-            fileid = idmap_lookup(fileid_map, mapping, path)[0]
+            fileid = fileid_map.lookup(mapping, path)[0]
             parent_text_revisions = []
             for parent_fileid_map, parent_mapping in zip(parent_fileid_maps, parent_mappings):
-                parent_text_revisions.append(idmap_reverse_lookup(parent_fileid_map, parent_mapping, fileid))
+                parent_text_revisions.append(parent_fileid_map.reverse_lookup(parent_mapping, fileid))
             if (text_revision != revmeta.get_revision_id(mapping) and 
                     not ghost_parents and not text_revision in parent_text_revisions):
                 self.invalid_text_revisions += 1
@@ -341,10 +330,10 @@ class SvnRepository(ForeignRepository):
         self.base = transport.base
         assert self.base is not None
         self._config = None
-        self._serializer = xml6.serializer_v6
+        self._serializer = None
         self.get_config().add_location(self.base)
         self._log = logwalker.LogWalker(transport=transport)
-        self.fileid_map = FileIdMap(simple_apply_changes, self)
+        self.fileid_map = FileIdMapStore(simple_apply_changes, self)
         self.revmap = RevidMap(self)
         self._default_mapping = None
         self._hinted_branch_path = branch_path
@@ -363,43 +352,34 @@ class SvnRepository(ForeignRepository):
                 use_cache = set(["fileids", "revids", "revinfo", "log"])
 
         if use_cache:
-            cache_dir = self.create_cache_dir()
-            assert isinstance(cache_dir, str)
-
-        if "log" in use_cache or "revids" in use_cache:
-            cache_file = os.path.join(cache_dir, 'cache-v%d' % CACHE_DB_VERSION)
-            assert isinstance(cache_file, str)
-            if not cachedbs().has_key(cache_file):
-                cachedbs()[cache_file] = cache.connect_cachefile(cache_file.decode(osutils._fs_enc).encode("utf-8"))
-            self.cachedb = cachedbs()[cache_file]
+            cache_obj = cache.cache_cls(self.uuid)
 
         if "log" in use_cache:
             self._log = logwalker.CachingLogWalker(self._log,
-                cache_db=self.cachedb)
+                cache_obj.open_logwalker())
 
         if "fileids" in use_cache:
-            cachedir_transport = get_transport(cache_dir.decode(osutils._fs_enc))
-            self.fileid_map = CachingFileIdMap(cachedir_transport, 
-                self.fileid_map)
+            self.fileid_map = CachingFileIdMapStore(
+                cache_obj.open_fileid_map(), self.fileid_map)
 
         if "revids" in use_cache:
-            self.revmap = DiskCachingRevidMap(self.revmap, self.cachedb)
+            self.revmap = DiskCachingRevidMap(self.revmap, cache_obj.open_revid_map())
             self._real_parents_provider = DiskCachingParentsProvider(
-                self._real_parents_provider, self.cachedb)
+                self._real_parents_provider, cache_obj.open_parents())
         else:
             self.revmap = MemoryCachingRevidMap(self.revmap)
 
         if "revinfo" in use_cache:
-            self.revinfo_cache = RevisionInfoCache(self.cachedb)
+            self.revinfo_cache = cache_obj.open_revision_cache()
         else:
             self.revinfo_cache = None
 
         self._parents_provider = graph.CachingParentsProvider(
             self._real_parents_provider)
         self.texts = SvnTexts(self)
-        self.revisions = VirtualRevisionTexts(self)
-        self.inventories = VirtualInventoryTexts(self)
-        self.signatures = VirtualSignatureTexts(self)
+        self.revisions = None
+        self.inventories = None
+        self.signatures = None
 
         self.branchprop_list = PathPropertyProvider(self._log)
 
@@ -426,6 +406,13 @@ class SvnRepository(ForeignRepository):
             self._lock_mode = None
             self._clear_cached_state()
 
+    def reconcile(self, other=None, thorough=False):
+        """Reconcile this repository."""
+        from bzrlib.plugins.svn.reconcile import RepoReconciler
+        reconciler = RepoReconciler(self, thorough=thorough)
+        reconciler.reconcile()
+        return reconciler
+
     def _clear_cached_state(self, revnum=None):
         self._cached_tags = {}
         self._cached_revnum = revnum
@@ -481,11 +468,10 @@ class SvnRepository(ForeignRepository):
         # XXX: Note ATM no callers actually pay attention to this return
         #      instead they just use the list of revision ids and ignore
         #      missing sigs. Consider removing this work entirely
-        revisions_with_signatures = set(self.signatures.get_parent_map(
-            [(r,) for r in revision_ids]))
-        revisions_with_signatures = set(
-            [r for (r,) in revisions_with_signatures])
-        revisions_with_signatures.intersection_update(revision_ids)
+        revisions_with_signatures = set()
+        for revid in revision_ids:
+            if self.has_signature_for_revision_id(revid):
+                revisions_with_signatures.add(revid)
         yield ("signatures", None, revisions_with_signatures)
         yield ("revisions", None, revision_ids)
 
@@ -505,6 +491,8 @@ class SvnRepository(ForeignRepository):
         result['latestrev'] = revdate(self.get_latest_revnum())
         # Approximate number of revisions
         result['revisions'] = self.get_latest_revnum()+1
+        result['svn-uuid'] = self.uuid
+        result['svn-last-revnum'] = self.get_latest_revnum()
         return result
 
     def get_mapping_class(self):
@@ -632,7 +620,8 @@ class SvnRepository(ForeignRepository):
         layoutname = self.get_config().get_guessed_layout()
         if layoutname is not None:
             config_guessed_layout = layout.layout_registry.get(layoutname)()
-            if self._hinted_branch_path is None or config_guessed_layout.is_branch(self._hinted_branch_path):
+            if (self._hinted_branch_path is None or 
+                config_guessed_layout.is_branch(self._hinted_branch_path)):
                 self._guessed_layout = config_guessed_layout
                 self._guessed_appropriate_layout = config_guessed_layout
                 return
@@ -663,14 +652,6 @@ class SvnRepository(ForeignRepository):
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.base)
 
-    def create_cache_dir(self):
-        cache_dir = cache.create_cache_dir()
-        dir = os.path.join(cache_dir, self.uuid)
-        if not os.path.exists(dir):
-            info("Initialising Subversion metadata cache in %s" % dir.decode(osutils._fs_enc))
-            os.mkdir(dir)
-        return dir
-
     def _check(self, revision_ids):
         ret = SubversionRepositoryCheckResult(self)
         pb = ui.ui_factory.nested_progress_bar()
@@ -694,6 +675,10 @@ class SvnRepository(ForeignRepository):
         assert revision_id != None
         return self.revision_tree(revision_id).inventory
 
+    def _iter_inventories(self, revision_ids):
+        for revid in revision_ids:
+            yield self.get_inventory(revid)
+
     def get_fileid_map(self, revmeta, mapping):
         return self.fileid_map.get_map(revmeta.get_foreign_revid(), mapping)
 
@@ -727,7 +712,7 @@ class SvnRepository(ForeignRepository):
         """
         return False
 
-    def _iter_reverse_revmeta_mapping_ancestry(self, branch_path, revnum, mapping, lhs_history=None):
+    def _iter_reverse_revmeta_mapping_ancestry(self, branch_path, revnum, mapping, lhs_history=None, pb=None):
         """Iterate over the (revmeta, mapping) entries for the ancestry
         of a specified path.
 
@@ -741,23 +726,31 @@ class SvnRepository(ForeignRepository):
                 todo.append(entry)
                 processed.add(entry)
         if lhs_history is None:
-            update_todo(todo, self._iter_reverse_revmeta_mapping_history(branch_path, revnum, to_revnum=0, mapping=mapping))
+            update_todo(todo, self._iter_reverse_revmeta_mapping_history(branch_path, revnum, to_revnum=0, mapping=mapping, pb=bp))
         else:
             update_todo(todo, lhs_history)
+        i = 0
         while todo:
             entry = todo.pop()
+            if pb is not None:
+                pb.update("finding rhs ancestors", i, i+len(todo))
+            i += 1
             (revmeta, mapping) = entry
             yield entry
             for rhs_parent_revid in revmeta.get_rhs_parents(mapping):
                 try:
-                    (_, rhs_parent_bp, rhs_parent_revnum), rhs_parent_mapping = self.lookup_revision_id(rhs_parent_revid)
+                    (_, rhs_parent_bp, rhs_parent_revnum), rhs_parent_mapping = self.lookup_revision_id(rhs_parent_revid, foreign_sibling=revmeta.get_foreign_revid())
                 except bzr_errors.NoSuchRevision:
                     pass
                 else:
-                    update_todo(todo, self._iter_reverse_revmeta_mapping_history(rhs_parent_bp, rhs_parent_revnum, to_revnum=0, mapping=mapping))
+                    update_todo(todo, self._iter_reverse_revmeta_mapping_history(rhs_parent_bp, rhs_parent_revnum, to_revnum=0, mapping=mapping, pb=pb))
 
     def _iter_reverse_revmeta_mapping_history(self, branch_path, revnum, 
         to_revnum, mapping, pb=None, limit=0): 
+        """Iterate over the history of a RevisionMetadata object.
+
+        This will skip hidden revisions.
+        """
         assert mapping is not None
         expected_revid = None
         iter = self._revmeta_provider.iter_reverse_branch_changes(branch_path, 
@@ -765,11 +758,14 @@ class SvnRepository(ForeignRepository):
         for revmeta in iter:
             (mapping, lhs_mapping) = revmeta.get_appropriate_mappings(mapping)
             assert not lhs_mapping.newer_than(mapping), "LHS mapping %r newer than %r" % (lhs_mapping, mapping)
+            if revmeta.is_hidden(mapping):
+                mapping = lhs_mapping
+                continue
             revid = revmeta.get_revision_id(mapping)
             if (expected_revid is not None and
                 not revid in (None, expected_revid)):
                 # Need to restart, branch root has changed
-                (_, branch_path, revnum), mapping = self.lookup_revision_id(revid)
+                (_, branch_path, revnum), mapping = self.lookup_revision_id(revid, foreign_sibling=revmeta.get_foreign_revid())
                 iter = self._revmeta_provider.iter_reverse_branch_changes(branch_path, revnum, to_revnum=to_revnum, pb=pb, limit=limit)
             if not mapping.is_branch_or_tag(revmeta.branch_path):
                 return
@@ -778,11 +774,12 @@ class SvnRepository(ForeignRepository):
             mapping = lhs_mapping
         if expected_revid is not None and expected_revid != NULL_REVISION:
             # Need to restart, branch root has changed
-            (_, branch_path, revnum), mapping = self.lookup_revision_id(expected_revid)
+            (_, branch_path, revnum), mapping = self.lookup_revision_id(expected_revid, foreign_sibling=revmeta.get_foreign_revid())
             for (revmeta, mapping) in self._iter_reverse_revmeta_mapping_history(branch_path, revnum, to_revnum=to_revnum, mapping=mapping, pb=pb, limit=limit):
                 yield (revmeta, mapping)
 
-    def iter_reverse_revision_history(self, revision_id, pb=None, limit=0):
+    def iter_reverse_revision_history(self, revision_id, pb=None, limit=0, 
+                                      project=None):
         """Iterate backwards through revision ids in the lefthand history
 
         :param revision_id: The revision id to start with.  All its lefthand
@@ -790,7 +787,7 @@ class SvnRepository(ForeignRepository):
         """
         if revision_id in (None, NULL_REVISION):
             return
-        (uuid, branch_path, revnum), mapping = self.lookup_revision_id(revision_id)
+        (uuid, branch_path, revnum), mapping = self.lookup_revision_id(revision_id, project=project)
         assert uuid == self.uuid
         for revmeta, mapping in self._iter_reverse_revmeta_mapping_history(
             branch_path, revnum, to_revnum=0, mapping=mapping, pb=pb, 
@@ -811,6 +808,13 @@ class SvnRepository(ForeignRepository):
         ancestry.reverse()
         return ancestry
 
+    def has_revisions(self, revision_ids):
+        ret = set()
+        for revision_id in revision_ids:
+            if self.has_revision(revision_id):
+                ret.add(revision_id)
+        return ret
+
     def has_revision(self, revision_id, project=None):
         """See Repository.has_revision()."""
         if revision_id is None:
@@ -915,7 +919,7 @@ class SvnRepository(ForeignRepository):
         return self.lookup_foreign_revision_id((self.uuid, path, revnum), mapping)
 
     def lookup_revision_id(self, revid, layout=None, ancestry=None, 
-                           project=None):
+                           project=None, foreign_sibling=None):
         """Parse an existing Subversion-based revision id.
 
         :param revid: The revision id.
@@ -926,9 +930,6 @@ class SvnRepository(ForeignRepository):
         """
         # FIXME: Use ancestry
         # If there is no entry in the map, walk over all branches:
-        if layout is None:
-            layout = self.get_layout()
-
         # Try a simple parse
         try:
             (uuid, branch_path, revnum), mapping = mapping_registry.parse_revision_id(revid)
@@ -942,6 +943,13 @@ class SvnRepository(ForeignRepository):
         except bzr_errors.InvalidRevisionId:
             pass
 
+        if layout is None:
+            layout = self.get_layout()
+        if foreign_sibling is not None and project is None:
+            try:
+                project = layout.parse(foreign_sibling[1])[1]
+            except errors.NotSvnBranchPath:
+                project = None
         return self.revmap.get_branch_revnum(revid, layout, project)
 
     def seen_bzr_revprops(self):
@@ -1000,7 +1008,7 @@ class SvnRepository(ForeignRepository):
             raise errors.RevpropChangeFailed(SVN_REVPROP_BZR_SIGNATURE)
 
     @needs_read_lock
-    def find_branches(self, layout=None, revnum=None):
+    def find_branches(self, using=False, layout=None, revnum=None):
         """Find branches underneath this repository.
 
         This will include branches inside other branches.
@@ -1016,34 +1024,42 @@ class SvnRepository(ForeignRepository):
         branches = []
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            for project, bp, nick in layout.get_branches(self, revnum, pb=pb):
+            for project, bp, nick, has_props in layout.get_branches(self, revnum, pb=pb):
                 branches.append(SvnBranch(self, bp, _skip_check=True))
         finally:
             pb.finished()
         return branches
 
     @needs_read_lock
-    def find_tags_between(self, project, layout, mapping, from_revnum, to_revnum, tags=None):
+    def find_tags_between(self, project, layout, mapping, from_revnum, 
+                          to_revnum, tags=None):
         if tags is None:
             tags = {}
         assert from_revnum <= to_revnum
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            entries = list(self._revmeta_provider.iter_all_changes(layout, mapping.is_branch_or_tag, to_revnum, from_revnum, project=project, pb=pb))
+            entries = []
+            for kind, item in self._revmeta_provider.iter_all_changes(layout, mapping.is_branch_or_tag, to_revnum, from_revnum, project=project):
+                if kind == "revision":
+                    pb.update("discovering tags", to_revnum-item.revnum, to_revnum-from_revnum)
+                    if layout.is_tag(item.branch_path):
+                        entries.append((kind, (item.branch_path, (item.revnum, item.get_tag_revmeta(mapping)))))
+                else:
+                    entries.append((kind, item))
             for kind, item in reversed(entries):
                 if kind == "revision":
-                    revmeta = item
-                    if layout.is_tag(revmeta.branch_path):
-                        tags[revmeta.branch_path] = revmeta.get_tag_revmeta(mapping)
+                    (branch_path, tag) = item
+                    tags[branch_path] = tag
                 elif kind == "delete":
-                    for t in list(tags):
-                        if changes.path_is_child(t, item):
+                    (path, revnum) = item
+                    for t, (lastrevnum, tagrevmeta) in tags.items():
+                        if changes.path_is_child(t, path) and revnum > lastrevnum:
                             del tags[t]
         finally:
             pb.finished()
 
         ret = {}
-        for path, revmeta in tags.iteritems():
+        for path, (lastrevnum, revmeta) in tags.iteritems():
             name = layout.get_tag_name(path, project)
             # Layout wasn't able to determine tag name from path
             if name is None:
@@ -1074,20 +1090,22 @@ class SvnRepository(ForeignRepository):
 
         if not layout in self._cached_tags:
             self._cached_tags[layout] = self.find_tags_between(project=project,
-                    layout=layout, mapping=mapping, from_revnum=0, to_revnum=revnum)
+                    layout=layout, mapping=mapping, from_revnum=0, 
+                    to_revnum=revnum)
         return self._cached_tags[layout]
 
-    def find_branchpaths(self, layout,
-                         from_revnum=0, to_revnum=None, 
-                         project=None):
+    def find_branchpaths(self, layout, from_revnum, to_revnum, project=None):
         """Find all branch paths that were changed in the specified revision 
         range.
 
+        :note: This ignores forbidden paths.
+
         :param revnum: Revision to search for branches.
         :return: iterator that returns tuples with (path, revision number, 
             still exists). The revision number is the revision in which the 
             branch last existed.
         """
+        assert from_revnum >= to_revnum
         if to_revnum is None:
             to_revnum = self.get_latest_revnum()
 
@@ -1133,8 +1151,9 @@ class SvnRepository(ForeignRepository):
                                         elif layout.is_branch_or_tag_parent(n, project):
                                             parents.append(n)
                                 except subvertpy.SubversionException, (_, num):
-                                    if num == subvertpy.ERR_FS_NOT_DIRECTORY:
-                                        pass
+                                    if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
+                                               ERR_RA_DAV_FORBIDDEN):
+                                        continue
                                     raise
         finally:
             pb.finished()
@@ -1157,18 +1176,29 @@ class SvnRepository(ForeignRepository):
                            revision_id=None):
         """See Repository.get_commit_builder()."""
         from bzrlib.plugins.svn.commit import SvnCommitBuilder
+        append_revisions_only = branch.get_config().get_append_revisions_only()
+        if append_revisions_only is None:
+            append_revisions_only = True
+        if parents == [] or parents == [NULL_REVISION]:
+            base_foreign_revid = None
+            base_mapping = None
+        else:
+            base_foreign_revid, base_mapping = \
+                self.lookup_revision_id(parents[0], project=branch.project)
         return SvnCommitBuilder(self, branch.get_branch_path(), parents, 
                                 config, timestamp, timezone, committer, 
                                 revprops, revision_id, 
-                                append_revisions_only=True)
+                                base_foreign_revid, base_mapping,
+                                append_revisions_only=append_revisions_only)
 
     def find_fileprop_paths(self, layout, from_revnum, to_revnum, 
                                project=None, check_removed=False):
-        if not check_removed and from_revnum == 0:
+        assert from_revnum >= to_revnum
+        if not check_removed and to_revnum == 0:
             it = iter([])
-            it = chain(it, layout.get_branches(self, to_revnum, project))
-            it = chain(it, layout.get_tags(self, to_revnum, project))
-            return iter(((branch, to_revnum, True) for (project, branch, nick) in it))
+            it = chain(it, layout.get_branches(self, from_revnum, project))
+            it = chain(it, layout.get_tags(self, from_revnum, project))
+            return iter(((branch, from_revnum, True) for (project, branch, nick, has_props) in it if has_props in (True, None)))
         else:
             return iter(self.find_branchpaths(layout, from_revnum, to_revnum, project))
 
