@@ -18,6 +18,8 @@
 
 import subvertpy
 
+ERR_RA_DAV_FORBIDDEN = getattr(subvertpy, "ERR_RA_DAV_FORBIDDEN", 175013)
+
 from bzrlib import ui
 from bzrlib.errors import (
     InvalidRevisionId,
@@ -25,7 +27,6 @@ from bzrlib.errors import (
     )
 from bzrlib.lru_cache import LRUCache
 
-from bzrlib.plugins.svn.cache import CacheTable
 from bzrlib.plugins.svn.errors import (
     InvalidBzrSvnRevision,
     InvalidPropertyValue,
@@ -52,15 +53,19 @@ class RevidMap(object):
         """
         last_revnum = self.repos.get_latest_revnum()
         fileprops_to_revnum = last_revnum
-        for entry_revid, branch, revnum, mapping in self.discover_revprop_revids(last_revnum, 0):
-            if revid == entry_revid:
-                return (self.repos.uuid, branch, revnum), mapping
-            fileprops_to_revnum = min(fileprops_to_revnum, revnum)
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for entry_revid, branch, revnum, mapping in self.discover_revprop_revids(last_revnum, 0, pb=pb):
+                if revid == entry_revid:
+                    return (self.repos.uuid, branch, revnum), mapping
+                fileprops_to_revnum = min(fileprops_to_revnum, revnum)
 
-        for entry_revid, branch, min_revno, max_revno, mapping in self.discover_fileprop_revids(layout, 0, fileprops_to_revnum, project):
-            if revid == entry_revid:
-                (foreign_revid, mapping_name) = self.bisect_revid_revnum(revid, branch, min_revno, max_revno)
-                return (foreign_revid, mapping_name)
+            for entry_revid, branch, min_revno, max_revno, mapping in self.discover_fileprop_revids(layout, fileprops_to_revnum, 0, project, pb=pb):
+                if revid == entry_revid:
+                    (foreign_revid, mapping_name) = self.bisect_revid_revnum(revid, branch, min_revno, max_revno)
+                    return (foreign_revid, mapping_name)
+        finally:
+            pb.finished()
         raise NoSuchRevision(self, revid)
 
     def discover_revprop_revids(self, from_revnum, to_revnum, pb=None):
@@ -79,27 +84,38 @@ class RevidMap(object):
                 branch_path = mapping.get_branch_root(revprops)
                 if branch_path is None:
                     continue
-                revno, revid = mapping.get_revision_id_revprops(revprops)
+                revno, revid, hidden = mapping.get_revision_id_revprops(revprops)
                 if revid is not None:
                     yield (revid, branch_path.strip("/"), revnum, mapping)
 
-    def discover_fileprop_revids(self, layout, from_revnum, to_revnum, project=None):
+    def find_branch_tips(self, layout, from_revnum, to_revnum, project=None):
+        assert from_revnum >= to_revnum
+        # TODO: Use RevisionMetadataBrowser to find the tip revmetas all at once
         reuse_policy = self.repos.get_config().get_reuse_revisions()
         assert reuse_policy in ("other-branches", "removed-branches", "none") 
         check_removed = (reuse_policy == "removed-branches")
-        # TODO: Some sort of progress indication
         for (branch, revno, exists) in self.repos.find_fileprop_paths(layout, from_revnum, to_revnum, project, check_removed=check_removed):
             assert isinstance(branch, str)
             assert isinstance(revno, int)
+            iterator = self.repos._revmeta_provider.iter_reverse_branch_changes(branch, revno, to_revnum=0, limit=0)
+            yield iterator.next()
+
+    def discover_fileprop_revids(self, layout, from_revnum, to_revnum, project=None, pb=None):
+        assert from_revnum >= to_revnum
+        for revmeta in self.find_branch_tips(layout, from_revnum, to_revnum,
+                                             project):
+            if pb is not None:
+                pb.update("finding fileprop revids", 
+                    from_revnum-revmeta.revnum, from_revnum-to_revnum)
             # Look at their bzr:revision-id-vX
             revids = set()
             try:
-                revmeta = self.repos._revmeta_provider.lookup_revision(branch, self.repos._log.find_latest_change(branch, revno))
                 if revmeta.consider_bzr_fileprops():
                     for revid, bzr_revno, mapping_name in revmeta.get_roundtrip_ancestor_revids():
                         revids.add(((bzr_revno, revid), mapping_name))
             except subvertpy.SubversionException, (_, num):
-                if num == subvertpy.ERR_FS_NOT_DIRECTORY:
+                if num in (subvertpy.ERR_FS_NOT_DIRECTORY,
+                           ERR_RA_DAV_FORBIDDEN):
                     continue
                 raise
 
@@ -111,7 +127,8 @@ class RevidMap(object):
                 except KeyError:
                     pass
                 else:
-                    yield (entry_revid, branch, 0, revno, mapping)
+                    yield (entry_revid, revmeta.branch_path, 0,
+                           revmeta.revnum, mapping)
 
     def bisect_revid_revnum(self, revid, branch_path, min_revnum, max_revnum):
         """Find out what the actual revnum was that corresponds to a revid.
@@ -186,11 +203,18 @@ class MemoryCachingRevidMap(object):
 
 class DiskCachingRevidMap(object):
 
-    def __init__(self, actual, cachedb=None):
-        self.cache = RevisionIdMapCache(cachedb)
+    def __init__(self, actual, cache):
+        self.cache = cache
         self.actual = actual
         self.revid_seen = set()
 
+    def remember_entry(self, entry_revid, branch, min_revnum, max_revnum, 
+                       mappingname):
+        if entry_revid not in self.revid_seen:
+            self.cache.insert_revid(entry_revid, branch, min_revnum, max_revnum,
+                                    mappingname)
+            self.revid_seen.add(entry_revid)
+
     def get_branch_revnum(self, revid, layout, project=None):
         # Check the record out of the cache, if it exists
         try:
@@ -218,18 +242,17 @@ class DiskCachingRevidMap(object):
                     fileprops_to_revnum = min(fileprops_to_revnum, revnum)
                     if entry_revid == revid:
                         found = (branch, revnum, revnum, mapping)
-                    if entry_revid not in self.revid_seen:
-                        self.cache.insert_revid(entry_revid, branch, revnum, revnum, mapping.name)
-                        self.revid_seen.add(entry_revid)
+                    self.remember_entry(entry_revid, branch, revnum, 
+                                            revnum, mapping.name)
+                if fileprops_to_revnum > last_checked:
+                    for entry_revid, branch, min_revno, max_revno, mapping in self.actual.discover_fileprop_revids(layout, fileprops_to_revnum, last_checked, project, pb):
+                        min_revno = max(last_checked, min_revno)
+                        if entry_revid == revid:
+                            found = (branch, min_revno, max_revno, mapping)
+                        self.remember_entry(entry_revid, branch, min_revno, 
+                                            max_revno, mapping.name)
             finally:
                 pb.finished()
-            for entry_revid, branch, min_revno, max_revno, mapping in self.actual.discover_fileprop_revids(layout, last_checked, fileprops_to_revnum, project):
-                min_revno = max(last_checked, min_revno)
-                if entry_revid == revid:
-                    found = (branch, min_revno, max_revno, mapping)
-                if entry_revid not in self.revid_seen:
-                    self.cache.insert_revid(entry_revid, branch, min_revno, max_revno, mapping.name)
-                    self.revid_seen.add(entry_revid)
             # We've added all the revision ids for this layout in the
             # repository, so no need to check again unless new revisions got 
             # added
@@ -242,28 +265,15 @@ class DiskCachingRevidMap(object):
 
         ((uuid, branch_path, revnum), mapping) = self.actual.bisect_revid_revnum(revid, 
             branch_path, min_revnum, max_revnum)
-        self.cache.insert_revid(revid, branch_path, revnum, revnum, mapping.name)
+        self.remember_entry(revid, branch_path, revnum, revnum, mapping.name)
         return (uuid, branch_path, revnum), mapping
 
 
-class RevisionIdMapCache(CacheTable):
+class RevisionIdMapCache(object):
     """Revision id mapping store. 
 
     Stores mapping from revid -> (path, revnum, mapping)
     """
-    def _create_table(self):
-        self.cachedb.executescript("""
-        create table if not exists revmap (revid text, path text, min_revnum integer, max_revnum integer, mapping text);
-        create index if not exists revid on revmap (revid);
-        create unique index if not exists revid_path_mapping on revmap (revid, path, mapping);
-        drop index if exists lookup_branch_revnum;
-        create index if not exists lookup_branch_revnum_non_unique on revmap (max_revnum, min_revnum, path, mapping);
-        create table if not exists revids_seen (layout text, max_revnum int);
-        create unique index if not exists layout on revids_seen (layout);
-        """)
-        # Revisions ids are quite expensive
-        self._commit_interval = 500
-
     def set_last_revnum_checked(self, layout, revnum):
         """Remember the latest revision number that has been checked
         for a particular layout.
@@ -271,8 +281,7 @@ class RevisionIdMapCache(CacheTable):
         :param layout: Repository layout.
         :param revnum: Revision number.
         """
-        self.cachedb.execute("replace into revids_seen (layout, max_revnum) VALUES (?, ?)", (layout, revnum))
-        self.commit_conditionally()
+        raise NotImplementedError(self.set_last_revnum_checked)
 
     def last_revnum_checked(self, layout):
         """Retrieve the latest revision number that has been checked 
@@ -281,13 +290,8 @@ class RevisionIdMapCache(CacheTable):
         :param layout: Repository layout.
         :return: Last revision number checked or 0.
         """
-        self.mutter("last revnum checked %r", layout)
-        ret = self.cachedb.execute(
-            "select max_revnum from revids_seen where layout = ?", (layout,)).fetchone()
-        if ret is None:
-            return 0
-        return int(ret[0])
-    
+        raise NotImplementedError(self.last_revnum_checked)
+
     def lookup_revid(self, revid):
         """Lookup the details for a particular revision id.
 
@@ -295,17 +299,7 @@ class RevisionIdMapCache(CacheTable):
         :return: Tuple with path inside repository, minimum revision number, maximum revision number and 
             mapping.
         """
-        assert isinstance(revid, str)
-        self.mutter("lookup revid %r", revid)
-        ret = self.cachedb.execute(
-            "select path, min_revnum, max_revnum, mapping from revmap where revid=? order by abs(min_revnum-max_revnum) asc", (revid,)).fetchone()
-        if ret is None:
-            raise NoSuchRevision(self, revid)
-        (path, min_revnum, max_revnum, mapping) = (ret[0].encode("utf-8"), int(ret[1]), int(ret[2]), ret[3].encode("utf-8"))
-        if min_revnum > max_revnum:
-            return (path, max_revnum, min_revnum, mapping)
-        else:
-            return (path, min_revnum, max_revnum, mapping)
+        raise NotImplementedError(self.lookup_revid)
 
     def lookup_branch_revnum(self, revnum, path, mapping):
         """Lookup a revision by revision number, branch path and mapping.
@@ -314,17 +308,7 @@ class RevisionIdMapCache(CacheTable):
         :param path: Subversion branch path.
         :param mapping: Mapping
         """
-        assert isinstance(revnum, int)
-        assert isinstance(path, str)
-        assert isinstance(mapping, str)
-        row = self.cachedb.execute(
-                "select revid from revmap where max_revnum=? and min_revnum=? and path=? and mapping=?", (revnum, revnum, path, mapping)).fetchone()
-        if row is not None:
-            ret = str(row[0])
-        else:
-            ret = None
-        self.mutter("lookup branch,revnum %r:%r -> %r", path, revnum, ret)
-        return ret
+        raise NotImplementedError(self.lookup_branch_revnum)
 
     def insert_revid(self, revid, branch, min_revnum, max_revnum, mapping):
         """Insert a revision id into the revision id cache.
@@ -338,76 +322,35 @@ class RevisionIdMapCache(CacheTable):
         :param mapping: Name of the mapping with which the revision 
                        was found
         """
-        assert revid is not None and revid != ""
-        assert isinstance(mapping, str)
-        assert isinstance(branch, str)
-        assert isinstance(min_revnum, int) and isinstance(max_revnum, int)
-        assert min_revnum <= max_revnum
-        self.mutter("insert revid %r:%r-%r -> %r", branch, min_revnum, max_revnum, revid)
-        if min_revnum == max_revnum:
-            cursor = self.cachedb.execute(
-                "update revmap set min_revnum = ?, max_revnum = ? WHERE revid=? AND path=? AND mapping=?",
-                (min_revnum, max_revnum, revid, branch, mapping))
-        else:
-            cursor = self.cachedb.execute(
-                "update revmap set min_revnum = MAX(min_revnum,?), max_revnum = MIN(max_revnum, ?) WHERE revid=? AND path=? AND mapping=?",
-                (min_revnum, max_revnum, revid, branch, mapping))
-        if cursor.rowcount == 0:
-            self.cachedb.execute(
-                "insert into revmap (revid,path,min_revnum,max_revnum,mapping) VALUES (?,?,?,?,?)",
-                (revid, branch, min_revnum, max_revnum, mapping))
+        raise NotImplementedError(self.insert_revid)
 
 
-class RevisionInfoCache(CacheTable):
+class RevisionInfoCache(object):
 
-    def _create_table(self):
-        self.cachedb.executescript("""
-        create table if not exists revmetainfo (path text, revnum integer, mapping text, revid text, revno int, hidden int, original_mapping text, stored_lhs_parent_revid text);
-        create unique index if not exists revmeta_path_mapping on revmetainfo(revnum, path, mapping);
-        """)
-        self._commit_interval = 500
+    def set_original_mapping(self, foreign_revid, original_mapping):
+        raise NotImplementedError(self.set_original_mapping)
 
-    def insert_revision(self, foreign_revid, mapping, revid, revno, hidden, 
-            original_mapping, stored_lhs_parent_revid):
+    def insert_revision(self, foreign_revid, mapping, revinfo, 
+            stored_lhs_parent_revid):
         """Insert a revision to the cache.
 
         :param foreign_revid: Foreign revision id
         :param mapping: Mapping used
-        :param revid: Revision id
-        :param revno: Revision number
-        :param hidden: Whether revision is hidden
+        :param revinfo: Tuple with (revno, revid, hidden)
         :param original_mapping: Original mapping used
         :param stored_lhs_parent_revid: Stored lhs parent revision
         """
-        if original_mapping is not None:
-            orig_mapping_name = original_mapping.name
-        else:
-            orig_mapping_name = None
-        self.cachedb.execute("insert into revmetainfo (path, revnum, mapping, revid, revno, hidden, original_mapping, stored_lhs_parent_revid) values (?, ?, ?, ?, ?, ?, ?, ?)", (foreign_revid[1], foreign_revid[2], mapping.name, revid, revno, hidden, orig_mapping_name, stored_lhs_parent_revid))
+        raise NotImplementedError(self.insert_revision)
 
     def get_revision(self, foreign_revid, mapping):
         """Get the revision metadata info for a (foreign_revid, mapping) tuple.
 
         :param foreign_revid: Foreign reviasion id
         :param mapping: Mapping
-        :return: Tuple with revid, stored revno, hidden, original_mapping, 
+        :return: Tuple with (stored revno, revid, hidden), original_mapping, 
             stored_lhs_parent_revid
         """
-        # Will raise KeyError if not present
-        # returns tuple with (revid, revno, hidden, original_mapping, stored_lhs_parent_revid)
-        row = self.cachedb.execute("select revid, revno, hidden, original_mapping, stored_lhs_parent_revid from revmetainfo where path = ? and revnum = ? and mapping = ?", (foreign_revid[1], foreign_revid[2], mapping.name)).fetchone()
-        if row is None:
-            raise KeyError((foreign_revid, mapping))
-        else:
-            if row[3] is None:
-                original_mapping = None
-            else:
-                original_mapping = mapping_registry.parse_mapping_name("svn-" + row[3].encode("utf-8"))
-            if row[4] is None:
-                stored_lhs_parent_revid = None
-            else:
-                stored_lhs_parent_revid = row[4].encode("utf-8")
-            return (row[0].encode("utf-8"), row[1], row[2], original_mapping, stored_lhs_parent_revid)
+        raise NotImplementedError(self.get_revision)
 
     def get_original_mapping(self, foreign_revid):
         """Find the original mapping for a revision.
@@ -415,10 +358,4 @@ class RevisionInfoCache(CacheTable):
         :param foreign_revid: Foreign revision id
         :return: Mapping object or None
         """
-        row = self.cachedb.execute("select original_mapping from revmetainfo where path = ? and revnum = ?", (foreign_revid[1], foreign_revid[2])).fetchone()
-        if row is None:
-            raise KeyError(foreign_revid)
-        if row[0] is None:
-            return None
-        else:
-            return mapping_registry.parse_mapping_name("svn-" + row[0].encode("utf-8"))
+        raise NotImplementedError(self.get_original_mapping)
