@@ -16,9 +16,13 @@
 """Subversion Tags Dictionary."""
 
 import subvertpy
-from subvertpy import properties
+from subvertpy import (
+    NODE_NONE,
+    properties,
+    )
 
 from bzrlib import (
+    ui,
     urlutils,
     )
 from bzrlib.errors import (
@@ -33,6 +37,11 @@ from bzrlib.plugins.svn import (
     commit,
     errors as svn_errors,
     mapping,
+    push,
+    )
+from bzrlib.plugins.svn.transport import (
+    check_dirs_exist,
+    create_branch_prefix,
     )
 
 def reverse_dict(orig):
@@ -41,6 +50,89 @@ def reverse_dict(orig):
         ret.setdefault(v, []).append(k)
     return ret
 
+def _resolve_reverse_tags_fallback(branch, reverse_tag_revmetas):
+    """Determine the revids for tags that were not found in the branch 
+    ancestry.
+    """
+    ret = {}
+    # For anything that's not in the branches' ancestry, just use 
+    # the latest mapping
+    for (revmeta, names) in reverse_tag_revmetas.iteritems():
+        mapping = revmeta.get_original_mapping() or branch.mapping
+        try:
+            revid = revmeta.get_revision_id(mapping)
+        except subvertpy.SubversionException, (_, ERR_FS_NOT_DIRECTORY,):
+            continue
+        for name in names:
+            assert isinstance(name, str)
+            ret[name.decode("utf-8")] = revid
+    return ret
+
+def _resolve_tags_svn_ancestry(branch, tag_revmetas):
+    """Resolve a name -> revmeta dictionary to a name -> revid dict.
+
+    The tricky bit here is figuring out what mapping to use. Preferably, 
+    we should be using whatever mapping makes the tag useful to the user, 
+    which generally means using the revid that is in the ancestry of the 
+    branch.
+
+    As fallback, we will use the mapping used by the branch. That will 
+    however cause question marks to show up rather than revno's in 
+    "bzr tags".
+    """
+    if len(tag_revmetas) == 0:
+        return {}
+    reverse_tag_revmetas = reverse_dict(tag_revmetas)
+    ret = {}
+    # Try to find the tags that are in the ancestry of this branch
+    # and use their appropriate mapping
+    pb = ui.ui_factory.nested_progress_bar()
+    try:
+        for (revmeta, mapping) in branch._iter_revision_meta_ancestry(
+            pb=pb):
+            if revmeta not in reverse_tag_revmetas:
+                continue
+            if len(reverse_tag_revmetas) == 0:
+                # No more tag revmetas to resolve, just return immediately
+                return ret
+            for name in reverse_tag_revmetas[revmeta]:
+                assert isinstance(name, str)
+                ret[name.decode("utf-8")] = revmeta.get_revision_id(mapping)
+            del reverse_tag_revmetas[revmeta]
+    finally:
+        pb.finished()
+    ret.update(_resolve_reverse_tags_fallback(branch, reverse_tag_revmetas))
+    return ret
+
+
+class ReverseTagDict(object):
+
+    def __init__(self, branch, repository, tags, project):
+        self.branch = branch
+        self.repository = repository
+        self.project = project
+        self._by_foreign_revid = {}
+        self._tags = tags
+        for name, revmeta in tags.iteritems():
+            self._by_foreign_revid.setdefault(revmeta.get_foreign_revid(), []).append(name)
+
+    def _lookup_revid(self, revid):
+        return self.repository.lookup_revision_id(revid, project=self.project)
+
+    def has_key(self, revid):
+        foreign_revid, mapping = self._lookup_revid(revid)
+        return self._by_foreign_revid.has_key(foreign_revid)
+
+    def get(self, revid, default=None):
+        foreign_revid, mapping = self._lookup_revid(revid)
+        return self._by_foreign_revid.get(foreign_revid, default)
+
+    def items(self):
+        return _resolve_tags_svn_ancestry(self.branch, self._tags).items()
+
+    def iteritems(self):
+        return iter(self.items())
+
 
 class SubversionTags(BasicTags):
     """Subversion tags object."""
@@ -48,25 +140,33 @@ class SubversionTags(BasicTags):
     def __init__(self, branch):
         self.branch = branch
         self.repository = branch.repository
-        self._parent_exists = False
+        self._parent_exists = set()
 
     def _ensure_tag_parent_exists(self, parent):
-        if self._parent_exists:
-            return
+        """Make sure that the container of these tags exists.
+
+        :param parent: Parent path
+        :return: True if the container already existed, False if it had to 
+            be created.
+        """
+        if parent in self._parent_exists:
+            return True
         assert isinstance(parent, str)
         bp_parts = parent.split("/")
-        existing_bp_parts = commit._check_dirs_exist(
+        existing_bp_parts = check_dirs_exist(
                 self.repository.transport, 
                 bp_parts, self.repository.get_latest_revnum())
         if existing_bp_parts == bp_parts:
-            self._parent_exists = True
-            return
-        commit.create_branch_prefix(self.repository, 
+            self._parent_exists.add(parent)
+            return True
+        create_branch_prefix(self.repository.transport, 
                 self._revprops("Add tags base directory."),
                 bp_parts, existing_bp_parts)
-        self._parent_exists = True
+        self._parent_exists.add(parent)
+        return False
 
     def set_tag(self, tag_name, tag_target):
+        """Set a new tag in a Subversion repository."""
         path = self.branch.layout.get_tag_path(tag_name, self.branch.project)
         assert isinstance(path, str)
         parent = urlutils.dirname(path)
@@ -75,11 +175,18 @@ class SubversionTags(BasicTags):
         except NoSuchRevision:
             mutter("not setting tag %s; unknown revision %s", tag_name, tag_target)
             return
-        if from_bp == path:
-            return
         self._ensure_tag_parent_exists(parent)
+        try:
+            current_from_foreign_revid = self._lookup_tag_revmeta(path).get_foreign_revid()
+            deletefirst = True
+        except KeyError:
+            current_from_foreign_revid = None
+            deletefirst = False
+        if current_from_foreign_revid == (from_uuid, from_bp, from_revnum):
+            # Already present
+            return
+        mutter("setting tag %s from %r (deletefirst: %r)", path, (from_uuid, from_bp, from_revnum), deletefirst)
         conn = self.repository.transport.get_connection(parent)
-        deletefirst = (conn.check_path(urlutils.basename(path), self.repository.get_latest_revnum()) != subvertpy.NODE_NONE)
         try:
             ci = svn_errors.convert_svn_error(conn.get_commit_editor)(
                     self._revprops("Add tag %s" % tag_name.encode("utf-8"),
@@ -108,73 +215,38 @@ class SubversionTags(BasicTags):
             revprops[mapping.SVN_REVPROP_BZR_SKIP] = ""
         return revprops
 
+    def _lookup_tag_revmeta(self, path):
+        revnum = self.repository.get_latest_revnum()
+        if self.repository.transport.check_path(path, revnum) == NODE_NONE:
+            raise KeyError
+        tip, mapping = self.repository._iter_reverse_revmeta_mapping_history(
+            path, revnum, to_revnum=0, mapping=self.branch.mapping).next()
+        return tip.get_tag_revmeta(mapping)
+
     def lookup_tag(self, tag_name):
+        # Note that this can't use _lookup_tag_revmeta, as there can
+        # be multiple tag container directories
         try:
             return self.get_tag_dict()[tag_name]
         except KeyError:
             raise NoSuchTag(tag_name)
 
     def _get_tag_dict_revmeta(self, from_revnum=None, to_revnum=None):
+        """Get a name -> revmeta dictionary."""
         if from_revnum is None or from_revnum == 0:
             return self.repository.find_tags(project=self.branch.project, 
                     layout=self.branch.layout,
                     mapping=self.branch.mapping,
                     revnum=self.branch._revnum)
-        else:
-            return self.repository.find_tags_between(project=self.branch.project,
+        elif from_revnum <= to_revnum:
+            return self.repository.find_tags_between(
+                    project=self.branch.project,
                     layout=self.branch.layout,
                     mapping=self.branch.mapping,
                     from_revnum=from_revnum,
                     to_revnum=to_revnum)
-
-    def _resolve_reverse_tags_fallback(self, reverse_tag_revmetas):
-        """Determine the revids for tags that were not found in the branch 
-        ancestry.
-        """
-        ret = {}
-        # For anything that's not in the branches' ancestry, just use 
-        # the latest mapping
-        for (revmeta, names) in reverse_tag_revmetas.iteritems():
-            mapping = revmeta.get_original_mapping() or self.branch.mapping
-            try:
-                revid = revmeta.get_revision_id(mapping)
-            except subvertpy.SubversionException, (_, ERR_FS_NOT_DIRECTORY):
-                continue
-            for name in names:
-                assert isinstance(name, basestring)
-                ret[name] = revid
-        return ret
-
-    def _resolve_tags_svn_ancestry(self, tag_revmetas):
-        """Resolve a name -> revmeta dictionary to a name -> revid dict.
-
-        The tricky bit here is figuring out what mapping to use. Preferably, 
-        we should be using whatever mapping makes the tag useful to the user, 
-        which generally means using the revid that is in the ancestry of the 
-        branch.
-
-        As fallback, we will use the mapping used by the branch. That will 
-        however cause question marks to show up rather than revno's in 
-        "bzr tags".
-        """
-        if len(tag_revmetas) == 0:
+        else:
             return {}
-        reverse_tag_revmetas = reverse_dict(tag_revmetas)
-        ret = {}
-        # Try to find the tags that are in the ancestry of this branch
-        # and use their appropriate mapping
-        for (revmeta, mapping) in self.branch._iter_revision_meta_ancestry():
-            if revmeta not in reverse_tag_revmetas:
-                continue
-            if len(reverse_tag_revmetas) == 0:
-                # No more tag revmetas to resolve, just return immediately
-                return ret
-            for name in reverse_tag_revmetas[revmeta]:
-                assert isinstance(name, basestring)
-                ret[name] = revmeta.get_revision_id(mapping)
-            del reverse_tag_revmetas[revmeta]
-        ret.update(self._resolve_reverse_tags_fallback(reverse_tag_revmetas))
-        return ret
 
     def _resolve_tags_ancestry(self, tag_revmetas, graph, last_revid):
         """Resolve a name -> revmeta dictionary using the ancestry of a branch.
@@ -196,20 +268,22 @@ class SubversionTags(BasicTags):
                 continue
             revmeta = foreign_revid_map[foreign_revid]
             for name in reverse_tag_revmetas[revmeta]:
-                assert isinstance(name, basestring)
-                ret[name] = revid
+                assert isinstance(name, str)
+                ret[name.decode("utf-8")] = revid
             del reverse_tag_revmetas[revmeta]
-        ret.update(self._resolve_reverse_tags_fallback(reverse_tag_revmetas))
+        ret.update(_resolve_reverse_tags_fallback(self.branch, reverse_tag_revmetas))
         return ret
 
     def get_tag_dict(self):
         tag_revmetas = self._get_tag_dict_revmeta()
-        return self._resolve_tags_svn_ancestry(tag_revmetas)
+        return _resolve_tags_svn_ancestry(self.branch, tag_revmetas)
 
     def get_reverse_tag_dict(self):
         """Returns a dict with revisions as keys
            and a list of tags for that revision as value"""
-        return reverse_dict(self.get_tag_dict())
+        return ReverseTagDict(self.branch, self.repository, 
+                              self._get_tag_dict_revmeta(),
+                              self.branch.project)
 
     def delete_tag(self, tag_name):
         path = self.branch.layout.get_tag_path(tag_name, self.branch.project)
