@@ -17,21 +17,21 @@
 """Tests for the bzr-svn plugin."""
 
 import os
+import subvertpy.tests
 import sys
 
-import bzrlib
-from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCaseInTempDir
-from bzrlib.trace import mutter
-from bzrlib.workingtree import WorkingTree
 
 from bzrlib import (
     osutils,
     urlutils,
     )
-from bzrlib.plugins.svn import cache
+from bzrlib.bzrdir import (
+    BzrDir,
+    )
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    )
 
-import subvertpy.tests
 
 class SubversionTestCase(subvertpy.tests.SubversionTestCase,TestCaseInTempDir):
 
@@ -42,12 +42,31 @@ class SubversionTestCase(subvertpy.tests.SubversionTestCase,TestCaseInTempDir):
         subvertpy.tests.SubversionTestCase.setUp(self)
         subvertpy.tests.SubversionTestCase.tearDown(self)
         TestCaseInTempDir.setUp(self)
-        self._old_connect_cachefile = cache.connect_cachefile
-        cache.connect_cachefile = lambda path: cache.sqlite3.connect(":memory:")
+
+        try:
+            from bzrlib.plugins.svn.cache import sqlitecache
+        except ImportError:
+            pass
+        else:
+            self._old_connect_sqlite = sqlitecache.connect_cachefile
+            def restore_sqlite():
+                sqlitecache.connect_cachefile = self._old_connect_sqlite
+            self.addCleanup(restore_sqlite)
+            sqlitecache.connect_cachefile = lambda path: sqlitecache.sqlite3.connect(":memory:")
+
+        try:
+            from bzrlib.plugins.svn.cache import tdbcache
+        except ImportError:
+            pass
+        else:
+            self._old_open_tdb = tdbcache.tdb_open
+            def restore_tdb():
+                tdbcache.tdb_open = self._old_open_tdb
+            self.addCleanup(restore_tdb)
+            tdbcache.tdb_open = lambda *args: dict()
 
     def tearDown(self):
         TestCaseInTempDir.tearDown(self)
-        cache.connect_cachefile = self._old_connect_cachefile
 
     def make_local_bzrdir(self, repos_path, relpath):
         """Create a repository and checkout."""
@@ -60,7 +79,7 @@ class SubversionTestCase(subvertpy.tests.SubversionTestCase,TestCaseInTempDir):
     def make_client_and_bzrdir(self, repospath, clientpath):
         repos_url = self.make_client(repospath, clientpath)
 
-        return BzrDir.open("svn+%s" % repos_url)
+        return BzrDir.open(repos_url)
 
 
 def test_suite():
@@ -75,6 +94,7 @@ def test_suite():
     testmod_names = [
             'test_branch', 
             'test_branchprops', 
+            'test_cache',
             'test_changes',
             'test_checkout',
             'test_commit',
@@ -83,6 +103,7 @@ def test_suite():
             'test_errors',
             'test_fetch',
             'test_fileids', 
+            'test_keywords',
             'test_layout',
             'test_logwalker',
             'test_mapping',
@@ -90,7 +111,6 @@ def test_suite():
             'test_push',
             'test_radir',
             'test_repository', 
-            'test_revids',
             'test_revmeta',
             'test_revspec',
             'test_svk',
@@ -104,6 +124,5 @@ def test_suite():
             'mapping3',
             'mapping3.test_scheme']
     suite.addTest(loader.loadTestsFromModuleNames(["%s.%s" % (__name__, i) for i in testmod_names]))
-    suite.addTest(loader.loadTestsFromModuleNames(["bzrlib.plugins.svn.foreign.test_versionedfiles"]))
 
     return suite
