@@ -18,6 +18,7 @@
 import os
 import sys
 
+from bzrlib.bzrdir import format_registry
 import bzrlib.gpg
 from bzrlib.repository import Repository
 from bzrlib.tests.blackbox import ExternalBase
@@ -27,6 +28,7 @@ from bzrlib.plugins.svn.convert import load_dumpfile
 from bzrlib.plugins.svn.layout.standard import RootLayout
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
+
 class TestBranch(SubversionTestCase, ExternalBase):
 
     def setUp(self):
@@ -55,8 +57,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
     def test_branch_onerev_stacked(self):
         repos_url = self.make_client('d', 'de')
         self.commit_something(repos_url)
-        self.run_bzr("branch --stacked %s dc" % repos_url)
-        self.check_output("2\n", "revno de")
+        self.run_bzr("branch --stacked %s dc" % repos_url, retcode=3)
         
     def test_log_empty(self):
         repos_url = self.make_repository('d')
@@ -70,6 +71,22 @@ class TestBranch(SubversionTestCase, ExternalBase):
         repos_url = self.make_repository('d')
         self.run_bzr('pack %s' % repos_url)
 
+    def test_push_create_prefix(self):
+        repos_url = self.make_repository('d')
+        
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/foo").modify()
+        dc.close()
+
+        self.run_bzr("branch %s/trunk dc" % repos_url)
+        self.build_tree({"dc/foo": "blaaaa"})
+        self.run_bzr("commit -m msg dc")
+        self.run_bzr_error([
+             'ERROR: Prefix missing for branches\\/mybranch; please create it before pushing.'],
+             ["push", "-d", "dc", "%s/branches/mybranch" % repos_url])
+        self.run_bzr("push --create-prefix -d dc %s/branches/mybranch" % repos_url)
+
     def test_push(self):
         repos_url = self.make_repository('d')
         
@@ -83,6 +100,55 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.run_bzr("push -d dc %s" % repos_url)
         self.check_output("", "status dc")
 
+    def test_push_empty_existing(self):
+        repos_url = self.make_repository('d')
+        
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        self.run_bzr("init dc")
+        self.build_tree({"dc/foo": "blaaaa"})
+        self.run_bzr("add dc/foo")
+        self.run_bzr("commit -m msg dc")
+        self.run_bzr_error([
+            # Ideally this should say:
+            # ['ERROR: Empty branch already exists at /trunk. Specify --overwrite or remove it before pushing.\n'
+             'ERROR: These branches have diverged.  Try using "merge" and then "push".\n'
+            ], ["push", "-d", "dc", "%s/trunk" % repos_url])
+
+    def test_dpush_empty_existing(self):
+        repos_url = self.make_repository('d')
+        
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        self.run_bzr("init dc")
+        self.build_tree({"dc/foo": "blaaaa"})
+        self.run_bzr("add dc/foo")
+        self.run_bzr("commit -m msg dc")
+        self.run_bzr_error(
+            ['ERROR: Empty branch already exists at /trunk. Specify --overwrite or remove it before pushing.\n'],
+            ["dpush", "-d", "dc", "%s/trunk" % repos_url])
+
+    def test_push_verbose(self):
+        repos_url = self.make_repository('d')
+        self.run_bzr("init dc")
+        self.build_tree({"dc/foo": "blaaaa"})
+        self.run_bzr("add dc/foo")
+        self.run_bzr("commit -m msg dc")
+        self.run_bzr("push -v -d dc %s/trunk" % repos_url)
+        self.build_tree({"dc/foo": "blaaaaa"})
+        self.run_bzr("commit -m msg dc")
+        self.run_bzr("push -v -d dc %s/trunk" % repos_url)
+
+    def test_reconcile(self):
+        repos_url = self.make_repository('d')
+
+        output, err = self.run_bzr("reconcile %s" % repos_url, retcode=0)
+        self.assertContainsRe(output, "Reconciliation complete.\n")
+
     def test_missing(self):
         repos_url = self.make_repository('d')
         
@@ -117,7 +183,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_empty(self):
         repos_url = self.make_repository('dp')
-        self.run_bzr("init --rich-root-pack dc")
+        self.run_bzr("init --default-rich-root dc")
         os.chdir("dc")
         self.run_bzr("dpush %s" % repos_url)
 
@@ -333,12 +399,14 @@ Node-copyfrom-path: x
 
     def test_svn_import_bzr_branch(self):
         self.run_bzr('init foo')
-        self.run_bzr_error('bzr: ERROR: Source repository is not a Subversion repository.\n', 'svn-import foo dc')
+        self.run_bzr_error(['bzr: ERROR: Source repository is not a Subversion repository.\n'], 
+                           ['svn-import', 'foo', 'dc'])
 
     def test_svn_import_bzr_repo(self):
         self.run_bzr('init-repo foo')
         self.run_bzr('init foo/bar')
-        self.run_bzr_error('bzr: ERROR: Source repository is not a Subversion repository.\n', 'svn-import foo/bar dc')
+        self.run_bzr_error(['bzr: ERROR: Source repository is not a Subversion repository.\n'],
+                           ['svn-import', 'foo/bar', 'dc'])
 
     def test_list(self):
         repos_url = self.make_repository("a")
@@ -442,6 +510,22 @@ if len(sys.argv) == 2:
         self.monkey_patch_gpg()
         self.run_bzr('sign-my-commits')
 
+    def test_pull_old(self):
+        svn_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(svn_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/foo").modify("bar1")
+        dc.close()
+
+        dc = self.get_commit_editor(svn_url)
+        trunk = dc.open_dir("trunk")
+        trunk.open_file("trunk/foo").modify("bar2")
+        dc.close()
+
+        self.run_bzr('branch %s/trunk trunk' % svn_url)
+        self.run_bzr('pull -d trunk -r1 %s/trunk' % svn_url)
+
     def test_knit_corruption(self):
         cwd = os.getcwd()
         svn_url = self.make_client('d', 'wc')
@@ -459,7 +543,7 @@ if len(sys.argv) == 2:
         self.client_update(".")
         os.chdir(cwd)
 
-        self.run_bzr('init-repo --1.9-rich-root --no-trees shared')
+        self.run_bzr('init-repo --default-rich-root --no-trees shared')
         os.chdir('shared')
         self.run_bzr('branch %s/trunk trunk' % svn_url)
         os.chdir(cwd)
@@ -475,7 +559,9 @@ if len(sys.argv) == 2:
         f.close()
         self.run_bzr('ci -m "Add good riddance."')
 
-        self.run_bzr('svn-push %s/branches/my-branch' % svn_url)
+        output, err = self.run_bzr('push %s/branches/my-branch' % svn_url)
+        self.assertEquals(output, '')
+        self.assertEquals(err, 'Created new branch at /branches/my-branch.\n')
         os.chdir(cwd)
 
         self.run_bzr('co %s/trunk bzr-trunk' % svn_url)
