@@ -35,7 +35,6 @@ from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 
 from bzrlib.plugins.svn.branch import (
-    FakeControlFiles,
     SvnBranchFormat,
     )
 from bzrlib.plugins.svn.convert import load_dumpfile
@@ -104,6 +103,74 @@ class WorkingSubversionBranch(SubversionTestCase):
         self.assertEquals(subvertpy.NODE_DIR, 
                 b.repository.transport.check_path("tags/mytag", 3))
 
+    def test_tag_set_dupe(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.add_dir("tags")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file("trunk/bla").modify()
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        target = b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping())
+        b.tags.set_tag(u"mytag", b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping()))
+
+        self.assertEquals(subvertpy.NODE_DIR, 
+                b.repository.transport.check_path("tags/mytag", 3))
+        self.assertEquals(3, b.repository.get_latest_revnum())
+
+        b.tags.set_tag(u"mytag", target)
+
+        self.assertEquals(3, b.repository.get_latest_revnum())
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals({u"mytag": target}, b.tags.get_tag_dict())
+
+    def test_tag_set_existing(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.add_dir("tags")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file("trunk/bla").modify()
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        b.tags.set_tag(u"mytag", b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping()))
+
+        self.assertEquals(subvertpy.NODE_DIR, 
+                b.repository.transport.check_path("tags/mytag", 3))
+        self.assertEquals(3, b.repository.get_latest_revnum())
+
+        oldtagrevid = b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping())
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals({u"mytag": oldtagrevid}, b.tags.get_tag_dict())
+
+        newtagrevid = b.repository.generate_revision_id(2, "trunk", b.repository.get_mapping())
+        b.tags.set_tag(u"mytag", newtagrevid)
+
+        self.assertEquals(subvertpy.NODE_DIR, 
+                b.repository.transport.check_path("tags/mytag", 4))
+        self.assertEquals(4, b.repository.get_latest_revnum())
+        b = Branch.open(repos_url + "/trunk")
+        log = self.client_log(repos_url, 4, 0)
+        self.assertEquals(log[0][0], None)
+        self.assertEquals(log[1][0], {'/tags': ('A', None, -1), 
+                                      '/trunk': ('A', None, -1)})
+        self.assertEquals(log[2][0], {'/trunk/bla': ('A', None, -1)})
+        self.assertEquals(log[3][0], {'/tags/mytag': ('A', '/trunk', 1)})
+        self.assertEquals(log[4][0], {'/tags/mytag': ('R', '/trunk', 2)})
+
+        self.assertEquals({u"mytag": newtagrevid}, b.tags.get_tag_dict())
+
     def test_tags_delete(self):
         repos_url = self.make_repository("a")
        
@@ -119,6 +186,28 @@ class WorkingSubversionBranch(SubversionTestCase):
         b = Branch.open(repos_url + "/trunk")
         self.assertEquals([], b.tags.get_tag_dict().keys())
 
+    def test_tag_set_no_parent_Dir(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file("trunk/bla").modify()
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        b.tags.set_tag(u"mytag", b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping()))
+
+        self.assertEquals(subvertpy.NODE_DIR, 
+                b.repository.transport.check_path("tags", 3))
+
+        self.assertEquals(subvertpy.NODE_DIR, 
+                b.repository.transport.check_path("tags/mytag", 4))
+        self.assertEquals(4, b.repository.get_latest_revnum())
+
     def test_tag_set_not_supported(self):
         repos_url = self.make_repository('a')
 
@@ -310,13 +399,14 @@ class WorkingSubversionBranch(SubversionTestCase):
 
         branch = Branch.open(repos_url+"/trunk")
         orig_history = branch.revision_history()
+        branch.get_config().set_user_option('append_revisions_only', 'False')
         branch.set_revision_history(orig_history[:-1])
         self.assertEquals(orig_history[:-1], branch.revision_history())
 
     def test_break_lock(self):
         repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
-        branch.control_files.break_lock()
+        branch.break_lock()
 
     def test_repr(self):
         repos_url = self.make_repository('a')
@@ -343,7 +433,6 @@ class WorkingSubversionBranch(SubversionTestCase):
         branch = Branch.open(repos_url)
         self.assertIs(None, branch.get_push_location())
 
-
     def test_revision_id_to_revno_none(self):
         """The None revid should map to revno 0."""
         repos_url = self.make_repository('a')
@@ -747,7 +836,7 @@ foohosts""")
 
         oldbranch = Branch.open(repos_url+"/trunk")
         newtree = oldbranch.create_checkout("e", lightweight=True)
-        self.assertEqual(oldbranch.generate_revision_id(1), newtree.base_revid)
+        self.assertEqual(oldbranch.generate_revision_id(1), newtree.last_revision())
         self.assertTrue(os.path.exists("e/.svn"))
         self.assertFalse(os.path.exists("e/.bzr"))
 
@@ -770,7 +859,7 @@ foohosts""")
         newtree = oldbranch.create_checkout("e", revision_id=
            oldbranch.generate_revision_id(1), lightweight=True)
         self.assertEqual(oldbranch.generate_revision_id(1),
-           newtree.base_revid)
+           newtree.last_revision())
         self.assertTrue(os.path.exists("e/.svn"))
         self.assertFalse(os.path.exists("e/.bzr"))
 
@@ -868,17 +957,6 @@ foohosts""")
                 newdir.open_branch().last_revision())
 
 
-class TestFakeControlFiles(TestCase):
-    def test_get_utf8(self):
-        f = FakeControlFiles()
-        self.assertRaises(NoSuchFile, f.get_utf8, "foo")
-
-
-    def test_get(self):
-        f = FakeControlFiles()
-        self.assertRaises(NoSuchFile, f.get, "foobla")
-
-
 class BranchFormatTests(TestCase):
     def setUp(self):
         self.format = SvnBranchFormat()
