@@ -16,12 +16,13 @@
 
 """Branch property access tests."""
 
-from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.branchprops import PathPropertyProvider
 from bzrlib.plugins.svn.logwalker import (
+    LogCache,
     LogWalker,
     CachingLogWalker,
     )
+from bzrlib.plugins.svn.tests import SubversionTestCase
 from bzrlib.plugins.svn.transport import SvnRaTransport
 
 class TestBranchProps(SubversionTestCase):
@@ -29,7 +30,7 @@ class TestBranchProps(SubversionTestCase):
         super(TestBranchProps, self).setUp()
 
     def get_log_walker(self, transport):
-        return CachingLogWalker(LogWalker(transport=transport))
+        return LogWalker(transport=transport)
 
     def test_get_old_properties(self):
         repos_url = self.make_repository('d')
