@@ -28,12 +28,16 @@ from bzrlib.workingtree import WorkingTree
 
 from bzrlib.plugins.svn.fileids import (
     FileIdMapCache,
+    get_local_changes,
     idmap_lookup,
     idmap_reverse_lookup,
     simple_apply_changes,
     )
-from bzrlib.plugins.svn.mapping import mapping_registry
+from bzrlib.plugins.svn.mapping import (
+    mapping_registry,
+    )
 from bzrlib.plugins.svn.layout.standard import (
+    CustomLayout,
     RootLayout,
     TrunkLayout,
     )
@@ -102,7 +106,7 @@ class TestComplexFileids(SubversionTestCase):
         dc.add_file("bar", "foo", 1).modify("data2")
         dc.close()
 
-        bzrdir = BzrDir.open("svn+%s" % repos_url)
+        bzrdir = BzrDir.open(repos_url)
         repository = bzrdir.find_repository()
 
         mapping = repository.get_mapping()
@@ -127,7 +131,7 @@ class TestComplexFileids(SubversionTestCase):
         dc.delete("foo")
         dc.close()
 
-        bzrdir = BzrDir.open("svn+%s" % repos_url)
+        bzrdir = BzrDir.open(repos_url)
         repository = bzrdir.find_repository()
         mapping = repository.get_mapping()
 
@@ -189,13 +193,15 @@ class TestComplexFileids(SubversionTestCase):
         self.assertEqual(inv1.path2id("dir/file"), inv2.path2id("dir/file"))
 
         rm_provider = repository._revmeta_provider
-        fileid, revid, child_create_revid = repository.get_fileid_map(rm_provider.get_revision("branches/mybranch", 2), mapping)["dir/file"]
+        fileid, revid, child_create_revid = repository.get_fileid_map(rm_provider.get_revision("branches/mybranch", 2), mapping).as_dict()["dir/file"]
         self.assertEqual(fileid, inv1.path2id("dir/file"))
         self.assertEqual(repository.generate_revision_id(1, "trunk", mapping), revid)
 
 
 class TestFileMapping(TestCase):
+
     def setUp(self):
+        TestCase.setUp(self)
         self.generate_file_id = lambda (uuid, bp, revnum), ip: "%d@%s:%s:%s" % (revnum, uuid, bp, ip)
 
     def apply_mappings(self, mappings, renames={}):
@@ -213,13 +219,13 @@ class TestFileMapping(TestCase):
         return map
 
     def test_simple(self):
-        map = self.apply_mappings({(1, ""): {u"foo": ('A', None, None)}})
+        map = self.apply_mappings({(1, ""): {u"foo": ('A', None)}})
         self.assertEqual({ u'foo': ("1@uuid::foo",
                                        (1, ""))
                          }, map)
 
     def test_simple_add(self):
-        map = self.apply_mappings({(1, ""): {u"": ('A', None, None), u"foo": ('A', None, None)}})
+        map = self.apply_mappings({(1, ""): {u"": ('A', None), u"foo": ('A', None)}})
         self.assertEqual({
             u'': ('1@uuid::', (1, "")),
             u'foo': ("1@uuid::foo", (1, "")) 
@@ -228,12 +234,12 @@ class TestFileMapping(TestCase):
     def test_copy(self):
         map = self.apply_mappings({
                 (1, ""): {
-                                   u"foo": ('A', None, None), 
-                                   u"foo/blie": ('A', None, None),
-                                   u"foo/bla": ('A', None, None)},
+                                   u"foo": ('A', None), 
+                                   u"foo/blie": ('A', None),
+                                   u"foo/bla": ('A', None)},
                 (2, ""): {
-                                   u"foob": ('A', 'foo', 1), 
-                                   u"foob/bla": ('M', None, None)}
+                                   u"foob": ('A', ('foo', 1)), 
+                                   u"foob/bla": ('M', None)}
                 })
         self.assertEquals(map,
             {u'foo': (u'1@uuid::foo', (1, '')),
@@ -244,10 +250,10 @@ class TestFileMapping(TestCase):
     def test_touchparent(self):
         map = self.apply_mappings(
                 {(1, ""): {
-                                   u"foo": ('A', None, None), 
-                                   u"foo/bla": ('A', None, None)},
+                                   u"foo": ('A', None), 
+                                   u"foo/bla": ('A', None)},
                  (2, ""): {
-                                   u"foo/bla": ('M', None, None)}
+                                   u"foo/bla": ('M', None)}
                 })
         self.assertEqual((1, ""), 
                          map[u"foo"][1])
@@ -257,10 +263,10 @@ class TestFileMapping(TestCase):
     def test_usemap(self):
         map = self.apply_mappings(
                 {(1, ""): {
-                                   u"foo": ('A', None, None), 
-                                   u"foo/bla": ('A', None, None)},
+                                   u"foo": ('A', None), 
+                                   u"foo/bla": ('A', None)},
                  (2, ""): {
-                                   u"foo/bla": ('M', None, None)}
+                                   u"foo/bla": ('M', None)}
                  }, 
                 renames={(1, ""): {u"foo": "myid"}})
         self.assertEqual("myid", map[u"foo"][0])
@@ -268,10 +274,10 @@ class TestFileMapping(TestCase):
     def test_usemap_later(self):
         map = self.apply_mappings(
                 {(1, ""): {
-                                   u"foo": ('A', None, None), 
-                                   u"foo/bla": ('A', None, None)},
+                                   u"foo": ('A', None), 
+                                   u"foo/bla": ('A', None)},
                  (2, ""): {
-                                   u"foo/bla": ('M', None, None)}
+                                   u"foo/bla": ('M', None)}
                  }, 
                 renames={(2, ""): {u"foo": "myid"}})
         self.assertEqual("1@uuid::foo", map[u"foo"][0])
@@ -279,17 +285,22 @@ class TestFileMapping(TestCase):
 
 
 class GetMapTests(SubversionTestCase):
+
     def setUp(self):
         super(GetMapTests, self).setUp()
         self.repos_url = self.make_repository("d")
         self.repos = Repository.open(self.repos_url)
 
+    def get_map(self, path, revnum, mapping):
+        rm_provider = self.repos._revmeta_provider
+        revmeta = rm_provider.get_revision(path, revnum)
+        fileid_map = self.repos.get_fileid_map(revmeta, mapping)
+        return fileid_map.as_dict()
+
     def test_empty(self):
         self.repos.set_layout(RootLayout())
         self.mapping = self.repos.get_mapping()
-        rm_provider = self.repos._revmeta_provider
-        self.assertEqual({"": (self.mapping.generate_file_id((self.repos.uuid, "", 0), u""), self.repos.generate_revision_id(0, "", self.mapping), None)}, 
-                         self.repos.get_fileid_map(rm_provider.get_revision("", 0), self.mapping))
+        self.assertEqual({"": (self.mapping.generate_file_id((self.repos.uuid, "", 0), u""), self.repos.generate_revision_id(0, "", self.mapping), None)}, self.get_map("", 0, self.mapping))
 
     def test_empty_trunk(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -298,9 +309,8 @@ class GetMapTests(SubversionTestCase):
         dc.add_dir("trunk")
         dc.close()
 
-        rm_provider = self.repos._revmeta_provider
         self.assertEqual({"": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), self.repos.generate_revision_id(1, "trunk", self.mapping), None)}, 
-                self.repos.get_fileid_map(rm_provider.get_revision("trunk", 1), self.mapping))
+                self.get_map("trunk", 1, self.mapping))
 
     def test_change_parent(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -314,8 +324,6 @@ class GetMapTests(SubversionTestCase):
         dc.open_dir("trunk").add_file("trunk/file").modify("data")
         dc.close()
 
-        rm_provider = self.repos._revmeta_provider
-
         self.assertEqual({
             "": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), 
                  self.repos.generate_revision_id(1, "trunk", self.mapping),
@@ -323,7 +331,7 @@ class GetMapTests(SubversionTestCase):
             "file": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 2), u"file"), 
                      self.repos.generate_revision_id(2, "trunk", self.mapping),
                      None)}, 
-            self.repos.get_fileid_map(rm_provider.get_revision("trunk", 2), self.mapping))
+            self.get_map("trunk", 2, self.mapping))
 
     def test_change_updates(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -340,7 +348,6 @@ class GetMapTests(SubversionTestCase):
         dc = self.get_commit_editor(self.repos_url)
         dc.open_dir("trunk").open_file("trunk/file").modify("otherdata")
         dc.close()
-        rm_provider = self.repos._revmeta_provider
 
         self.assertEqual({
             "": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), 
@@ -349,7 +356,7 @@ class GetMapTests(SubversionTestCase):
             "file": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 2), u"file"), 
                      self.repos.generate_revision_id(3, "trunk", self.mapping),
                      None)}, 
-            self.repos.get_fileid_map(rm_provider.get_revision("trunk", 3), self.mapping))
+            self.get_map("trunk", 3, self.mapping))
 
     def test_sibling_unrelated(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -370,7 +377,6 @@ class GetMapTests(SubversionTestCase):
         trunk.open_file("trunk/file").modify('otherdata')
         dc.close()
 
-        rm_provider = self.repos._revmeta_provider
         self.assertEqual({
             "": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), 
                  self.repos.generate_revision_id(1, "trunk", self.mapping),
@@ -381,7 +387,7 @@ class GetMapTests(SubversionTestCase):
             "file": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 2), u"file"), 
                      self.repos.generate_revision_id(3, "trunk", self.mapping),
                      None)}, 
-            self.repos.get_fileid_map(rm_provider.get_revision("trunk", 3), self.mapping))
+            self.get_map("trunk", 3, self.mapping))
 
     def test_copy(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -401,8 +407,6 @@ class GetMapTests(SubversionTestCase):
         trunk.add_file("trunk/bar", "trunk/file", 2)
         dc.close()
 
-        rm_provider = self.repos._revmeta_provider
-
         self.assertEqual({
             "": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), 
                  self.repos.generate_revision_id(1, "trunk", self.mapping),
@@ -413,7 +417,7 @@ class GetMapTests(SubversionTestCase):
             "file": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 2), u"file"), 
                      self.repos.generate_revision_id(2, "trunk", self.mapping),
                      None)}, 
-            self.repos.get_fileid_map(rm_provider.get_revision("trunk", 3), self.mapping))
+            self.get_map("trunk", 3, self.mapping))
 
     def test_copy_nested_modified(self):
         self.repos.set_layout(TrunkLayout(0))
@@ -435,8 +439,6 @@ class GetMapTests(SubversionTestCase):
         dir.open_file("trunk/bar/file").modify("data2")
         dc.close()
 
-        rm_provider = self.repos._revmeta_provider
-
         self.assertEqual({
           "": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 1), u""), 
             self.repos.generate_revision_id(1, "trunk", self.mapping),
@@ -453,7 +455,7 @@ class GetMapTests(SubversionTestCase):
           "bar/file": (self.mapping.generate_file_id((self.repos.uuid, "trunk", 3), u"bar/file"), 
               self.repos.generate_revision_id(3, "trunk", self.mapping),
               (self.repos.uuid, "trunk", 3))},
-            self.repos.get_fileid_map(rm_provider.get_revision("trunk", 3), self.mapping))
+            self.get_map("trunk", 3, self.mapping))
 
     def test_304134(self):
         self.make_checkout(self.repos_url, 'svn-co')
@@ -533,11 +535,11 @@ class LookupTests(TestCase):
 
     def test_trivial(self):
         idmap = {"filename": ("myfileid", "myrev", None)}
-        self.assertEquals(("myfileid", "myrev", None), idmap_lookup(idmap, None, "filename"))
+        self.assertEquals(("myfileid", "myrev", None), idmap_lookup(idmap.__getitem__, None, "filename"))
 
     def test_nonexistent(self):
         idmap = {}
-        self.assertRaises(KeyError, idmap_lookup, idmap, None, "filename")
+        self.assertRaises(KeyError, idmap_lookup, idmap.__getitem__, None, "filename")
 
     def test_implicit(self):
         idmap = {"parent": ("parentfileid", "parentrev", ("someuuid", "somebp", 42))}
@@ -545,12 +547,12 @@ class LookupTests(TestCase):
         self.assertEquals((mapping.generate_file_id(("someuuid", "somebp", 42), u"parent/foo"),
                            mapping.revision_id_foreign_to_bzr(("someuuid", "somebp", 42)),
                          ('someuuid', 'somebp', 42)),
-                          idmap_lookup(idmap, mapping, u"parent/foo"))
+                          idmap_lookup(idmap.__getitem__, mapping, u"parent/foo"))
 
     def test_not_implicit_asserts(self):
         idmap = {"parent": ("parentfileid", "parentrev", None)}
         mapping = mapping_registry.get_default()()
-        self.assertRaises(AssertionError, idmap_lookup, idmap, mapping, u"parent/foo")
+        self.assertRaises(KeyError, idmap_lookup, idmap.__getitem__, mapping, u"parent/foo")
 
 
 class ReverseLookupTests(TestCase):
@@ -578,3 +580,20 @@ class ReverseLookupTests(TestCase):
         self.assertRaises(KeyError, 
                           idmap_reverse_lookup, idmap, mapping, 
                           mapping.generate_file_id(("someuuid", "somebp", 42), u"parent/foo"))
+
+
+class LocalChangesTests(TestCase):
+
+    def _generate_revid(self, revnum, path):
+        return "%s:%d" % (path, revnum)
+
+    def test_trivial_change(self):
+        paths = { "trunk/path": ("M", None, -1)}
+        branch = "trunk"
+        self.assertEquals({"path": ("M", None)}, get_local_changes(paths, branch))
+
+    def test_bug_352509(self):
+        paths = { "trunk/internal/org.restlet": ('R', 'trunk/plugins/internal/org.restlet', 1111) }
+        branch = "trunk/internal/org.restlet"
+        self.assertEquals({"": ("M", None)},
+            get_local_changes(paths, branch))
