@@ -21,7 +21,12 @@ from bzrlib.revision import Revision
 from bzrlib.tests import TestCase
 
 from bzrlib.plugins.svn import mapping
-from bzrlib.plugins.svn.errors import InvalidPropertyValue
+from bzrlib.plugins.svn.errors import (
+    InvalidPropertyValue,
+    )
+from bzrlib.plugins.svn.layout.standard import (
+    TrunkLayout,
+    )
 from bzrlib.plugins.svn.mapping import (
     estimate_bzr_ancestors,
     escape_svn_path,
@@ -30,6 +35,7 @@ from bzrlib.plugins.svn.mapping import (
     get_roundtrip_ancestor_revids,
     is_bzr_revision_fileprops,
     is_bzr_revision_revprops,
+    mapping_registry,
     parse_merge_property,
     parse_revision_metadata, 
     parse_revid_property,
@@ -41,6 +47,7 @@ from bzrlib.plugins.svn.mapping2 import (
     BzrSvnMappingv2,
     )
 from bzrlib.plugins.svn.mapping3.base import BzrSvnMappingv3
+from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
 from bzrlib.plugins.svn.mapping4 import BzrSvnMappingv4
 
 
@@ -65,6 +72,13 @@ class MetadataMarshallerTests(TestCase):
                 generate_revision_metadata(None, None,
                     None, {"propbla": "bloe", "propfoo": "bla"}))
 
+    def test_generate_revision_metadata_properties_newline(self):
+        self.assertEquals("properties: \n" + 
+                "\tpropbla: bloe\n" +
+                "\tpropbla: bla\n",
+                generate_revision_metadata(None, None,
+                    None, {"propbla": "bloe\nbla"}))
+
     def test_parse_revision_metadata_empty(self):
         parse_revision_metadata("", None)
 
@@ -97,6 +111,14 @@ class MetadataMarshallerTests(TestCase):
                                 "\tha: ha\n", rev)
         self.assertEquals({"foo": "bar", "ha": "ha"}, rev.properties)
 
+    def test_parse_revision_metadata_properties_newline(self):
+        rev = Revision('someid')
+        parse_revision_metadata("properties: \n" + 
+                                "\tfoo: bar\n" + 
+                                "\tfoo: bar2\n" +
+                                "\tha: ha\n", rev)
+        self.assertEquals({"foo": "bar\nbar2", "ha": "ha"}, rev.properties)
+
     def test_parse_revision_metadata_no_colon(self):
         rev = Revision('someid')
         self.assertRaises(InvalidPropertyValue, 
@@ -163,11 +185,26 @@ def sha1(text):
     return sha(text).hexdigest()
 
 
-class ParseRevisionIdTests(object):
+class GenerateRevisionIdTests(TestCase):
+
+    def test_v4_space(self):
+        self.assertEqual("svn-v4:uuid:trunk%20l:1",
+            BzrSvnMappingv4().revision_id_foreign_to_bzr(("uuid", "trunk l", 1)))
+
+    def test_v4(self):
+        self.assertEqual("svn-v4:uuid:trunk:1",
+            BzrSvnMappingv4().revision_id_foreign_to_bzr(("uuid", "trunk", 1)))
+
+    def test_v4_slash(self):
+        self.assertEqual("svn-v4:uuid:project/trunk:1",
+            BzrSvnMappingv4().revision_id_foreign_to_bzr(("uuid", "project/trunk", 1)))
+
+
+class ParseRevisionIdTests(TestCase):
 
     def test_v4(self):
         self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv4()), 
-                mapping_registry.parse_revision_id("svn-v3:uuid:trunk:1"))
+                mapping_registry.parse_revision_id("svn-v4:uuid:trunk:1"))
 
     def test_v3(self):
         self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv3(TrunkBranchingScheme())), 
@@ -178,15 +215,15 @@ class ParseRevisionIdTests(object):
                 mapping_registry.parse_revision_id("svn-v3-undefined:uuid:trunk:1"))
 
     def test_v2(self):
-        self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv2()), 
+        self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv2(TrunkLayout())), 
                          mapping_registry.parse_revision_id("svn-v2:1@uuid-trunk"))
 
     def test_v1(self):
-        self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv1()), 
+        self.assertEqual((("uuid", "trunk", 1), BzrSvnMappingv1(TrunkLayout())), 
                          mapping_registry.parse_revision_id("svn-v1:1@uuid-trunk"))
 
     def test_except(self):
-        self.assertRaises(InvalidRevisionId, 
+        self.assertRaises(KeyError, 
                          mapping_registry.parse_revision_id, "svn-v0:1@uuid-trunk")
 
     def test_except_nonsvn(self):
