@@ -26,6 +26,7 @@ from bzrlib.errors import (
     AlreadyBranchError,
     BzrError,
     DivergedBranches,
+    AppendRevisionsOnlyViolation,
     )
 from bzrlib.inventory import InventoryDirectory
 from bzrlib.merge import (
@@ -42,6 +43,9 @@ from bzrlib.plugins.svn import (
     format,
     transport,
     )
+from bzrlib.plugins.svn.branch import (
+    InterOtherSvnBranch,
+    )
 from bzrlib.plugins.svn.errors import MissingPrefix
 from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
@@ -51,9 +55,8 @@ from bzrlib.plugins.svn.mapping import (
     SVN_PROP_BZR_REVISION_ID,
     )
 from bzrlib.plugins.svn.push import (
+    InterToSvnRepository,
     determine_branch_path,
-    dpush, 
-    push,
     )
 from bzrlib.plugins.svn.tests import (
     SubversionTestCase,
@@ -76,19 +79,27 @@ class TestDPush(SubversionTestCase):
         os.mkdir("dc")
         self.bzrdir = self.svndir.sprout("dc")
 
-
     def tearDown(self):
+        TestCase.tearDown(self)
         transport.disabled_capabilities = set()
 
-    def commit_editor(self):
-        return self.get_commit_editor(self.repos_url)
+    def commit_editor(self, message="Test commit"):
+        return self.get_commit_editor(self.repos_url, message)
+
+    def dpush(self, source, target):
+        return InterOtherSvnBranch(source, target)._update_revisions_lossy()
 
     def test_change_single(self):
         self.build_tree({'dc/foo/bla': 'other data'})
         wt = self.bzrdir.open_workingtree()
         newid = wt.commit(message="Commit from Bzr")
 
-        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+        source_branch = self.bzrdir.open_branch()
+        source_branch.lock_write()
+        try:
+            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+        finally:
+            source_branch.unlock()
 
         self.assertEquals([newid], revid_map.keys())
 
@@ -112,7 +123,12 @@ class TestDPush(SubversionTestCase):
         self.build_tree({'dc/foo/bla': 'yet other data'})
         newid2 = wt.commit(message="Commit from Bzr")
 
-        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+        source_branch = self.bzrdir.open_branch()
+        source_branch.lock_write()
+        try:
+            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+        finally:
+            source_branch.unlock()
 
         self.assertEquals(set([newid1, newid2]), set(revid_map.keys()))
 
@@ -138,7 +154,12 @@ class TestDPush(SubversionTestCase):
         wt.add(["foo/bliel"])
         newid2 = wt.commit(message="Commit from Bzr")
 
-        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+        source_branch = self.bzrdir.open_branch()
+        source_branch.lock_write()
+        try:
+            revid_map = self.dpush(source_branch, self.svndir.open_branch())
+        finally:
+            source_branch.unlock()
 
         self.assertEquals(set([newid1, newid2]), set(revid_map.keys()))
         repos = self.svndir.find_repository()
@@ -158,9 +179,14 @@ class TestDPush(SubversionTestCase):
         wt.add('file')
         wt.commit(message="Commit from Bzr")
 
-        self.assertRaises(DivergedBranches, 
-                          dpush, svndir.open_branch(),
-                          self.bzrdir.open_branch())
+        source_branch = self.bzrdir.open_branch()
+        source_branch.lock_write()
+        try:
+            self.assertRaises(DivergedBranches, 
+                              self.dpush, source_branch, 
+                              svndir.open_branch())
+        finally:
+            source_branch.unlock()
 
 
 class TestPush(SubversionTestCase):
@@ -178,8 +204,8 @@ class TestPush(SubversionTestCase):
         os.mkdir("dc")
         self.bzrdir = self.svndir.sprout("dc")
 
-    def commit_editor(self):
-        return self.get_commit_editor(self.repos_url)
+    def commit_editor(self, message="Test commit"):
+        return self.get_commit_editor(self.repos_url, message)
 
     def test_empty(self):
         svnbranch = self.svndir.open_branch()
@@ -462,15 +488,15 @@ class TestPush(SubversionTestCase):
                 lambda: Branch.open(oc_url).pull(self.bzrdir.open_branch()))
 
     def test_different_branch_path(self):
-        # A       ,> C
+        # A       ,> C -> D
         # \ -> B /
-        dc = self.commit_editor()
+        dc = self.commit_editor("Test commit 1")
         trunk = dc.add_dir("trunk")
         trunk.add_file('trunk/foo').modify("data")
         dc.add_dir("branches")
         dc.close()
 
-        dc = self.commit_editor()
+        dc = self.commit_editor("Test commit 2")
         branches = dc.open_dir('branches')
         mybranch = branches.add_dir('branches/mybranch', 'trunk')
         mybranch.open_file("branches/mybranch/foo").modify('data2')
@@ -483,21 +509,18 @@ class TestPush(SubversionTestCase):
         self.build_tree({'mybranch/foo': 'bladata'})
         wt = self.bzrdir.open_workingtree()
         revid = wt.commit(message="Commit from Bzr")
+
         b = Branch.open("%s/trunk" % self.repos_url)
-        push(b.repository.get_graph(), b.repository, b.get_branch_path(),
-                b.get_config(), b.last_revision(), wt.branch.repository, 
-             wt.branch.repository.get_revision(wt.branch.revision_history()[-2]))
+        wt.branch.push(b, stop_revision=wt.branch.revision_history()[-2])
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0])
         if not b.mapping.can_use_revprops and b.mapping.can_use_fileprops:
             self.assertEquals('M',
                 self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0]['/trunk'][0])
         b = Branch.open("%s/trunk" % self.repos_url)
-        push(b.repository.get_graph(), b.repository, b.get_branch_path(),
-                b.get_config(), b.last_revision(), wt.branch.repository, 
-                wt.branch.repository.get_revision(wt.branch.last_revision()))
+        wt.branch.push(b)
         mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
-        self.assertEquals("/branches/mybranch", 
-            self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0]['/trunk'][1])
+        self.assertEquals({'/trunk/foo': ('M', None, -1)},
+            self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
 
     def test_comics(self):
         dc = self.commit_editor()
@@ -552,9 +575,11 @@ class PushNewBranchTests(SubversionTestCase):
         newtree = newrepos.revision_tree(revid)
 
         bzrwt.lock_read()
-        self.assertEquals(bzrwt.inventory.root.file_id,
-                          newtree.inventory.root.file_id)
-        bzrwt.unlock()
+        try:
+            self.assertEquals(bzrwt.inventory.root.file_id,
+                              newtree.inventory.root.file_id)
+        finally:
+            bzrwt.unlock()
 
     def test_single_revision(self):
         repos_url = self.make_repository("a")
@@ -563,9 +588,11 @@ class PushNewBranchTests(SubversionTestCase):
         newbranch = newdir.import_branch(bzrwt.branch)
         newtree = newbranch.repository.revision_tree(revid)
         bzrwt.lock_read()
-        self.assertEquals(bzrwt.inventory.root.file_id,
-                          newtree.inventory.root.file_id)
-        bzrwt.unlock()
+        try:
+            self.assertEquals(bzrwt.inventory.root.file_id,
+                              newtree.inventory.root.file_id)
+        finally:
+            bzrwt.unlock()
         self.assertEquals(revid, newbranch.last_revision())
         self.assertEquals([revid], newbranch.revision_history())
 
@@ -681,7 +708,10 @@ class PushNewBranchTests(SubversionTestCase):
         finally:
             wt.unlock()
         self.assertTrue(os.path.exists("bzrco/baz.txt"))
-        wt.branch.push(Branch.open(repos_url+"/trunk"))
+        target_branch = Branch.open(repos_url+"/trunk")
+        self.assertRaises(AppendRevisionsOnlyViolation, wt.branch.push, target_branch)
+        target_branch.get_config().set_user_option('append_revisions_only', 'False')
+        wt.branch.push(target_branch)
 
     def test_push_merge_unchanged_file(self):
         def check_tree(t):
@@ -920,8 +950,6 @@ class PushNewBranchTests(SubversionTestCase):
         self.assertEquals(bzrwt2.branch.revision_history(),
                 Branch.open(repos_url+"/trunk").revision_history())
 
-
-
     def test_complex_rename(self):
         repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
@@ -988,16 +1016,18 @@ class PushNewBranchTests(SubversionTestCase):
         bzrwt.add(["foo"])
         revid1 = bzrwt.commit("Initial")
         bzrwt.lock_write()
-        old_ie = bzrwt.inventory.root.copy()
-        new_ie = bzrwt.inventory.root.copy()
-        foo_ie = bzrwt.inventory[bzrwt.path2id("foo")].copy()
-        new_ie.file_id = "mynewroot"
-        foo_ie.parent_id = new_ie.file_id
-        bzrwt.apply_inventory_delta([
-            ("", None, old_ie.file_id, None),
-            (None, "", new_ie.file_id, new_ie),
-            ("foo", "foo", foo_ie.file_id, foo_ie)])
-        bzrwt.unlock()
+        try:
+            old_ie = bzrwt.inventory.root.copy()
+            new_ie = bzrwt.inventory.root.copy()
+            foo_ie = bzrwt.inventory[bzrwt.path2id("foo")].copy()
+            new_ie.file_id = "mynewroot"
+            foo_ie.parent_id = new_ie.file_id
+            bzrwt.apply_inventory_delta([
+                ("", None, old_ie.file_id, None),
+                (None, "", new_ie.file_id, new_ie),
+                ("foo", "foo", foo_ie.file_id, foo_ie)])
+        finally:
+            bzrwt.unlock()
         revid2 = bzrwt.commit(message="Commit from Bzr")
         return bzrwt, old_ie, new_ie, foo_ie, revid1, revid2
 
@@ -1113,6 +1143,9 @@ class PushNewBranchTests(SubversionTestCase):
         revid3 = bzrwt.commit("Merge something", rev_id="mergerevid")
 
         trunk = Branch.open(repos_url + "/branches/foo")
+        self.assertRaises(AppendRevisionsOnlyViolation, trunk.pull, 
+            bzrwt.branch)
+        trunk.get_config().set_user_option('append_revisions_only', 'False')
         trunk.pull(bzrwt.branch)
 
         self.assertEquals([revid1, revid2, revid3], trunk.revision_history())
@@ -1154,13 +1187,15 @@ class PushNewBranchTests(SubversionTestCase):
         bzrdir = svndir.sprout("dc")
         wt = bzrdir.open_workingtree()
         wt.lock_write()
-        wt.apply_inventory_delta([
-            ("", None, wt.inventory.root.file_id, None),
-            ("mysubdir", "", wt.inventory.path2id("mysubdir"), 
-                InventoryDirectory(wt.inventory.path2id("mysubdir"), "", None))])
-        os.rename("dc/mysubdir/myfile", "dc/myfile")
-        osutils.rmtree("dc/mysubdir")
-        wt.unlock()
+        try:
+            wt.apply_inventory_delta([
+                ("", None, wt.inventory.root.file_id, None),
+                ("mysubdir", "", wt.inventory.path2id("mysubdir"), 
+                    InventoryDirectory(wt.inventory.path2id("mysubdir"), "", None))])
+            os.rename("dc/mysubdir/myfile", "dc/myfile")
+            osutils.rmtree("dc/mysubdir")
+        finally:
+            wt.unlock()
         wt.commit("Change branch root")
         svnbranch = svndir.open_branch()
         svnbranch.pull(wt.branch)
@@ -1182,10 +1217,13 @@ class PushNewBranchTests(SubversionTestCase):
 
         wt = bzrdir.open_workingtree()
         wt.lock_write()
-        wt.commit("This is pointless.")
-        svnbranch = svndir.open_branch()
-        svnbranch.pull(wt.branch)
-        self.assertEquals(svnbranch.last_revision(), wt.branch.last_revision())
+        try: 
+            wt.commit("This is pointless.")
+            svnbranch = svndir.open_branch()
+            svnbranch.pull(wt.branch)
+            self.assertEquals(svnbranch.last_revision(), wt.branch.last_revision())
+        finally:
+            wt.unlock()
  
 
 class TestPushTwice(SubversionTestCase):
