@@ -16,6 +16,7 @@
 
 """Subversion transport tests."""
 
+import stat
 import subvertpy
 from subvertpy import ra
 from unittest import TestCase
@@ -25,6 +26,7 @@ from bzrlib.errors import (
     FileExists,
     InvalidURL,
     NoSuchFile,
+    TransportNotPossible,
     )
 
 from bzrlib.plugins.svn.tests import SubversionTestCase
@@ -37,15 +39,16 @@ from bzrlib.plugins.svn.transport import (
     )
 
 class SvnRaTest(SubversionTestCase):
+
     def test_open_nonexisting(self):
         self.assertRaises(InvalidURL, SvnRaTransport, 
                           "svn+nonexisting://foo/bar")
 
     def test_create(self):
         repos_url = self.make_repository('a')
-        t = SvnRaTransport("svn+%s" % repos_url)
+        t = SvnRaTransport(repos_url)
         self.assertIsInstance(t, SvnRaTransport)
-        self.assertEqual(t.base, "svn+%s" % repos_url)
+        self.assertEqual(t.base, repos_url)
 
     def test_create_direct(self):
         repos_url = self.make_repository('a')
@@ -144,7 +147,16 @@ class SvnRaTest(SubversionTestCase):
 
     def test_has_dot(self):
         t = SvnRaTransport(self.make_repository('a'))
-        self.assertEqual(False, t.has("."))
+        self.assertEqual(True, t.has("."))
+
+    def test_stat(self):
+        t = SvnRaTransport(self.make_repository('a'))
+        try:
+            statob = t.stat(".")
+        except TransportNotPossible:
+            pass
+        else:
+            self.assertTrue(stat.S_ISDIR(statob.st_mode))
 
     def test_has_nonexistent(self):
         t = SvnRaTransport(self.make_repository('a'))
