@@ -24,7 +24,10 @@ from bzrlib.inventory import (
     )
 from bzrlib.osutils import has_symlinks
 from bzrlib.repository import Repository
-from bzrlib.revision import NULL_REVISION
+from bzrlib.revision import (
+    NULL_REVISION,
+    CURRENT_REVISION,
+    )
 from bzrlib.tests import TestSkipped
 from bzrlib.workingtree import WorkingTree
 
@@ -161,6 +164,28 @@ class TestBasisTree(SubversionTestCase):
             (repo.generate_revision_id(2, "", repo.get_mapping()), "y\n")],
             tree.annotate_iter(tree.path2id("file")))
 
+    def test_get_dir_properties(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/bla": None})
+        self.client_add("dc/bla")
+        self.client_set_prop("dc/bla", "bzrbla", "bloe")
+        self.client_commit('dc', 'msg')
+        self.client_set_prop("dc/bla", "bzrbla", "bloe2")
+        t = WorkingTree.open('dc').basis_tree()
+        props = t.get_file_properties(t.path2id('bla'), 'bla')
+        self.assertEquals("bloe", props["bzrbla"])
+
+    def test_get_file_properties(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/bla": "data"})
+        self.client_add("dc/bla")
+        self.client_set_prop("dc/bla", "bzrbla", "bloe")
+        self.client_commit('dc', 'msg')
+        self.client_set_prop("dc/bla", "bzrbla", "bloe2")
+        t = WorkingTree.open('dc').basis_tree()
+        props = t.get_file_properties(t.path2id('bla'), 'bla')
+        self.assertEquals("bloe", props["bzrbla"])
+
     def test_executable_link(self):
         if not has_symlinks():
             return
@@ -187,6 +212,7 @@ class TestBasisTree(SubversionTestCase):
 
 
 class TestInventoryExternals(SubversionTestCase):
+
     def test_add_nested_norev(self):
         """Add a nested tree with no specific revision referenced."""
         repos_url = self.make_client('d', 'dc')
@@ -199,6 +225,7 @@ class TestInventoryExternals(SubversionTestCase):
         self.assertEqual(TreeReference(
             mapping.generate_file_id((repos.uuid, "", 0), u""),
              'bla', inv.path2id('blie'), 
+             reference_revision=CURRENT_REVISION,
              revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))), 
              inv[inv.path2id('blie/bla')])
 
@@ -214,6 +241,7 @@ class TestInventoryExternals(SubversionTestCase):
         self.assertEqual(TreeReference(
             mapping.generate_file_id((repos.uuid, "", 0), u""),
              'bla', 'blabloe', 
+             reference_revision=CURRENT_REVISION,
              revision=mapping.revision_id_foreign_to_bzr((repos.uuid, "", 1))), 
              inv[inv.path2id('bla')])
 
@@ -236,6 +264,7 @@ class TestInventoryExternals(SubversionTestCase):
 
 
 class TestSvnRevisionTree(SubversionTestCase):
+
     def setUp(self):
         super(TestSvnRevisionTree, self).setUp()
         repos_url = self.make_client('d', 'dc')
