@@ -109,6 +109,18 @@ class TestWorkingTree(SubversionTestCase):
         self.assertTrue(inv.has_filename("bl"))
         self.assertFalse(inv.has_filename("aa"))
 
+    def test_smart_add_unicode(self):
+        try:
+            self.make_client('a', u'dć'.encode(osutils._fs_enc))
+        except UnicodeDecodeError:
+            raise TestSkipped("This platform does not support unicode paths")
+        self.build_tree({u"dć/bé".encode(osutils._fs_enc): "data"})
+        tree = WorkingTree.open(u"dć")
+        tree.smart_add([u"dć/bé"])
+        inv = tree.read_working_inventory()
+        self.assertIsInstance(inv, Inventory)
+        self.assertTrue(inv.has_filename(u"bé"))
+
     def test_is_control_filename(self):
         self.make_client('a', 'dc')
         bzrdir = BzrDir.open("dc")
@@ -231,16 +243,12 @@ class TestWorkingTree(SubversionTestCase):
         self.make_client('a', 'dc')
         tree = WorkingTree.open("dc")
         tree.lock_write()
+        tree.unlock()
 
     def test_lock_read(self):
         self.make_client('a', 'dc')
         tree = WorkingTree.open("dc")
         tree.lock_read()
-
-    def test_unlock(self):
-        self.make_client('a', 'dc')
-        tree = WorkingTree.open("dc")
-        tree.lock_read()
         tree.unlock()
 
     def test_get_ignore_list_empty(self):
@@ -509,7 +517,8 @@ class TestWorkingTree(SubversionTestCase):
         self.build_tree({"dc/bl": None})
 
         tree = WorkingTree.open("dc")
-        self.assertEqual(['.svn', 'bl'], list(tree.extras()))
+        extras = list(tree.extras())
+        self.assertEqual([u'bl'], extras)
 
     def test_executable(self):
         if not supports_executable():
@@ -752,8 +761,34 @@ class TestWorkingTree(SubversionTestCase):
         self.client_commit("dc", "Added files and links")
         WorkingTree.open("dc").read_working_inventory()
 
+    def test_get_dir_properties(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/bla": None})
+        self.client_add("dc/bla")
+        self.client_set_prop("dc/bla", "bzrbla", "bloe")
+        wt = WorkingTree.open('dc')
+        props = wt.get_file_properties(wt.path2id('bla'), 'bla')
+        self.assertEquals("bloe", props["bzrbla"])
+
+    def test_get_file_properties(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/bla": "data"})
+        self.client_add("dc/bla")
+        self.client_set_prop("dc/bla", "bzrbla", "bloe")
+        wt = WorkingTree.open('dc')
+        props = wt.get_file_properties(wt.path2id('bla'), 'bla')
+        self.assertEquals("bloe", props["bzrbla"])
+
+    def test_invdelta_remove(self):
+        repos_url = self.make_client('a', 'dc')
+        self.build_tree({"dc/bla": "data"})
+        self.client_add("dc/bla")
+        wt = WorkingTree.open('dc')
+        wt.apply_inventory_delta([("bla", None, wt.path2id("bla"), None)])
+
 
 class IgnoreListTests(TestCase):
+
     def test_empty(self):
         self.assertEquals([], generate_ignore_list({}))
 
