@@ -16,6 +16,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Simple transport for accessing Subversion smart servers."""
 
+import itertools
+import stat
 import subvertpy
 from subvertpy import ERR_FS_NOT_DIRECTORY
 from subvertpy.client import get_config
@@ -29,28 +31,42 @@ from bzrlib import (
     urlutils,
     )
 from bzrlib.errors import (
-    NoSuchFile,
-    TransportNotPossible, 
+    BzrError,    
     FileExists,
-    NotLocalUrl,
     InvalidURL,
+    NoSuchFile,
+    NotLocalUrl,
     RedirectRequested,
+    TransportNotPossible, 
     )
 from bzrlib.trace import (
     mutter,
     warning,
     )
-from bzrlib.transport import Transport
+from bzrlib.transport import (
+    Transport,
+    )
 
 import bzrlib.plugins.svn
-from bzrlib.plugins.svn.auth import create_auth_baton
+from bzrlib.plugins.svn.auth import (
+    AUTH_PARAM_DEFAULT_USERNAME,
+    AUTH_PARAM_DEFAULT_PASSWORD,
+    create_auth_baton,
+    )
 from bzrlib.plugins.svn.errors import (
     convert_svn_error,
     NoSvnRepositoryPresent,
     convert_error,
     )
 
-svn_config = get_config()
+ERR_MALFORMED_FILE = getattr(subvertpy, "ERR_MALFORMED_FILE", 200002) 
+
+try:
+    svn_config = get_config()
+except subvertpy.SubversionException, (msg, num):
+    if num == ERR_MALFORMED_FILE:
+        raise BzrError(msg)
+    raise
 
 
 # This variable is here to allow tests to temporarily disable features
@@ -73,9 +89,35 @@ def get_svn_ra_transport(bzr_transport):
     if ra_transport is not None:
         return ra_transport
 
+    svnbase = bzr_transport.external_url()
+    if svnbase.startswith("readonly+"):
+        svnbase = svnbase[len("readonly+"):]
+    # If this is a HTTP transport, use the existing connection to check 
+    # that the remote end supports version control.
+    from bzrlib.transport.http._urllib import HttpTransport_urllib, Request
+    if isinstance(bzr_transport, HttpTransport_urllib):
+        url = bzr_transport._unsplit_url(bzr_transport._unqualified_scheme,
+            None, None, bzr_transport._host, bzr_transport._port, 
+            bzr_transport._path)
+        req = Request('OPTIONS', url, accepted_errors=[200]) 
+        req.follow_redirections = True
+        resp = bzr_transport._perform(req)
+        dav_entries = list(itertools.chain(*[entry.split(",") for entry in resp.headers.getheaders('DAV')]))
+        if not "version-control" in dav_entries:
+            raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
     # Save _svn_ra transport here so we don't have to connect again next time
     # we try to use bzr svn on this transport
-    ra_transport = SvnRaTransport(bzr_transport.base)
+    shared_connection = getattr(bzr_transport, "_shared_connection", None)
+    if shared_connection is not None and shared_connection.credentials is not None:
+        if isinstance(shared_connection.credentials, dict):
+            creds = shared_connection.credentials
+        elif isinstance(shared_connection.credentials[0], dict):
+            creds = shared_connection.credentials[0]
+        else:
+            creds = None
+    else:
+        creds = None
+    ra_transport = SvnRaTransport(svnbase, credentials=creds)
     bzr_transport._svn_ra = ra_transport
     return ra_transport
 
@@ -102,25 +144,14 @@ def url_join_unescaped_path(url, path):
     return _url_escape_uri(urlutils.join(url, path))
 
 
-svnplus_warning_showed = False
-
-def warn_svnplus(url):
-    global svnplus_warning_showed
-    if not svnplus_warning_showed:
-        warning("The svn+ syntax is deprecated, use %s instead.", url)
-        svnplus_warning_showed = True
-
-
 def bzr_to_svn_url(url):
     """Convert a Bazaar URL to a URL understood by Subversion.
 
     This will possibly remove the svn+ prefix.
     """
     if (url.startswith("svn+http://") or 
-        url.startswith("svn+file://") or
         url.startswith("svn+https://")):
         url = url[len("svn+"):] # Skip svn+
-        warn_svnplus(url)
 
     url = _url_unescape_uri(url)
 
@@ -132,12 +163,12 @@ def bzr_to_svn_url(url):
 
 class SubversionProgressReporter(object):
 
-    def __init__(self):
+    def __init__(self, url):
+        self._scheme = urlparse.urlsplit(url)[0]
         self._last_progress = 0
         # This variable isn't used yet as of bzr 1.12, and finding 
         # the right Transport object will be tricky in bzr-svn's case
         # so just setting it to None for now.
-        self._transport = None
 
     def update(self, progress, total):
         # The counter seems to reset sometimes
@@ -147,26 +178,26 @@ class SubversionProgressReporter(object):
         changed = progress - self._last_progress
         assert changed >= 0, "changed was %d (%d -> %d)" % (changed, self._last_progress, progress)
         self._last_progress = progress
-        ui.ui_factory.report_transport_activity(self._transport, changed, None)
+        ui.ui_factory.report_transport_activity(self, changed, None)
 
 
-def Connection(url):
+def Connection(url, auth=None):
     # Subvertpy <= 0.6.3 has a bug in the reference counting of the 
     # progress update function
     if subvertpy.__version__ >= (0, 6, 3):
-        progress_cb = SubversionProgressReporter().update
+        progress_cb = SubversionProgressReporter(url).update
     else:
         progress_cb = None
-
     try:
         mutter('opening SVN RA connection to %r' % url)
         ret = subvertpy.ra.RemoteAccess(url.encode('utf8'), 
-                auth=create_auth_baton(url),
+                auth=auth,
                 client_string_func=bzrlib.plugins.svn.get_client_string,
                 progress_cb=progress_cb)
         if 'transport' in debug.debug_flags:
             ret = MutteringRemoteAccess(ret)
-    except subvertpy.SubversionException, (msg, num):
+    except subvertpy.SubversionException, e:
+        msg, num = e.args
         if num in (subvertpy.ERR_RA_SVN_REPOS_NOT_FOUND,):
             raise NoSvnRepositoryPresent(url=url)
         if num == subvertpy.ERR_BAD_URL:
@@ -186,18 +217,29 @@ def Connection(url):
             raise RedirectRequested(source=_url_escape_uri(url),
                                     target=_url_escape_uri(urlutils.join(url, new_url)), 
                                     is_permanent=True)
-        raise convert_error(subvertpy.SubversionException(msg, num))
-
+        raise convert_error(e)
     from bzrlib.plugins.svn import lazy_check_versions
     lazy_check_versions()
-
     return ret
 
 
 class ConnectionPool(object):
     """Collection of connections to a Subversion repository."""
-    def __init__(self):
+
+    def __init__(self, url):
         self.connections = set()
+        self.auth_baton = create_auth_baton(url)
+
+    def set_credentials(self, credentials):
+        if isinstance(credentials, dict):
+            user = credentials.get('user')
+            password = credentials.get('password')
+        else:
+            user, password = credentials[:2]
+        if user is not None:
+            self.auth_baton.set_parameter(AUTH_PARAM_DEFAULT_USERNAME, user)
+        if password is not None:
+            self.auth_baton.set_parameter(AUTH_PARAM_DEFAULT_PASSWORD, password)
 
     def get(self, url):
         # Check if there is an existing connection we can use
@@ -208,14 +250,14 @@ class ConnectionPool(object):
                 return c
         # Nothing available? Just pick an existing one and reparent:
         if len(self.connections) == 0:
-            return Connection(url)
+            return Connection(url, self.auth_baton)
         c = self.connections.pop()
         try:
             c.reparent(_url_escape_uri(url))
             return c
         except NotImplementedError:
             self.connections.add(c)
-            return Connection(url)
+            return Connection(url, self.auth_baton)
         except:
             self.connections.add(c)
             raise
@@ -230,24 +272,27 @@ class SvnRaTransport(Transport):
     
     This implements just as much of Transport as is necessary 
     to fool Bazaar. """
+
     @convert_svn_error
-    def __init__(self, url="", pool=None, _uuid=None, _repos_root=None):
+    def __init__(self, url, from_transport=None, credentials=None):
         bzr_url = url
         self.svn_url = bzr_to_svn_url(url)
         Transport.__init__(self, bzr_url)
-
-        if pool is None:
-            self.connections = ConnectionPool()
-
+        if from_transport is None:
+            self.connections = ConnectionPool(bzr_url)
+            if credentials is not None:
+                assert isinstance(credentials, dict)
+                self.connections.set_credentials(credentials)
             # Make sure that the URL is valid by connecting to it.
             self.connections.add(self.connections.get(self.svn_url))
         else:
-            self.connections = pool
-
-        self._repos_root = _repos_root
-        self._uuid = _uuid
+            self.connections = from_transport.connections
+            if credentials is not None:
+                assert isinstance(credentials, dict)
+                self.connections.set_credentials(credentials)
+        self._repos_root = None
+        self._uuid = None
         self.capabilities = {}
-
         from bzrlib.plugins.svn import lazy_check_versions
         lazy_check_versions()
 
@@ -263,9 +308,7 @@ class SvnRaTransport(Transport):
 
     def has(self, relpath):
         """See Transport.has()."""
-        # TODO: Raise TransportNotPossible here instead and 
-        # catch it in bzrdir.py
-        return False
+        return self.check_path(relpath, -1) != subvertpy.NODE_NONE
 
     def get(self, relpath):
         """See Transport.get()."""
@@ -275,7 +318,25 @@ class SvnRaTransport(Transport):
 
     def stat(self, relpath):
         """See Transport.stat()."""
-        raise TransportNotPossible('stat not supported on Subversion')
+        class StatResult(object):
+            def __init__(self, svn_stat):
+                if svn_stat['kind'] == subvertpy.NODE_DIR:
+                    self.st_mode = stat.S_IFDIR
+                elif svn_stat['kind'] == subvertpy.NODE_FILE:
+                    self.st_mode = stat.S_IFREG
+                self.st_size = svn_stat['size']
+                self.st_mtime = svn_stat['time']
+        conn = self.get_connection()
+        try:
+            stat_fn = getattr(conn, "stat", None)
+            if stat_fn is None:
+                raise TransportNotPossible("stat not supported in this version")
+            try:
+                return StatResult(stat_fn(relpath, -1))
+            except NotImplementedError:
+                raise TransportNotPossible("stat not supported in this version")
+        finally:
+            self.add_connection(conn)
 
     def put_file(self, name, file, mode=0):
         raise TransportNotPossible("put_file not supported on Subversion")
@@ -436,6 +497,12 @@ class SvnRaTransport(Transport):
         finally:
             self.add_connection(conn)
 
+    @convert_svn_error
+    def create_prefix(self):
+        create_branch_prefix(self.clone_root(),
+                {"svn:log": "Creating prefix"},
+                self.svn_url[len(self.get_repos_root()):].strip("/").split("/")[:-1])
+
     @convert_svn_error
     def mkdir(self, relpath, message="Creating directory"):
         conn = self.get_connection()
@@ -514,18 +581,8 @@ class SvnRaTransport(Transport):
     def lock_write(self, path_revs, comment=None, steal_lock=False):
         return self.PhonyLock() # FIXME
 
-    def _is_http_transport(self):
-        return False
-        return (self.svn_url.startswith("http://") or 
-                self.svn_url.startswith("https://"))
-
     def clone_root(self):
-        if self._is_http_transport():
-            return SvnRaTransport(self.get_repos_root(), 
-                                  bzr_to_svn_url(self.base),
-                                  pool=self.connections)
-        return SvnRaTransport(self.get_repos_root(),
-                              pool=self.connections)
+        return SvnRaTransport(self.get_repos_root(), self)
 
     def clone(self, offset=None):
         """See Transport.clone()."""
@@ -534,7 +591,7 @@ class SvnRaTransport(Transport):
         else:
             newurl = urlutils.join(self.base, offset)
 
-        return SvnRaTransport(newurl, pool=self.connections)
+        return SvnRaTransport(newurl, self)
 
     def local_abspath(self, relpath):
         """See Transport.local_abspath()."""
@@ -560,6 +617,13 @@ class MutteringRemoteAccess(object):
         mutter('svn check-path -r%d %s' % (revnum, path))
         return self.actual.check_path(path, revnum)
 
+    def stat(self, path, revnum):
+        mutter('svn stat -r%d %s' % (revnum, path))
+        stat = getattr(self.actual, "stat", None)
+        if stat is None:
+            raise NotImplementedError(self.stat)
+        return stat(path, revnum)
+
     def has_capability(self, cap):
         return self.actual.has_capability(cap)
 
@@ -642,3 +706,60 @@ class MutteringRemoteAccess(object):
         mutter("svn replay %d (low water mark: %d)" % (revision, low_water_mark))
         return self.actual.replay(revision, low_water_mark, editor, send_deltas)
 
+
+def create_branch_prefix(transport, revprops, bp_parts, existing_bp_parts=None):
+    """Create a branch prefixes (e.g. "branches")
+
+    :param transport: Subversion transport
+    :param revprops: Revision properties to set
+    :param bp_parts: Branch path elements that should be created (list of names, 
+        ["branches", "foo"] for "branches/foo")
+    :param existing_bp_parts: Branch path elements that already exist.
+    """
+    conn = transport.get_connection()
+    if existing_bp_parts is None:
+        existing_bp_parts = check_dirs_exist(conn, bp_parts, -1)
+    try:
+        ci = convert_svn_error(conn.get_commit_editor)(revprops)
+        try:
+            root = ci.open_root()
+            name = None
+            batons = [root]
+            for p in existing_bp_parts:
+                if name is None:
+                    name = p
+                else:
+                    name += "/" + p
+                batons.append(batons[-1].open_directory(name))
+            for p in bp_parts[len(existing_bp_parts):]:
+                if name is None:
+                    name = p
+                else:
+                    name += "/" + p
+                batons.append(batons[-1].add_directory(name))
+            for baton in reversed(batons):
+                baton.close()
+        except:
+            ci.abort()
+            raise
+        ci.close()
+    finally:
+        transport.add_connection(conn)
+
+
+def check_dirs_exist(transport, bp_parts, base_rev):
+    """Make sure that the specified directories exist.
+
+    :param transport: SvnRaTransport to use.
+    :param bp_parts: List of directory names in the format returned by 
+        os.path.split()
+    :param base_rev: Base revision to check.
+    :return: List of the directories that exists in base_rev.
+    """
+    for i in range(len(bp_parts), 0, -1):
+        current = bp_parts[:i]
+        path = "/".join(current).strip("/")
+        assert isinstance(path, str)
+        if transport.check_path(path, base_rev) == subvertpy.NODE_DIR:
+            return current
+    return []
